<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: b2e7896c3e3c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-12 18:25 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_cache_backendNone implements we_cache_backend{
	public const BACKEND_NONE = 'none';
	public const BACKEND_MEMCACHED = 'memcached';
	public const BACKEND_REDIS = 'redis';
	public const BACKEND_DB = 'database';

	public static function getBackend(string $backend, string $id): we_cache_backend{
		switch($backend){
			default:
			case self::BACKEND_NONE:
				return new self();
			case self::BACKEND_MEMCACHED:
				return ((defined('MEMCACHE_SERVERS') && MEMCACHE_SERVERS) && extension_loaded('memcached') ?
					new we_cache_backendMemcache($id) :
					new self());
			case self::BACKEND_REDIS:
				return ((defined('MEMCACHE_SERVERS') && MEMCACHE_SERVERS) && extension_loaded('redis') ?
					new we_cache_backendRedis($id) :
					new self());
			case self::BACKEND_DB:
				return new we_cache_backendDB();
		}
	}

	public function __construct(){

	}

	/**
	 *
	 * @param array $servers @unused-param
	 * @return void
	 */
	public function setServers(array $servers): void{

	}

	/**
	 *
	 * @param string $hash @unused-param
	 * @return bool
	 */
	public function clean(string $hash = ''): bool{
		return false;
	}

	/**
	 * @param string $entry @unused-param
	 * @return mixed
	 * @throws we_exception_cache
	 */
	public function get(string $entry){
		throw new we_exception_cache();
	}

	/**
	 *
	 * @param string $entry @unused-param
	 * @param int $default @unused-param
	 * @param int $expiry @unused-param
	 * @return int
	 */
	public function increment(string $entry, int $default = 1, int $expiry = 3600): int{
		return 0;
	}

	/**
	 *
	 * @param string $entry @unused-param
	 * @param mixed $data @unused-param
	 * @param int $expiry @unused-param
	 * @return bool
	 */
	public function set(string $entry, $data, int $expiry): bool{
		return false;
	}

	/**
	 *
	 * @param string $entry @unused-param
	 * @param int $expiry @unused-param
	 * @return bool
	 */
	public function touch(string $entry, int $expiry = 3600): bool{
		return false;
	}

	public function quit(): void{

	}

	public function getStatus(): string{
		return g_l('cache', '[cache_type_none]');
	}

}
