<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: af87aa135b22 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-03-25 09:29 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_chooser_multiDir extends we_chooser_base{
	/** @var int[] */
	protected $ids = [];

	/** @var string */
	protected $cmd_del = '';

	/** @var string */
	protected $addbut = '';

	/** @var string */
	protected $css = '';

	/** @var int */
	protected $nr = 0;

	/** @var int */
	protected $lines = 1;

	/** @var string */
	protected $thirdDelPar = '';

	/** @var array */
	protected $Record = [];

	/** @var string[] */
	protected $fields = ['ID', 'Path'];

	/**
	 *
	 * @param int[] $ids
	 * @param string $cmd_del
	 * @param string $addbut
	 * @param int[] $ws
	 * @param string $ct
	 * @param string $table
	 * @param string $css
	 * @param string $thirdDelPar
	 * @param string $extraDelFn
	 */
	public function __construct(array $ids, string $cmd_del, string $addbut, array $ws = [], string $ct = 'ContentType', string $table = FILE_TABLE, string $css = '', string $thirdDelPar = '', string $extraDelFn = ''){
		$this->ids = $ids;
		$this->table = $table;
		$this->cmd_del = $cmd_del;
		$this->addbut = $addbut;
		$this->css = $css;
		$this->extraDelFn = $extraDelFn;
		$this->thirdDelPar = $thirdDelPar;
		switch($table){
			case CUSTOMER_TABLE :
				$this->fields = ['ID', 'Username'];
		}
		$this->fields[] = $ct;
		$this->isEditable = true;
	}

	function getLine(int $lineNr): string{
		switch($lineNr){
			case 0:
				return '<tr>
	<td class="chooserFileIcon" data-contenttype="' . $this->Record['ContentType'] . '"></td>
	<td class="' . $this->css . '">' . $this->Record['Path'] . '</td>
	<td class="buttons">' . ((($this->isEditable && $this->cmd_del) || $this->CanDelete) ?
					we_html_button::create_button(we_html_button::TRASH, "javascript:" . $this->getJsSetHot() . ($this->extraDelFn ?: '') . ";we_cmd('" . $this->cmd_del . "','" . $this->Record['ID'] . "'" . (strlen($this->thirdDelPar) ? ",'" . $this->thirdDelPar . "'" : '') . ");") :
					'') . '</td>
</tr>';
			default:
				return '';
		}
	}

	function getRootLine(int $lineNr): string{
		switch($lineNr){
			case 0:
				return '<tr>
	<td class="chooserFileIcon" data-contenttype="' . we_base_ContentTypes::FOLDER_DOCUMENT . '"></td>
	<td class="' . $this->css . '">/</td>
	<td class="buttons">' . ((($this->isEditable && $this->cmd_del) || $this->CanDelete) ?
					we_html_button::create_button(we_html_button::TRASH, "javascript:" . $this->getJsSetHot() . ($this->extraDelFn ?: "") . ";we_cmd('" . $this->cmd_del . "','0');") :
					'') . '</td>
</tr>';
			default:
				return '';
		}
	}

	function get(we_base_jsCmd $jsCmd): string{
		$out = '<table class="default" style="width: 100%;">';

		$this->nr = 0;

		foreach($this->ids as $id){
			$this->Record = getHash('SELECT ' . $this->fields[0] . ' AS ID,' . $this->fields[1] . ' AS Path,' . $this->fields[2] . ' AS ContentType FROM ' . DB_WE::$inst->escape($this->table) . ' WHERE ID=' . ((int) $id));
			if($this->Record){
				for($i = 0; $i < $this->lines; $i++){
					$out .= $this->getLine($i);
				}
			} else if(!$id){
				for($i = 0; $i < $this->lines; $i++){
					$out .= $this->getRootLine($i);
				}
			}
			$this->nr++;
		}
		$out .= '</table>';

		$jsCmd->addCmd('setIconOfDocClass', 'chooserFileIcon');

		return we_html_element::div(['class' => 'multichooserDiv'],
															we_html_element::div(['class' => 'multichooser'], $out) .
				($this->addbut ? we_html_element::div(['class' => 'button'], $this->addbut) : '')
		);
	}

}
