<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
// Class Declaration Starts:
class we_customer_generatePassword{
	/** @var int */
	private $PasswordLength; // Variable To Assign Password Length

	private const CAPS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
	private const SMALL = 'abcdefghjkmnpqrstuvwxyz';
	private const NUMS = '0123456789';
	private const SPECS = '+-*$#@!%&/()=?\\<>_'; // This Can be removed if not Needed

	/** @var array<string,mixed> */
	private $condition;

	/** @var int */
	private $minLen;

	/** @var string */
	private $all;

	/** @var string */
	private $set = '';

	public function __construct(int $passLen, array $condition){ //Constructor To Assign Values
		$this->condition = $condition; // Will Store the Condition Array to Global Variable
		// Will calculate the Minimum Length
		$this->minLen = $this->condition['caps'] + $this->condition['small'] + $this->condition['nums'] + $this->condition['specs'];
		// Compute the Total Password Length and Store it to the Global Variable
		$this->PasswordLength = max($this->minLen, $passLen);
		$this->all = self::CAPS . self::SMALL . self::NUMS . self::SPECS;
	}

	public function PassGen(): string{ // Function to Generate Random Passowrd
		$i = 1;
		$password = [];
		while($i <= $this->PasswordLength){
			if($i <= $this->minLen){
				if($i <= $this->condition['specs']){
					$this->set = self::SPECS;
				} elseif($i <= ($this->condition['specs'] + $this->condition['small'])){
					$this->set = self::SMALL;
				} elseif($i <= ($this->condition['specs'] + $this->condition['small'] + $this->condition['nums'])){
					$this->set = self::NUMS;
				} elseif($i <= ($this->condition['specs'] + $this->condition['small'] + $this->condition['nums'] + $this->condition['caps'])){
					$this->set = self::CAPS;
				} elseif($i <= ($this->condition['specs'] + $this->condition['small'] + $this->condition['nums'] + $this->condition['specs'])){
					$this->set = self::SPECS;
				}
			} elseif($i <= $this->PasswordLength){
				$this->set = $this->all;
			}
			$tmp = self::getPwdChar($this->set);
			$password[] = $tmp;
			$i++;
		}

		shuffle($password);
		return implode('', $password);
	}

	public function PassCheck(string $pass): bool{ // Function To Check Whether the Password have those Conditions
		$cond = ['caps' => 0, 'small' => 0, 'nums' => 0, 'specs' => 0];
		for($i = 0; $i < strlen($pass); $i++){
			$c = $pass[$i];
			if(str_contains($c, self::CAPS)){
				$cond['caps']++;
			}
			if(str_contains($c, self::SMALL)){
				$cond['small']++;
			}
			if(str_contains($c, self::NUMS)){
				$cond['nums']++;
			}
			if(str_contains($c, self::SPECS)){
				$cond['specs']++;
			}
		}
		return ($this->condition['caps'] <= $cond['caps'] && $this->condition['small'] <= $cond['small'] && $this->condition['nums'] <= $cond['nums'] && $this->condition['specs'] <= $cond['specs']);
	}

	private static function getPwdChar(string $set): string{
		//mt_getrandmax(); // Returns the maximum value that can be returned by a call  rand
		$num = rand() % strlen($set);
		return $set[$num];
	}

}
