<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 5b5c824e90ed $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-06-01 13:20 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredClassMethod,PhanUndeclaredMethod,PhanUndeclaredExtendedClass
use Sabre\DAV;

class we_davfs_phpDir extends DAV\FSExt\Directory{

	/**
	 * Returns a specific child node, referenced by its name.
	 *
	 * This method must throw Sabre\DAV\Exception\NotFound if the node does not
	 * exist.
	 *
	 * @param string $name
	 *
	 * @throws DAV\Exception\NotFound
	 *
	 * @return DAV\INode
	 */
	public function getChild($name){
		$path = $this->path . '/' . $name;

		if(!file_exists($path)){
			throw new DAV\Exception\NotFound('File could not be located');
		}
		if(in_array($name, ['.', '..'])){
			throw new DAV\Exception\Forbidden('Permission denied to . and ..');
		}
		return (is_dir($path) ?
			new self($path) :
			new we_davfs_phpFile($path));
	}

	public function createFile($name, $data = null){
		//FIXME: hook?
		if(!str_ends_with($name, '.php')){
			throw new DAV\Exception\Forbidden('Permission denied to create files other than .php');
		}
		return parent::createFile($name, $data);
	}

	public function delete(){
		$hook = new we_hook_handler('delete', [$this]);
		$hook->executeHook();
		return parent::delete();
	}

}
