<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: ac7c44c87b87 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-05-19 13:23 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_dialog_browseServer{

	private static function printHeaderHTML(bool $ret): string{
		return '
<table class="selectorHeaderTable">
	<tr style="vertical-align:middle">
		<td class="defaultfont lookinText">' . g_l('fileselector', '[lookin]') . '</td>
		<td class="lookin"><select name="lookin" id="lookin" onchange="WE().util.dialogTop(window).fscmd.setDir(lookin.options[lookin.selectedIndex].value);" class="defaultfont" style="width:100%">
				<option value="/">/</option>
			</select></td>
		<td>' . we_html_button::create_button(we_html_button::ROOT_DIR, "javascript:WE().util.dialogTop(window).fscmd.setDir('/');") . '</td>
		<td>' . we_html_button::create_button(we_html_button::LEVEL_UP, "javascript:WE().util.dialogTop(window).fscmd.goUp();") . '</td>
' . ($ret && !we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ? '' : '
			<td>' . we_html_button::create_button(we_html_button::NEW_DIR, "javascript:WE().util.dialogTop(window).fscmd.drawNewFolder();", '', "", false, false, "_ss") . '</td>
			<td>' . we_html_button::create_button(we_html_button::ADD_FILE, "javascript:javascript:openFile();", '', "", false, false, "_ss") . '</td>
			<td class="trash">' . we_html_button::create_button(we_html_button::TRASH, "javascript:WE().util.dialogTop(window).fscmd.delFile();", '', "", false, false, "_ss") . '</td>') .
			'</tr>
</table>';
	}

	private static function printFooterTable(bool $ret, string $filter, string $currentName): string{
		if($ret){
			$cancel_button = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");
			$yes_button = we_html_button::create_button(we_html_button::OK, "javascript:WE().util.dialogTop(window).exit_close();");
		} else {
			$cancel_button = we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).exit_close();");
			$yes_button = we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:editFile();");
		}
		$options = '';
		if($filter === "all_Types"){
			$options = '<option value="' . str_replace(' ', '%20', g_l('contentTypes', '[all_Types]')) . '">' . g_l('contentTypes', '[all_Types]') . '</option>';
			foreach(we_base_ContentTypes::getFiles() as $key){
				$options .= '<option value="' . rawurlencode(g_l('contentTypes', '[' . $key . ']')) . '">' . g_l('contentTypes', '[' . $key . ']') . '</option>';
			}
		}
		return '
<table id="footer" class=" defaultfont">' .
			($filter === "all_Types" ? '
	<tr>
		<td class="description">' . g_l('fileselector', '[type]') . '</td>
		<td>
			<select name="filter" class="weSelect" onchange="WE().util.dialogTop(window).fscmd.setFilter(this.options[this.selectedIndex].value)" style="width:100%">
				' . $options . '</select></td>
	</tr>' : '') . '
	<tr>
		<td class="description">' . g_l('fileselector', '[name]') . '</td>
		<td style="text-align:left">' . we_html_tools::textInput('fname', $currentName, 0, ['style' => "width:100%", 'readonly' => "readonly"]) . '</td>
	</tr>
</table>
<div id="footerButtons">' . we_html_button::position_yes_no_cancel($yes_button, '', $cancel_button) . '</div>
';
	}

	private static function delDir(we_base_jsCmd $jsCmd, string $dir): void{
		$d = dir($dir);
		while(false !== ($entry = $d->read())){
			if($entry != '.' && $entry != '..'){
				if(is_dir($dir . '/' . $entry)){
					self::delDir($jsCmd, $dir . '/' . $entry);
				} else if(is_file($dir . '/' . $entry)){
					if(!we_base_file::deleteLocalFile($dir . '/' . $entry)){
						$jsCmd->addMsg(sprintf(g_l('alert', '[delete][notok][file]'), $entry), we_base_util::WE_MESSAGE_ERROR);
					}
				} else {
					$jsCmd->addMsg(sprintf(g_l('alert', '[delete][notok][noexist]'), $entry), we_base_util::WE_MESSAGE_ERROR);
				}
			}
		}
		if(!we_base_file::deleteLocalFolder($dir)){
			$jsCmd->addMsg(sprintf(g_l('alert', '[delete][notok][folder]'), $dir), we_base_util::WE_MESSAGE_ERROR);
		}
	}

	private static function getDataType(string $dat): string{
		$ct = we_base_ContentTypes::getFromFilename($dat);
		try{
			return g_l('contentTypes', '[' . $ct . ']', true);
		} catch (Exception $e){
			return '';
		}
	}

	private static function readFiles(string $dir, we_base_jsCmd $jsCmd): array{
		$arDir = $arFile = $ordDir = $ordFile = $final = [];
		@chdir($dir);
		$dir_obj = @dir($dir);

		$ord = we_base_request::int("ord", 10);

		if(!$dir_obj){
			$jsCmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
			throw new we_exception_base('no access');
		}

		while(($entry = $dir_obj->read()) !== false){
			if($entry == '.' || $entry == '..'){
				continue;
			}
			if(is_link($dir . '/' . $entry) || is_dir($dir . '/' . $entry)){
				$arDir[] = $entry;
				switch($ord){
					case 10:
					case 11:
						$ordDir[] = $entry;
						break;
					case 20:
					case 21:
						$ordDir[] = self::getDataType($dir . '/' . $entry);
						break;
					case 30:
					case 31:
						$ordDir[] = filectime($dir . '/' . $entry);
						break;
					case 40:
					case 41:
						$ordDir[] = filesize($dir . '/' . $entry);
						break;
				}
			} else {
				$arFile[] = $entry;
				switch($ord){
					case 10:
					case 11:
						$ordFile[] = $entry;
						break;
					case 20:
					case 21:
						$ordFile[] = self::getDataType($dir . '/' . $entry);
						break;
					case 30:
					case 31:
						$ordFile[] = filectime($dir . '/' . $entry);
						break;
					case 40:
					case 41:
						$ordFile[] = filesize($dir . '/' . $entry);
						break;
				}
			}
		}
		$dir_obj->close();


		switch($ord){
			case 10:
			case 20:
			case 30:
			case 40:
				asort($ordDir, SORT_NATURAL | SORT_FLAG_CASE);
				asort($ordFile, SORT_NATURAL | SORT_FLAG_CASE);
				break;
			case 11:
			case 21:
			case 31:
			case 41:
				arsort($ordDir, SORT_NATURAL | SORT_FLAG_CASE);
				arsort($ordFile, SORT_NATURAL | SORT_FLAG_CASE);
				break;
		}

		foreach(array_keys($ordDir) as $key){
			$final[] = $arDir[$key];
		}
		foreach(array_keys($ordFile) as $key){
			$final[] = $arFile[$key];
		}
		return $final;
	}

	public static function getDialog(): void{
		$docroot = WE_DOCUMENT_ROOT;
		$cmd1 = we_base_request::_(we_base_request::STRING, 'we_cmd', '', 1);
		$cmd4 = we_base_request::_(we_base_request::STRING, 'we_cmd', '', 4);

		$filter = we_base_request::string('we_cmd', '', 2) ?: 'all_Types';
		$url = we_base_request::_(we_base_request::URL, 'we_cmd', '', 3);

		$parsedUrl = parse_url($url);
		$url = (is_array($parsedUrl) && !empty($parsedUrl['host'])) && $parsedUrl['host'] === $_SERVER['SERVER_NAME'] ? $parsedUrl['path'] : $url;

		$currentDir = str_replace('\\', '/', ( $url ?
			($url === '/' ? '' :
			( parse_url($url) === false && is_dir($docroot . $url) ?
			$url :
			dirname($url))) :
			''));
		$currentName = basename($url);
		if(!file_exists($docroot . $currentDir . '/' . $currentName)){
			$currentDir = '';
			$currentName = '';
		}
		$currentID = $docroot . $currentDir . ($filter === we_base_ContentTypes::FOLDER_DOCUMENT ? '' : (($currentDir != '') ? '/' : '') . $currentName);

		$currentID = str_replace('\\', '/', $currentID);

		$rootDir = we_base_request::_(we_base_request::FILE, 'we_cmd', '', 5);
		$selectInternal = we_base_request::bool('we_cmd', false, 6);
		try{
			$filterName = g_l('contentTypes', '[' . $filter . ']', true);
		} catch (Exception $ex){
			$filterName = '';
		}


		echo we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'selectors.css') .
			we_html_element::cssElement('
#fsfooter{
	 bottom:0;
}
') .
			we_html_element::jsScript(JS_DIR . 'selectors/we_sselector_header.js', '', ['id' => 'loadVarSelectors', 'data-selector' => setDynamicVar([
					'options' => [
					],
					'data' => [
						'currentType' => ($filter == we_base_ContentTypes::FOLDER_DOCUMENT) ? we_base_ContentTypes::FOLDER_DOCUMENT : '',
						'currentDir' => str_replace($rootDir, '', $currentDir),
						'currentID' => $currentID,
						'rootDir' => $rootDir,
						'currentName' => $currentName,
						'currentFilter' => str_replace(' ', '%20', $filterName),
						'browseServer' => $cmd1 ? false : true,
						'cmd1' => $cmd1,
						'cmd4' => $cmd4,
						'sitepath' => $docroot,
						'filter' => $filter
					],
					'click' => [
						'oldID ' => 0,
					]
			])]), we_html_element::body([
				'onload' => "init()",
				'onunload' => "doUnload();"
				], we_html_element::div(['id' => 'fsheader'], self::printHeaderHTML(!empty($cmd1))) .
				we_html_element::iframe('fsbody', 'about:blank') .
				we_html_element::div(['id' => 'fsfooter'], self::printFooterTable(!empty($cmd1), $filter, $currentName)) .
				we_html_element::iframe('fscmd', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=selectorBrowseCmd&ret=' . ($cmd1 ? 1 : 0) . '&filter=' . $filter . '&currentName=' . $currentName . '&selectInternal=' . ((int) $selectInternal), 'display:none;'))
		);
	}

	public static function getCommand(): void{
		$jsCmd = new we_base_jsCmd();
		$js = '';

		$selectInternal = we_base_request::bool('selectInternal', false);

		$txt = we_base_request::_(we_base_request::FILE, 'txt', '');
		switch(we_base_request::string("cmd")){
			case 'new_folder':
				$js .= 'drawDir(WE().util.dialogTop(window).fileSelect.data.currentDir);';
				if($txt === ''){
					$jsCmd->addMsg(g_l('alert', '[we_filename_empty]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				if(preg_match('|[\'"<>/]|', $txt)){
					$jsCmd->addMsg(g_l('alert', '[name_nok]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$path = str_replace('//', '/', WE_DOCUMENT_ROOT . we_base_request::_(we_base_request::FILE, 'pat') . '/' . $txt);
				if(!is_dir($path)){
					if(!we_base_file::createLocalFolderByPath($path)){
						$jsCmd->addMsg(g_l('alert', '[create_folder_nok]'), we_base_util::WE_MESSAGE_ERROR);
					} else {
						$js .= 'selectFile("' . $txt . '");WE().util.dialogTop(window).fileSelect.data.currentID="' . $path . '";';
					}
				} else {
					$jsCmd->addMsg(sprintf(g_l('alert', '[path_exists]'), str_replace(WE_DOCUMENT_ROOT, '', $path)), we_base_util::WE_MESSAGE_ERROR);
				}
				break;
			case 'rename_folder':
				if($txt === ''){
					$jsCmd->addMsg(g_l('alert', '[we_filename_empty]'), we_base_util::WE_MESSAGE_ERROR);
					$js .= "drawDir(WE().util.dialogTop(window).fileSelect.data.currentDir);";
					break;
				}
				if(preg_match('|[\'"<>/]|', $txt)){
					$jsCmd->addMsg(g_l('alert', '[name_nok]'), we_base_util::WE_MESSAGE_ERROR);
					$js .= "drawDir(WE().util.dialogTop(window).fileSelect.data.currentDir);";
					break;
				}
				$pat = we_base_request::_(we_base_request::FILE, 'pat');
				$old = str_replace('//', '/', WE_DOCUMENT_ROOT . $pat . '/' . we_base_request::_(we_base_request::FILE, 'sid'));
				$new = str_replace('//', '/', WE_DOCUMENT_ROOT . $pat . '/' . $txt);
				if($old != $new){
					if(!is_dir($new)){
						if(!rename($old, $new)){
							$jsCmd->addMsg(g_l('alert', '[rename_folder_nok]'), we_base_util::WE_MESSAGE_ERROR);
						} else {
							$js .= 'selectFile("' . $txt . '");';
						}
					} else {
						$we_responseText = sprintf(g_l('alert', '[path_exists]'), str_replace(WE_DOCUMENT_ROOT, '', $new));
						$jsCmd->addMsg($we_responseText, we_base_util::WE_MESSAGE_ERROR);
					}
				}
				$js .= 'drawDir(WE().util.dialogTop(window).fileSelect.data.currentDir);';

				break;
			case 'rename_file':
				if($txt === ''){
					$jsCmd->addMsg(g_l('alert', '[we_filename_empty]'), we_base_util::WE_MESSAGE_ERROR);
					$js .= "drawDir(WE().util.dialogTop(window).fileSelect.data.currentDir);";
					break;
				}
				if(preg_match('|[\'"<>/]|', $txt)){
					$jsCmd->addMsg(g_l('alert', '[name_nok]'), we_base_util::WE_MESSAGE_ERROR);
					$js .= "drawDir(WE().util.dialogTop(window).fileSelect.data.currentDir);";
					break;
				}
				$pat = we_base_request::_(we_base_request::FILE, 'pat');
				$txt = we_base_request::_(we_base_request::FILE, 'txt');
				$old = str_replace('//', '/', WE_DOCUMENT_ROOT . $pat . '/' . we_base_request::_(we_base_request::FILE, "sid"));
				$new = str_replace('//', '/', WE_DOCUMENT_ROOT . $pat . '/' . $txt);
				if($old != $new){
					if(!file_exists($new)){
						if(!rename($old, $new)){
							$jsCmd->addMsg(g_l('alert', '[rename_file_nok]'), we_base_util::WE_MESSAGE_ERROR);
						} else {
							$js .= 'selectFile("' . $txt . '");';
						}
					} else {
						$jsCmd->addMsg(sprintf(g_l('alert', '[path_exists]'), str_replace(WE_DOCUMENT_ROOT, '', $new)), we_base_util::WE_MESSAGE_ERROR);
					}
				}
				$js .= "drawDir(WE().util.dialogTop(window).fileSelect.data.currentDir);selectFile(WE().util.dialogTop(window).fileSelect.data.currentName);";
				break;
			case 'delete_file':
				if(!($fid = we_base_request::_(we_base_request::FILE, 'fid'))){
					break;
				}
				$foo = fI('SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($fid) . '"');
				if(preg_match('|' . WEBEDITION_PATH . '|', $fid) || ($fid == rtrim(WEBEDITION_PATH, '/')) || str_contains($fid, '..') || $foo || $fid == WE_DOCUMENT_ROOT || $fid . '/' == WE_DOCUMENT_ROOT){
					$jsCmd->addMsg(g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				if(we_base_request::bool("ask")){
					if(!is_link($fid) && is_dir($fid)){
						$js .= 'if(window.confirm("' . strtr(g_l('alert', '[delete][folder]'), ["\n" => '\n']) . '")){delFile(0);}';
					} else if(is_link($fid) || is_file($fid)){
						$js .= 'if(window.confirm("' . strtr(g_l('alert', '[delete][selected]'), ["\n" => '\n']) . '")){delFile(0);}';
					}
				} else {
					if(!is_link($fid) && is_dir($fid)){
						self::delDir($jsCmd, $fid);
					} else if(!we_base_file::deleteLocalFile($fid)){
						$jsCmd->addMsg(sprintf(g_l('alert', '[delete][notok][error]'), $fid), we_base_util::WE_MESSAGE_ERROR);
					}
				}
				$js .= "selectFile('');drawDir(WE().util.dialogTop(window).fileSelect.data.currentDir);";
		}

		echo we_html_tools::getHtmlTop('',
																 we_html_element::jsScript(JS_DIR . 'selectors/sselector_cmd.js', "WE().util.dialogTop(window).fileSelect.data.filter='" . we_base_request::string('filter') . "';selectInternal=" . intval($selectInternal) . ";") .
			we_html_element::jsElement($js) .
			$jsCmd->getCmds(), we_html_element::body()
		);
	}

	public static function getBody(): void{
		we_base_WE::setDebugging();

		$nf = we_base_request::_(we_base_request::RAW, 'nf');
		$sid = we_base_request::_(we_base_request::RAW, 'sid');
		$selectInternal = we_base_request::bool('selectInternal', false);
		$org = we_base_request::_(we_base_request::FILE, 'dir', '/');
		$contentFilter = we_base_request::string('file'); //FIXME: totaler nonsense!!
		$curID = we_base_request::_(we_base_request::FILE, 'curID');
		$jsCmd = new we_base_jsCmd();

		$fileContentTypes = DB_WE::$inst->getAllFirstq('SELECT Text,ContentType FROM ' . FILE_TABLE . ' WHERE ParentID=' . ($org == '/' ? '0' : '(SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . DB_WE::$inst->escape($org) . '")'), false);
		$set_rename = false;
		$thumbFold = trim(WE_THUMBNAIL_DIRECTORY, '/');
		$dir = rtrim(WE_DOCUMENT_ROOT . $org, '/');
		try{
			$files = self::readFiles($dir, $jsCmd);

			$body = ($nf === 'new_folder' ? '
				<tr class="selected">
					<td class="selector selectoricon" data-contenttype="' . we_base_ContentTypes::FOLDER_DOCUMENT . '" data-extension=""></td>
					<td class="selector filename">' . we_html_tools::textInput("txt", g_l('fileselector', '[new_folder_name]'), 0, ['id' => "txt", 'onblur' => "setScrollTo();we_form.submit();",
					'onkeydown' => "keypressed(event)"], 'text', '100%') . '</td>
					<td class="selector filetype">' . g_l('fileselector', '[folder]') . '</td>
					<td class="selector moddate">' . date(g_l('weEditorInfo', '[date_format_sec]')) . '</td>
					<td class="selector filesize"></td>
				</tr>' :
				'');


			foreach($files as $e){
				$entry = (string) $e;
				$name = str_replace('//', '/', $org . '/' . $entry);
				$islink = is_link($dir . '/' . $entry);
				$isfolder = is_dir($dir . '/' . $entry) && !$islink;

				$type = $isfolder ? g_l('contentTypes', '[folder]') : self::getDataType($dir . '/' . $entry);
				$ext = strrchr($name, '.');

				switch($entry){
					case 'webEdition':
					case WE_THUMBNAIL_DIRECTORY:
					case $thumbFold:
						$indb = 'folder';
						break;
					default:
						if((preg_match('|^' . WE_DOCUMENT_ROOT . '/?webEdition/|', $dir) ||
							preg_match('|^' . WE_DOCUMENT_ROOT . '/?webEdition$|', $dir)) &&
							(!preg_match('|^' . WE_DOCUMENT_ROOT . '/?webEdition/generated/we_backup|', $dir) ||
							$entry === 'download')){
							$indb = 'folder';
						} else {
							$indb = $fileContentTypes[$entry] ?? false;
						}
				}
				$indb = we_base_WE::isDebug() ? false : $indb;
				switch($entry){
					case '.':
					case '..':
						$show = false;
						continue 2;
					default:
						switch($contentFilter){
							case '':
							case g_l('contentTypes', '[all_Types]'):
							case $type:
								$show = true;
								break;
							default:
								$show = ($type == g_l('contentTypes', '[folder]'));
						}
				}
				$bgcol = ($curID == ($dir . '/' . $entry) && !( $nf === 'new_folder')) ? 'selected' : '';
				$onclick = $ondblclick = '';
				$cursor = 'cursor:default;';
				if(!(( $nf === 'rename_folder' || $nf === 'rename_file') && ($entry == $sid) && ($isfolder))){
					if($indb){
						if($isfolder){
							$onclick = "dbFolderClick('" . $entry . "');";
							$ondblclick = "dbFolderDblClick('" . $entry . "');";
							$cursor = 'cursor:pointer;';
						} elseif($selectInternal){
							$onclick = "internalClick('" . $entry . "');";
							$cursor = "cursor:pointer;";
						}
					} else {
						if($isfolder){
							$onclick = "folderClick('" . $entry . "')";
							$ondblclick = "folderDblClick('" . $entry . "',1,0);";
						} else {
							$onclick = "fileClick('" . $entry . "');";
						}
						$cursor = "cursor:pointer;";
					}
				}
				$text_to_show = $date = '';
				switch((($entry == $sid) && (!$indb) ? $nf : '')){
					case 'rename_folder':
						if($isfolder){
							$text_to_show = we_html_tools::textInput('txt', $entry, 0, ['onblur' => "setScrollTo();we_form.submit();", 'onkeydown' => "keypressed(event)"], "text", "100%");
							$set_rename = true;
							$type = g_l('contentTypes', '[folder]');
							$date = date(g_l('weEditorInfo', '[date_format_sec]'));
						}
						break;
					case 'rename_file':
						$text_to_show = we_html_tools::textInput('txt', $entry, 0, ['onblur' => "setScrollTo();we_form.submit();", 'onkeydown' => "keypressed(event)"], "text", "100%");
						$set_rename = true;
						$type = '<div class="cutText" title="' . htmlspecialchars($type) . '">' . htmlspecialchars($type) . '</div>';
						$date = date(g_l('weEditorInfo', '[date_format_sec]'));
						break;
					default:
						$text_to_show = '<div class="cutText" title="' . htmlspecialchars($entry) . '">' . htmlspecialchars($entry) . '</div>';
						$type = '<div class="cutText" title="' . htmlspecialchars($type) . '">' . htmlspecialchars($type) . '</div>';
						$date = (file_exists($dir . '/' . $entry) ? date(g_l('weEditorInfo', '[date_format_sec]'), filemtime($dir . '/' . $entry) ?: 0) : 'n/a');
				}

				if($show){
					$filesize = (!$isfolder && !$islink && file_exists($dir . '/' . $entry) ? filesize($dir . '/' . $entry) ?: 0 : 0);

					$size = ($isfolder ?
						'' :
						($islink ?
						'-> ' . readlink($dir . '/' . $entry) :
						'<span' . ($indb ? ' style="color:#006699"' : '') . ' title="' . $filesize . '">' . we_base_file::getHumanFileSize($filesize) . '</span>'));

					$body .= '<tr ' . ($indb ? 'class="WEFile"' : '') . ' id="' . htmlspecialchars($entry) . '"' . ($ondblclick ? ' onDblClick="' . $ondblclick . '"' : '') . ($onclick ? ' onclick="' . $onclick . '"' : '') . ' class="' . $bgcol . '" style="height:2ex;' . $cursor . '"' . '>
	<td class="selector selectoricon" data-contenttype="' . ($indb ?: ($islink ? 'symlink' : ($isfolder ? we_base_ContentTypes::FOLDER_DOCUMENT : we_base_ContentTypes::APPLICATION))) . '" data-extension="' . $ext . '"></td>
	<td class="selector filename">' . $text_to_show . '</td>
	<td class="selector filetype">' . $type . '</td>
	<td class="selector moddate">' . $date . '</td>
	<td class="selector filesize">' . $size . '</td>
 </tr>';
				}
			}
			$isRename = false;
			switch($nf){
				case 'rename_folder':
				case 'rename_file':
					if(!$set_rename){
						break;
					}
					$isRename = true;
				//no break
				case 'new_folder':
					$body .= we_html_element::hiddens(['cmd' => $nf,
							'pat' => we_base_request::_(we_base_request::RAW, "pat", ""),
							($isRename ? 'sid' : '') => $sid,
							($isRename ? 'oldtxt' : '') => ''
					]);
			}
			$onload = '';
			switch($nf){
				case 'rename_folder':
				case 'rename_file':
					if(!$set_rename){
						break;
					}
				//no break
				case 'new_folder':
					$onload = "initSelector('" . $nf . "');";
			}
		} catch (we_exception_base $e){
			$body = '<tr><td><div class="middlefont" style="padding-top:2em;text-align:center">-- ' . g_l('alert', '[access_denied]') . ' --</div></td></tr>';
		}

		echo we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'selectors.css') .
			we_html_element::jsScript(JS_DIR . 'selectors/we_sselector_body.js', '', ['id' => 'loadVarSelectors', 'data-selector' => setDynamicVar([
					'allentries' => $files
			])]) . $jsCmd->getCmds(), we_html_element::body(['onload' => $onload . "WE().util.setIconOfDocClass(document,'selectoricon');doScrollTo();$('table.sort').tablesorter();"], '
	<form name="' . we_base_constants::WE_FORM . '" target="fscmd" action="we_cmd.php?we_cmd[0]=selectorBrowseCmd" method="post" onsubmit="return false;">
	<table class="selector sort" data-skipSort="0"><thead><tr>
		<th class="selector selectoricon"></th>
		<th class="selector filename">' . we_html_element::a(['href' => '#', 'onclick' => "reorder('name');"], g_l('fileselector', '[filename]')) . '</th>
		<th class="selector filetype">' . we_html_element::a(['href' => '#', 'onclick' => "reorder('type');"], g_l('fileselector', '[type]')) . '</th>
		<th class="selector moddate">' . we_html_element::a(['href' => '#', 'onclick' => "reorder('date');"], g_l('fileselector', '[modified]')) . '</th>
		<th class="selector filesize">' . we_html_element::a(['href' => '#', 'onclick' => "reorder('size');"], g_l('fileselector', '[filesize]')) . '</th>
		<th class="selector remain"></th>
	</tr></thead><tbody>' .
				$body .
				'</tbody></table></form>'));
	}

}
