<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0776cbaeceaa $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2021-04-08 13:42 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
we_html_tools::protect([we_base_perm::WORK_COLLECTION_NEW]); // FIXME: use perms

class we_dialog_newCollection extends we_dialog_base{
	/** @var string */
	private $jsCommandOnSuccess = '';

	/** @var int */
	private $id = 0;

	/** @var int */
	private $fixedPID = -1;

	/** @var string */
	private $fixedRemTable = '';

	/** @var we_contents_collection */
	private $we_doc = null;

	/** @var bool */
	private $doSave = false;

	/** @var string */
	private $transaction = '';

	public function __construct(bool $noInternals = false){
		parent::__construct(false);
		//$this->title = g_l('button', '[new_collection][value]');
		$this->initByHttp();

		$this->we_doc = new we_contents_collection();
		if($this->transaction){
			$this->we_doc = we_contents_root::getSessionDocument($this->transaction);
			assert($this->we_doc instanceof we_contents_collection);
			$this->we_doc->InsertRecursive = true;

			if($this->doSave){
				if(($jsMessage = $this->we_doc->checkFieldsOnSave())){
					$this->jsCmd->addMsg($jsMessage, we_base_util::WE_MESSAGE_ERROR);
				} elseif($this->we_doc->we_save()){
					$this->jsCmd->addMsg(sprintf(g_l('weEditor', '[we/collection][response_save_ok]'), $this->we_doc->getText()), we_base_util::WE_MESSAGE_NOTICE);
					$this->jsCmd->addCmd('we_cmd', ['do_onSuccess']);
				} else {
					$this->jsCmd->addMsg('unknown error ', we_base_util::WE_MESSAGE_ERROR);
				}
			} else {
				$this->we_doc->saveInSession($this->transaction);
			}
		} else {
			$this->transaction = we_base_WE::$transaction;
			$this->we_doc->we_new();
			$this->we_doc->remTable = $this->fixedRemTable ?: '';
			$this->we_doc->saveInSession(we_base_WE::$transaction);
		}
	}

	protected function getDialogPartsHTML(): we_html_multiIconBoxContent{
		assert($this->we_doc instanceof we_contents_collection);

		return (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[path]'), $this->we_doc->formPath($this->fixedPID !== -1, true)))->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('global', '[content]'), $this->we_doc->formContent()))->noline());
	}

	protected function getHiddenArgs(): string{
		return we_html_element::hiddens([
				'we_transaction' => (string) $this->transaction,
				'dosave' => '0',
				'fixedpid' => (string) $this->fixedPID,
				'fixedremtable' => $this->fixedRemTable,
		]);
	}

	protected function initByHttp(): void{
		$this->transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', '');
		$this->cmd = we_base_request::string('we_cmd', '', 0);
		$this->doSave = we_base_request::bool('dosave');
		$this->jsCommandOnSuccess = we_base_request::string('we_cmd', '', 1);
		$this->fixedPID = we_base_request::int('fixedpid', -1);
		$this->fixedRemTable = we_base_request::string('fixedremtable');
	}

	protected function getJs(): string{
		return parent::getJs() .
			we_html_element::jsScript(JS_DIR . '/dialogs/we_dialog_newCollection.js', '', ['id' => 'loadVarWe_dialog_newCollection', 'data-dialog' => setDynamicVar([
					'name' => $this->we_doc->Name,
					'cmd' => $this->cmd,
					'cmdOnSuccess' => $this->jsCommandOnSuccess,
					'data' => ['id' => $this->we_doc->ID, 'text' => $this->we_doc->Path],
		])]);
	}

	protected function getDialogButtons(): string{
		return we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, "javascript:we_cmd('save_notclose');"), '', $this->getBtnCancel());
	}

}
