<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0fb4cd0ea2f4 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-12-15 12:13 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_document_video extends we_document_binary{

	const ELEMENT_KEY = [
		'width' => self::ETYPE_NUMBER,
		'height' => self::ETYPE_NUMBER,
		'filesize' => self::ETYPE_NUMBER,
		'origheight' => self::ETYPE_NUMBER,
		'origwidth' => self::ETYPE_NUMBER,
		'poster' => self::ETYPE_NUMBER,
	];

	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::VIDEO;

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		if(isWE()){
			$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_PREVIEW;
		}
	}

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		switch($this->EditPageNr){
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				return new we_editor_document_preview($this, $jsCmd);
			default:
				return parent::editor($jsCmd);
		}
	}

	function formProperties(): string{
		//$weSuggest = we_gui_suggest::getInstance();
		$row = 0;
		$content = (new we_html_table(['class' => 'default propertydualtable'], 5, 3))
				->setCol($row, 0, $this->formInput('attrib', 'width#' . self::ETYPE_NUMBER, g_l('weClass', '[width]') . ' (origwidth: ' . $this->getElementI('origwidth') . ')', 155, 0, [
							'type' => 'number', 'oninput' => "_EditorFrame.setEditorIsHot(true);"]))
				->setCol($row, 1, $this->formInput('attrib', 'height#' . self::ETYPE_NUMBER, g_l('weClass', '[height]') . ' (origheight: ' . $this->getElementI('origheight') . ')', 155, 0, [
							'type' => 'number', 'oninput' => "_EditorFrame.setEditorIsHot(true);"]))
				->setCol($row, 2, $this->formDocChooser(155, 'poster', 'attrib'))
				->setCol(++$row, 0, $this->formSelectFromArray('attrib', 'autoplay', [0 => g_l('global', '[false]'), 1 => g_l('global', '[true]')], g_l('weClass', '[autoplay]'), 1, false, [
							'style' => 'width:155px;',
							'onchange' => "_EditorFrame.setEditorIsHot(true);"]))
				->setCol($row, 1, $this->formSelectFromArray('attrib', 'controller', [1 => g_l('global', '[true]'), 0 => g_l('global', '[false]')], g_l('weClass', '[controller]'), 1, false, [
							'style' => 'width:155px;',
							'onchange' => "_EditorFrame.setEditorIsHot(true);"]))
				->setCol($row, 2, $this->formColor(155, 'bgcolor', 'attrib'))
				->setCol(++$row, 0, $this->formSelectFromArray('attrib', 'mute', [0 => g_l('global', '[false]'), 1 => g_l('global', '[true]')], g_l('weClass', '[mute]'), 1, false, [
							'style' => 'width:155px;',
							'onchange' => "_EditorFrame.setEditorIsHot(true);"]))
				->setCol($row, 1, $this->formSelectFromArray('attrib', 'loop', [0 => g_l('global', '[false]'), 1 => g_l('global', '[true]')], g_l('weClass', '[loop]'), 1, false, [
							'style' => 'width:155px;',
							'onchange' => "_EditorFrame.setEditorIsHot(true);"]))
				->setCol($row, 2, $this->formInput('attrib', 'name', g_l('weClass', '[name]'), 155, 0, ['oninput' => "_EditorFrame.setEditorIsHot(true);"]));
		return $content->getHtml();
	}

	public function getHtml(bool $dyn = false, bool $preload = false): string{
		$filterAttribs = ['poster', 'width', 'height', 'autoplay', 'controller', 'bgcolor', 'mute', 'loop', 'name', 'filesize'];
		$data = $this->getElementS('data');
		if($this->ID || ($data && !is_dir($data) && is_readable($data))){

			if(($bdid = $this->getElementI('poster'))){
				$posterPath = id2path($bdid);
				$poster = ($posterPath && file_exists(WE_DOCUMENT_ROOT . $posterPath) ?
						$posterPath :
						'');
			} else {
				$poster = '';
			}
			$width = $preload ? 100 : $this->getElementI('width');
			$height = $preload ? 100 : $this->getElementI('height');
			$play = !$preload && $this->getElementB('autoplay');
			$control = !$preload && $this->getElementB('controller');
			$bgcolor = $preload ? '' : $this->getElementS('bgcolor');
			$mute = $this->getElementS('mute');
			$loop = $this->getElementS('loop');
			$name = $this->getElementS('name');

			$extraAttribs = [];
			foreach(array_diff($this->nextElement('attrib'), $filterAttribs) as $attrib){
				$extraAttribs[$attrib] = $this->getElementS($attrib);
			}

			return
					we_weTag_tag::htmlTag('video', array_merge($extraAttribs, [
						'style' => (
						($width ? 'max-width:' . $width . 'px;' : '') .
						($height ? 'max-height:' . $height . 'px;' : '') .
						//($preload ? 'margin-left:2em;' : '') .
						($bgcolor ? 'background-color:' . $bgcolor . ';' : '')),
						($play ? 'autoplay' : '') => 'autoplay',
						(!$preload && $control ? 'controls' : '') => 'controls',
						($mute ? 'muted' : '') => 'muted',
						($loop ? 'loop' : '') => 'loop',
						($name ? 'name' : '') => $name,
						($poster ? 'poster' : '') => $poster,
						'preload' => ($preload || !$poster ? 'metadata' : 'none')
							]), we_weTag_tag::htmlTag('source', ['src' => ( $dyn ?
								WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showBinaryDoc&we_cmd[1]=' . $this->ContentType . '&we_cmd[2]=' . we_base_WE::$transaction . '&rand=' . we_base_file::getUniqueId() :
								$this->Path),
								'type' => 'video/' . str_replace('.', '', $this->Extension)
							])
							, true);
		}
		return '';
	}

	protected function getPreviewImg(int $width = 150, int $height = 100): string{
		$file = (($bdid = $this->getElementI('poster')) ?
				DB_WE::$inst->getHash('SELECT Path,Extension,ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . $bdid) : '');

		return ($bdid && $file ?
				we_document_image::getPreview($bdid, $file['Path'], $file['Extension'], $width, $height) :
				$this->getHtml(true, true));
	}

	/**
	 * sets extra attributes for the image
	 *
	 * @return void
	 * @param array<string,mixed> $attribs
	 */
	function initByAttribs(array $attribs): void{
		foreach($attribs as $a => $b){
			switch($a){
				case 'id':
				case 'name':
					break;
				default:
					if((array_key_exists($a, self::ELEMENT_KEY) ? self::ELEMENT_KEY[$a] : self::ETYPE_TEXT) === self::ETYPE_NUMBER){
						$this->setElementI($a, (int) $b, 'attrib');
					} else {
						$this->setElementS($a, (string) $b, 'attrib');
					}
			}
		}
	}

	public static function checkAndPrepare(string $formname, we_contents_root $doc): void{
		$reqName = 'we_ui_' . $formname;
		if(empty($_FILES[$reqName]) || !is_array($_FILES[$reqName]) || empty($_FILES[$reqName]['name']) || !is_array($_FILES[$reqName]['name'])){
			return;
		}

		foreach($_FILES[$reqName]['name'] as $videoName => $filename){
			'@phan-var string $filename';
			'@phan-var string $videoName';
			if(!($videoDataId = we_base_request::string(we_fileupload::USERINPUT_TYPE_VIDEO . $videoName)) || empty($_SESSION[$videoDataId])){
				continue;
			}

			if(we_base_request::_(we_base_request::BOOL, we_fileupload::USERINPUT_TYPE_DEL . $videoName)){
				$_SESSION[$videoDataId]['doDelete'] = true;
				continue;
			}
			$_SESSION[$videoDataId]['doDelete'] = false;
			if($filename && (getContentTypeFromFile($filename) === we_base_ContentTypes::VIDEO)){
				$videoid = $doc->getElementI($videoName);
				$_SESSION[$videoDataId]['serverPath'] = $serverPath = TEMP_PATH . we_base_file::getUniqueId();
				we_base_file::insertIntoCleanUp($serverPath);
				move_uploaded_file((string) $_FILES[$reqName]['tmp_name'][$videoName], $serverPath);
				$unique = we_base_file::getUniqueId();
				$tmp_Filename = $videoName . '_' . $unique . '_' . preg_replace('[^A-Za-z0-9._-]', '', (string) $_FILES[$reqName]['name'][$videoName]);

				if($videoid){
					$_SESSION[$videoDataId]['id'] = $videoid;
				}
				[$tmp_Filename, $extension] = self::cleanUploadFilename($tmp_Filename);
				$_SESSION[$videoDataId] = array_merge($_SESSION[$videoDataId], [
					'fileName' => $tmp_Filename,
					'extension' => $extension,
					'text' => $tmp_Filename . $extension,
					'unique' => $unique,
					'type' => $_FILES[$reqName]['type'][$videoName],
					'size' => $_FILES[$reqName]['size'][$videoName],
				]);
			}
		}
	}

}
