<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e9585d3826ed $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-04-04 15:48 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_editor_info extends we_editor_base{

	public function show(): string{
		assert($this->we_doc instanceof we_document_base);
		$this->jsCmd->addCmd('setIconOfDocClass', 'docIcon');

		if(!in_array($this->we_doc->ContentType, we_base_ContentTypes::FOLDER_TYPES) && $this->we_doc->ContentType !== we_base_ContentTypes::COLLECTION){
			$fs = $this->we_doc->getFilesize();
		}

		$parts = (new we_html_multiIconBoxContent())
				->add((new we_html_multiIconBoxContentLine('ID', (string) ($this->we_doc->ID ?: '-'), we_html_element::span(['class' => 'docIcon', 'data-contenttype' => $this->we_doc->ContentType,
									'data-extension' => ($this->we_doc instanceof we_document_files ? $this->we_doc->Extension : '')])))
						->noline()->rightHead())
				->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[content_type]'), ($this->we_doc->ContentType ? g_l('weEditorInfo', '[' . $this->we_doc->ContentType . ']') : '')))
						->space(we_html_multiIconBox::SPACE_ICON)
						->noline()
						->rightHead())
				->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[file_size]'), (string) (empty($fs) ? 0 : we_base_file::getHumanFileSize($fs) . '&nbsp;(' . $fs . '&nbsp;Byte)')))
						->space(we_html_multiIconBox::SPACE_ICON)
						->rightHead()
						->hide(!isset($fs))
						->noline()
				)
				->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[creation_date]'), $this->we_doc->Creation_Date->format(g_l('weEditorInfo', '[date_format]')), we_html_multiIconBox::INFO_CALENDAR))
						->noline()
						->preline())
				->add((new we_html_multiIconBoxContentLine(g_l('modules_users', '[created_by]'), fS('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->we_doc->CreatorID)) ?: '-'))
						->space(we_html_multiIconBox::SPACE_ICON)
						->noline()
						->rightHead()
						->hide(empty($this->we_doc->CreatorID))
				)
				->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[changed_date]'), $this->we_doc->Mod_Date->format(g_l('weEditorInfo', '[date_format]'))))
						->space(we_html_multiIconBox::SPACE_ICON)
						->noline()
						->rightHead())
				->add((new we_html_multiIconBoxContentLine(g_l('modules_users', '[changed_by]'), fS('SELECT CONCAT(First," ",Second," (",username,")") FROM ' . USER_TABLE . ' WHERE ID=' . intval($this->we_doc->ModifierID)) ?: '-'))
						->space(we_html_multiIconBox::SPACE_ICON)
						->noline()
						->rightHead()
						->hide(empty($this->we_doc->ModifierID))
				)
				->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[lastLive]'), ($this->we_doc->Publish_Date ? $this->we_doc->Publish_Date->format(g_l('weEditorInfo', '[date_format]')) : '-') .
						(!in_array($this->we_doc->Table, [TEMPLATES_TABLE, VFILE_TABLE], true) && !in_array($this->we_doc->ContentType, we_base_ContentTypes::FOLDER_TYPES) && $this->we_doc->Publish_Date && $this->we_doc->Mod_Date > $this->we_doc->Publish_Date ?
						'<div style="margin-bottom:10px;">' . we_html_button::create_button('revert_published', "javascript:top.we_cmd('revert_published_question');") . '</div>' :
						'')))
				->space(we_html_multiIconBox::SPACE_ICON)
				->noline()
				->hide(!in_array($this->we_doc->ContentType, [we_base_ContentTypes::HTML, we_base_ContentTypes::WEDOCUMENT], true))
				->rightHead()
		);

		$anzeige = '';
		if(!in_array($this->we_doc->ContentType, we_base_ContentTypes::FOLDER_TYPES)){
			switch($this->we_doc->Table){
				case TEMPLATES_TABLE:
				case VFILE_TABLE:
					break;
				default:
					$rp = realpath($this->we_doc->getRealPath()) ?: '';
					$http = $this->we_doc->getHttpPath();

					switch($this->we_doc->ContentType){
						default:
							$showlink = false;
							break;
						case we_base_ContentTypes::WEDOCUMENT:
							$showlink = true;
							if(we_modules_info::isActive(we_modules_info::WORKFLOW)){
								$anzeige = (we_workflow_utility::inWorkflow($this->we_doc->ID, $this->we_doc->Table) ?
										we_workflow_utility::getDocumentStatusInfo($this->we_doc->ID, $this->we_doc->Table) :
										we_workflow_utility::getLogButton($this->we_doc->ID, $this->we_doc->Table));
							}
							break;

						case we_base_ContentTypes::HTML:
						case we_base_ContentTypes::IMAGE:
						case we_base_ContentTypes::VIDEO:
						case we_base_ContentTypes::AUDIO:
						case we_base_ContentTypes::PDF:
							$showlink = true;
					}

					$published = !(($this->we_doc->ContentType === we_base_ContentTypes::HTML || $this->we_doc->ContentType === we_base_ContentTypes::WEDOCUMENT) && !$this->we_doc->Publish_Date);

					$parts->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[local_path]'), ($this->we_doc->ID == 0 || !$published ? '-' : '<span title="' . htmlspecialchars($rp) . '">' . htmlspecialchars(we_base_util::shortenPath($rp, 74)) . '</span>'), we_html_multiIconBox::PROP_PATH))
									->noline()
									->preline())
							->add((new we_html_multiIconBoxContentLine(g_l('weEditorInfo', '[http_path]'), ($this->we_doc->ID == 0 || !$published ? '-' : ($showlink ? '<a href="' . $http . '" target="_blank" title="' . htmlspecialchars($http) . '">' : '') . $http . ($showlink ? '</a>' : ''))))
									->space(we_html_multiIconBox::SPACE_ICON)
									->noline()
									->rightHead());
			}

			if($anzeige){
				$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_workflow', '[workflow]'), $anzeige, we_html_multiIconBox::INFO_WORKFLOW))
								->rightHead()
								->preline()
								->noline());
			}

			//show usage
			if(we_modules_info::isActive(we_modules_info::COLLECTION) || we_base_ContentTypes::isBinaryFile($this->we_doc->ContentType)){
				if(we_base_ContentTypes::isRealFile($this->we_doc->ContentType) || we_base_ContentTypes::isWEFile($this->we_doc->ContentType)){
					$formReference = $this->we_doc->formReferences($this->getJsCmd());

					$parts->add((new we_html_multiIconBoxContentLine(
											g_l('weClass', '[isUsed]') . (we_base_ContentTypes::isBinaryFile($this->we_doc->ContentType) ? '' : ' in ' . g_l('global', '[vfile]')) . ' (' . $formReference['num'] . ')',
											$formReference['form'], we_html_multiIconBox::INFO_REFERENCES))
									->rightHead()
									->preline());
				}
			}

			switch($this->we_doc->ContentType){
				case we_base_ContentTypes::TEMPLATE:
					assert($this->we_doc instanceof we_contents_template);
					[$cnt, $select] = $this->we_doc->formTemplateDocuments();
					$parts->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[documents]') . ($cnt ? ' (' . $cnt . ')' : ''), $select, we_html_multiIconBox::INFO_REFERENCES))
									->noline()
									->preline());
					[$cnt, $select] = $this->we_doc->formTemplatesUsed();
					$parts->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[usedTemplates]') . ($cnt ? ' (' . $cnt . ')' : ''), $select))
									->space(we_html_multiIconBox::SPACE_ICON)
									->noline()
									->rightHead()
					);
					[$cnt, $select] = $this->we_doc->formTemplateUsedByTemplate();
					$parts->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[usedByTemplates]') . ($cnt ? ' (' . $cnt . ')' : ''), $select))
									->space(we_html_multiIconBox::SPACE_ICON)
									->noline()
									->rightHead());
					break;
				case we_base_ContentTypes::IMAGE:
					assert($this->we_doc instanceof we_document_image);
					$metaData = $this->we_doc->getMetaData();
					$metaDataTable = '<table class="default">';
					if(empty($metaData['exif'])){
						$metaDataTable .= '<tr><td style="padding:0 5px 5px 0;" class="defaultfont" colspan="2">' . g_l('metadata', (is_callable("exif_read_data") ? '[no_exif_data]' : '[no_exif_installed]')) . '</td></tr>';
					} else {
						foreach($metaData['exif'] as $key => $val){
							$metaDataTable .= '<tr><td style="padding:0 5px 5px 0;" class="defaultfont">' . htmlspecialchars($key) . ':</td><td style="padding:0 5px 5px 0;">' . $val . '</td></tr>';
						}
					}
					$metaDataTable .= '</table>';
					$parts->add((new we_html_multiIconBoxContentLine(g_l('metadata', '[info_exif_data]'), $metaDataTable, we_html_multiIconBox::PROP_META))
									->rightHead()
					);

					$metaDataTable = '<table class="default">';
					if(empty($metaData['iptc'])){
						$metaDataTable .= '<tr><td style="padding:0 5px 5px 0;" class="defaultfont" colspan="2">' . g_l('metadata', '[no_iptc_data]') . '</td></tr>';
					} else {
						foreach($metaData['iptc'] as $key => $val){
							$metaDataTable .= '<tr><td style="padding:0 5px 5px 0;" class="defaultfont">' . htmlspecialchars($key) . ':</td><td style="padding:0 5px 5px 0;">' . htmlspecialchars($val) . '</td></tr>';
						}
					}
					$metaDataTable .= '</table>';
					$parts->add((new we_html_multiIconBoxContentLine(g_l('metadata', '[info_iptc_data]'), $metaDataTable, we_html_multiIconBox::PROP_META))
									->rightHead());
					break;
				case we_base_ContentTypes::PDF:
					if(!($this->we_doc instanceof we_document_pdf)){
						$parts->add((new we_html_multiIconBoxContentLine(g_l('metadata', '[info_pdf_data]'), g_l('alert', '[error]'), we_html_multiIconBox::PROP_META))
										->rightHead());
						break;
					}
					$metaData = $this->we_doc->getMetaData();
					$metaDataTable = '<table class="default">';
					if(!empty($metaData['pdf'])){
						foreach($metaData['pdf'] as $key => $val){
							$metaDataTable .= '<tr><td style="padding:0 5px 5px 0;" class="defaultfont">' . htmlspecialchars($key) . ':</td><td style="padding:0 5px 5px 0;" class="defaultfont">' . htmlspecialchars($val) . '</td></tr>';
						}
					}
					$metaDataTable .= '</table>';
					$parts->add((new we_html_multiIconBoxContentLine(g_l('metadata', '[info_pdf_data]'), $metaDataTable, we_html_multiIconBox::PROP_META))
									->rightHead());
					break;

				case we_base_ContentTypes::APPLICATION:
				default:
					if(we_base_ContentTypes::isBinaryFile($this->we_doc->ContentType)){
						//$metaDataTable = g_l('metadata', '[no_metadata_supported]');
					}
			}
		}
		return $this->getPage((new we_html_multiIconBox($parts))->marginLeft()->get());
	}

}
