<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 75c07183db77 $
 * $Author: Lukas Imhof $
 * $Date: 2022-03-30 21:45 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_gui_mainMenu{
	private static $isSeemode = false;

	public const E_MODULE = 'module';
	public const E_SETTINGS = 'extras';

	/**
	 * @return we_gui_menu
	 */
	public static function getMenu(): we_gui_menu{
		self::$isSeemode = we_editor_functions::isSeeMode();
		return new we_gui_menu([
			self::getNew(),
			self::getFile(),
			self::getCockpit(),
			self::getModules(),
			self::getSettings(),
			self::getHelp()
		]);
	}

	private static function getNew(): we_gui_menuTop{
		return
				(new we_gui_menuTop(g_l('menu_global', '[new]'), 'fa fa-plus-circle'))
				->set([
					self::getDT(),
					// New > Object
					self::getObj(),
					(new we_gui_menuSub('file_new_media', g_l('menu_global', '[media]'), [we_base_perm::WORK_IMAGE_NEW, we_base_perm::WORK_VIDEO_NEW, we_base_perm::WORK_OTHER_NEW]))
					->set([
// File > Image
						new we_gui_menuEntry(g_l('menu_global', '[image]'), ['new', FILE_TABLE, '', we_base_ContentTypes::IMAGE], [we_base_perm::WORK_IMAGE_NEW]),
						// New > Other > Other (Binary)
						new we_gui_menuEntry(g_l('menu_global', '[pdf]'), ['new', FILE_TABLE, '', we_base_ContentTypes::PDF], [we_base_perm::WORK_OTHER_NEW]),
// New Other > video
						new we_gui_menuEntry(g_l('contentTypes', '[' . we_base_ContentTypes::VIDEO . ']'), ['new', FILE_TABLE, '', we_base_ContentTypes::VIDEO], [
							we_base_perm::WORK_VIDEO_NEW]),
// New Other > audio
						new we_gui_menuEntry(g_l('contentTypes', '[' . we_base_ContentTypes::AUDIO . ']'), ['new', FILE_TABLE, '', we_base_ContentTypes::AUDIO], [
							we_base_perm::WORK_VIDEO_NEW]),
					]),
					// New > Other
					(new we_gui_menuSub('file_new_other', g_l('menu_global', '[other]'), [we_base_perm::WORK_HTML_NEW, we_base_perm::WORK_JS_NEW,
						we_base_perm::WORK_CSS_NEW, we_base_perm::WORK_TEXT_NEW, we_base_perm::WORK_HTACCESS_NEW,
						we_base_perm::WORK_OTHER_NEW]))
					->set([
// New > Other > html
						new we_gui_menuEntry(g_l('menu_global', '[html_page]'), ['new', FILE_TABLE, '', we_base_ContentTypes::HTML], [we_base_perm::WORK_HTML_NEW]),
// New > Other > Javascript
						new we_gui_menuEntry(g_l('menu_global', '[javascript]'), ['new', FILE_TABLE, '', we_base_ContentTypes::JS], [we_base_perm::WORK_JS_NEW]),
						// New > Other > JSON
						new we_gui_menuEntry(g_l('menu_global', '[json]'), ['new', FILE_TABLE, '', we_base_ContentTypes::JSON], [we_base_perm::WORK_OTHER_NEW]),

// New > Other > CSS
						new we_gui_menuEntry(g_l('menu_global', '[css_stylesheet]'), ['new', FILE_TABLE, '', we_base_ContentTypes::CSS], [we_base_perm::WORK_CSS_NEW]),
// New > Other > Text
						new we_gui_menuEntry(g_l('menu_global', '[text_plain]'), ['new', FILE_TABLE, '', we_base_ContentTypes::TEXT], [we_base_perm::WORK_TEXT_NEW]),
						// New > Other > XML
						new we_gui_menuEntry(g_l('menu_global', '[text_xml]'), ['new', FILE_TABLE, '', we_base_ContentTypes::XML], [we_base_perm::WORK_TEXT_NEW]),
						// New > Other > htaccess
						new we_gui_menuEntry(g_l('menu_global', '[htaccess]'), ['new', FILE_TABLE, '', we_base_ContentTypes::HTACCESS], [we_base_perm::WORK_HTACCESS_NEW]),
						// New > Other > Other (Binary)
						new we_gui_menuEntry(g_l('menu_global', '[other_files]'), ['new', FILE_TABLE, '', we_base_ContentTypes::APPLICATION], [we_base_perm::WORK_OTHER_NEW]),
					]),
					new we_gui_menuSeparator(self::$isSeemode),
					// New > Directory
					(new we_gui_menuSub('file_new_dir', g_l('menu_global', '[directory]'), [we_base_perm::WORK_DOCUMENT_FOLDER_NEW, we_base_perm::WORK_TEMPLATES_FOLDER_NEW,
						we_base_perm::OBJECTFILE_NEW_FOLDER, we_base_perm::WORK_COLLECTION_FOLDER_NEW], self::$isSeemode))
					->set([
// New > Directory > Document
						new we_gui_menuEntry(g_l('menu_global', '[document_directory]'), ['new', FILE_TABLE, '', we_base_ContentTypes::FOLDER_DOCUMENT], [we_base_perm::WORK_DOCUMENT_FOLDER_NEW], self::$isSeemode),
// New > Directory > Template
						new we_gui_menuEntry(g_l('menu_global', '[template_directory]'), ['new', TEMPLATES_TABLE, '', we_base_ContentTypes::FOLDER_TEMPLATE], [we_base_perm::WORK_TEMPLATES_FOLDER_NEW], self::$isSeemode),
						//new we_gui_menuEntry(g_l('menu_global', '[phpclass_directory]'), ['new', TEMPLATES_TABLE, '', we_base_ContentTypes::FOLDER_PHPCLASS], [we_base_perm::WORK_TEMPLATES_FOLDER_NEW], self::$isSeemode),
// New > directory > objectfolder
						new we_gui_menuEntry(g_l('modules_object', '[menu][object_directory]'), ['new', OBJECT_FILES_TABLE, '', we_base_ContentTypes::FOLDER_OBJECTFILE], [
							we_base_perm::OBJECTFILE_NEW_FOLDER], !we_modules_info::isActive(we_modules_info::OBJECT) || (we_editor_functions::isSeeMode())),
						/* // New > directory > classfolder
						  new we_gui_menuEntry(g_l('modules_object', '[menu][object_directory]'),['new_object_folder'],[we_base_perm::OBJECT_NEW],
						  !we_modules_info::isActive(we_modules_info::OBJECT) || ($_SESSION['weS']['we_mode'] != we_base_constants::MODE_NORMAL)), */
						// New > Directory > Collection
						new we_gui_menuEntry(g_l('menu_global', '[collection_directory]'), ['new', VFILE_TABLE, '', we_base_ContentTypes::FOLDER_VFILE], [we_base_perm::WORK_COLLECTION_FOLDER_NEW], self::$isSeemode || !we_modules_info::isActive(we_modules_info::COLLECTION)),
					]),
					new we_gui_menuSeparator(self::$isSeemode, [we_base_perm::OBJECT_NEW, we_base_perm::WORK_TEMPLATE_NEW]),
// New > Template
					new we_gui_menuEntry(g_l('menu_global', '[template]'), ['new_template'], [we_base_perm::WORK_TEMPLATE_NEW], self::$isSeemode),
					//new we_gui_menuEntry(g_l('menu_global', '[phpClass]'), ['new_phpClass'], [we_base_perm::WORK_TEMPLATE_NEW], self::$isSeemode),
// New > Class
					new we_gui_menuEntry(g_l('modules_object', '[menu][class]'), ['new_object'], [we_base_perm::OBJECT_NEW], !we_modules_info::isActive(we_modules_info::OBJECT) || (we_editor_functions::isSeeMode())),
					new we_gui_menuSeparator(self::$isSeemode || !we_modules_info::isActive(we_modules_info::COLLECTION), [we_base_perm::WORK_COLLECTION_NEW]),
// File > COLLECTION
					new we_gui_menuEntry(g_l('menu_global', '[collection]'), ['new', VFILE_TABLE, '', we_base_ContentTypes::COLLECTION], [we_base_perm::WORK_COLLECTION_NEW], !we_modules_info::isActive(we_modules_info::COLLECTION)),
		]);
	}

	private static function getDT(): we_gui_menuSub{
		$dt = (new we_gui_menuSub('file_new_wedoc0', g_l('menu_global', '[webEdition_page]'), [we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW]))
			->set([
			// New > webEdition Document > empty page
			new we_gui_menuEntry(g_l('menu_global', '[empty_page]'), ['new', FILE_TABLE, '', we_base_ContentTypes::WEDOCUMENT], [we_base_perm::WORK_DOCUMENT_NO_DOCTYPE_ALLOWED]),
			new we_gui_menuSeparator(!self::$isSeemode),
		]);

		$dtq = we_doctype_docTypes::getDoctypeQuery(false, true);
		$all = DB_WE::$inst->getAllq('SELECT dt.ID,dt.Text,dt.isFolder,dt.ParentID FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where']);

		if($all && we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_NO_DOCTYPE_ALLOWED])){
			$dt->append(new we_gui_menuSeparator()); // separator
		}
		$parents = [0 => $dt];
		$parent = 0;
// New > webEdition Document > Doctypes*
		foreach($all as $cur){
			if($cur['isFolder']){//FIXME:must be fixed
				$parents[$parent]->append(new we_gui_menuSub('file_new_wedoc' . $cur['ID'], strtr($cur['Text'], [',' => ' ', '"' => '', '\'' => '',]), [we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW]));
				$parent = $cur['ID'];
			} else {
				$parents[$parent]->append(new we_gui_menuEntry(strtr($cur['Text'], [',' => ' ', '"' => '', '\'' => '',]), ['new', FILE_TABLE, '', we_base_ContentTypes::WEDOCUMENT,
						$cur['ID']], [
						we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW]));
			}
		}
		return $dt;
	}

	private static function getObj(): we_gui_menuSub{
		$ret = new we_gui_menuSub('file_new_weobj', g_l('modules_object', '[menu][object]'), [we_base_perm::OBJECTFILE_NEW], !we_modules_info::isActive(we_modules_info::OBJECT));

		if(!we_modules_info::isActive(we_modules_info::OBJECT)){
			return $ret;
		}
		// object from which class
		$ac = we_users_util::getAllowedClasses();
		if(!$ac || !($all = DB_WE::$inst->getAllq('SELECT ID,Text FROM ' . OBJECT_TABLE . ' WHERE ID IN(' . implode(',', $ac) . ') ORDER BY Text'))){
			$ret->hide = true;
			return $ret;
		}
		foreach($all as $cur){
			$ret->append(new we_gui_menuEntry(strtr($cur['Text'], ['"' => '', '\'' => '']), ['new', OBJECT_FILES_TABLE, '', we_base_ContentTypes::OBJECT_FILE, $cur['ID']], [
					we_base_perm::OBJECTFILE_NEW]));
		}
		return $ret;
	}

	private static function getFile(): we_gui_menuTop{
		return
			//File
				(new we_gui_menuTop(g_l('menu_global', '[file]'), 'fa fa-file'))
				->set([
					// File > Open
					(new we_gui_menuSub('file_open', g_l('menu_global', '[open]'), [], !self::$isSeemode))
					->set([
						// File > Open > Document
						new we_gui_menuEntry(g_l('menu_global', '[open_document]') . '&hellip;', ['open_document'], [we_base_perm::WORK_DOCUMENTS_CAN_SEE], !self::$isSeemode),
						// File > open > Object
						new we_gui_menuEntry(g_l('modules_object', '[menu][open_object]') . '&hellip;', ['open_objectFile'], [we_base_perm::OBJECTFILES_CAN_SEE], !self::$isSeemode || !we_modules_info::isActive(we_modules_info::OBJECT)),
// File > Open > Collection
						new we_gui_menuEntry(g_l('menu_global', '[collection]') . '&hellip;', ['open_collection'], [we_base_perm::WORK_COLLECTIONS_CAN_SEE], !self::$isSeemode || !we_modules_info::isActive(we_modules_info::COLLECTION)),
					]),
					// File > Delete
					(self::$isSeemode ?
					new we_gui_menuEntry(g_l('menu_global', '[delete]') . '&hellip;', ['we_selector_delete'], [we_base_perm::WORK_DOCUMENT_DELETE]) :
					(new we_gui_menuSub('file_delete', g_l('menu_global', '[delete]'), [we_base_perm::WORK_DOCUMENT_DELETE, we_base_perm::OBJECTFILE_DELETE,
						we_base_perm::WORK_TEMPLATE_DELETE,
						we_base_perm::OBJECT_DELETE]))
					->set([
						// File > Delete > Documents
						new we_gui_menuEntry(g_l('menu_global', '[documents]'), ['del', 1, FILE_TABLE], [we_base_perm::WORK_DOCUMENT_DELETE], self::$isSeemode),
						// File > Delete > Objects
						new we_gui_menuEntry(g_l('modules_object', '[menu][objects]'), ['del', 1, OBJECT_FILES_TABLE], [we_base_perm::OBJECTFILE_DELETE], !we_modules_info::isActive(we_modules_info::OBJECT) || (we_editor_functions::isSeeMode())),
						new we_gui_menuSeparator(self::$isSeemode, [we_base_perm::WORK_TEMPLATES_CAN_SEE, we_base_perm::OBJECTS_CAN_SEE]),
// File > Delete > Templates
						new we_gui_menuEntry(g_l('menu_global', '[templates]'), ['del', 1, TEMPLATES_TABLE], [we_base_perm::WORK_TEMPLATE_DELETE], self::$isSeemode),
// File > Delete > Classes
						new we_gui_menuEntry(g_l('modules_object', '[menu][classes]'), ['del', 1, OBJECT_TABLE], [we_base_perm::OBJECT_DELETE], !we_modules_info::isActive(we_modules_info::OBJECT) || (we_editor_functions::isSeeMode())),
						new we_gui_menuSeparator(self::$isSeemode || !we_modules_info::isActive(we_modules_info::COLLECTION), [we_base_perm::WORK_COLLECTION_DELETE,
							we_base_perm::WORK_COLLECTION_FOLDER_DELETE]),
// File > Delete > Collection
						new we_gui_menuEntry(g_l('global', '[vfile]'), ['del', 1, VFILE_TABLE], [we_base_perm::WORK_COLLECTION_DELETE, we_base_perm::WORK_COLLECTION_FOLDER_DELETE], self::$isSeemode || !we_modules_info::isActive(we_modules_info::COLLECTION)),
					])
					),
					// File > Move
					(new we_gui_menuSub('file_mv', g_l('menu_global', '[move]'), [we_base_perm::WORK_DOCUMENT_MOVE, we_base_perm::OBJECTFILE_MOVE, we_base_perm::WORK_TEMPLATE_MOVE], self::$isSeemode))
					->set([
						// File > Move > Documents
						new we_gui_menuEntry(g_l('menu_global', '[documents]'), ['mv', 1, FILE_TABLE], [we_base_perm::WORK_DOCUMENT_MOVE], self::$isSeemode),
						// File > move > objects
						new we_gui_menuEntry(g_l('modules_object', '[menu][objects]'), ['mv', 1, OBJECT_FILES_TABLE], [we_base_perm::OBJECTFILE_MOVE], !we_modules_info::isActive(we_modules_info::OBJECT) || (we_editor_functions::isSeeMode())),
						new we_gui_menuSeparator(self::$isSeemode, [we_base_perm::WORK_TEMPLATE_MOVE]),
// File > Move > Templates
						new we_gui_menuEntry(g_l('menu_global', '[templates]'), ['mv', 1, TEMPLATES_TABLE], [we_base_perm::WORK_TEMPLATE_MOVE], self::$isSeemode),
					]),
					// File > add to collection
					(new we_gui_menuSub('file_addcoll', g_l('menu_global', '[add_to_collection]'), [we_base_perm::WORK_COLLECTION_SAVE], self::$isSeemode || !we_modules_info::isActive(we_modules_info::COLLECTION)))
					->set([
// File > add to collection > documents
						new we_gui_menuEntry(g_l('menu_global', '[documents]'), ['tocollection', 1, FILE_TABLE], [we_base_perm::WORK_COLLECTION_SAVE],
															 self::$isSeemode || !we_modules_info::isActive(we_modules_info::COLLECTION)),
						/// File > add to collection > objects
						new we_gui_menuEntry(g_l('modules_object', '[menu][objects]'), ['tocollection', 1, OBJECT_FILES_TABLE], [we_base_perm::WORK_COLLECTION_SAVE], !we_modules_info::isActive(we_modules_info::OBJECT) || self::$isSeemode || !we_modules_info::isActive(we_modules_info::COLLECTION)),
					]),
					new we_gui_menuSeparator(self::$isSeemode),
// File > Close All
					new we_gui_menuEntry(g_l('menu_global', '[close_all_documents]'), ['close_all_documents'], [], self::$isSeemode),
// File > Close All But this
					new we_gui_menuEntry(g_l('menu_global', '[close_all_but_active_document]'), ['close_all_but_active_document'], [], self::$isSeemode),
					new we_gui_menuSeparator(),
					// File > Search
					(new we_gui_menuSub('file_search', g_l('searchtool', '[suchen]')))
					->set([
// File > Search
						new we_gui_menuEntry(g_l('menu_global', '[search]') . '&hellip;', ['we_' . we_modules_info::SEARCH . '_edit']),
// File > unpublished pages
						new we_gui_menuEntry(g_l('menu_global', '[media]') . '&hellip;', ['we_' . we_modules_info::SEARCH . '_edit', '', '', 0, 5], [we_base_perm::WORK_DOCUMENTS_CAN_SEE]),
						// File > unpublished pages
						new we_gui_menuEntry(g_l('menu_global', '[unpublished_pages]') . '&hellip;', ['openUnpublishedPages'], [we_base_perm::WORK_DOCUMENTS_CAN_SEE]),
						// File > unpublished objects
						new we_gui_menuEntry(g_l('modules_object', '[menu][unpublished_objects]') . '&hellip;', ['openUnpublishedObjects'], [we_base_perm::OBJECTFILES_CAN_SEE], !we_modules_info::isActive(we_modules_info::OBJECT)),
					]),
					new we_gui_menuSeparator(),
					// File > Import/Export
					(new we_gui_menuSub('file_imex', g_l('menu_global', '[import_export]'), [we_base_perm::EXIM_EXPORT_GENERICXML, we_base_perm::EXIM_EXPORT_CSV,
						we_base_perm::EXIM_IMPORT_FILE, we_base_perm::EXIM_IMPORT_SITE, we_base_perm::EXIM_IMPORT_GENERICXML,
						we_base_perm::EXIM_IMPORT_CSV,
						we_base_perm::EXIM_IMPORT_WXML, we_base_perm::EXIM_EXPORTNODOWNLOAD, we_base_perm::EXIM_EXPORT, we_base_perm::EXIM_EXPORT_EDIT, we_base_perm::EXIM_EXPORT_MAKE]))
					->set([
						// File > Import/Export > Import
						new we_gui_menuEntry(g_l('menu_global', '[import]') . '&hellip;', ['import'], [we_base_perm::EXIM_IMPORT_FILE, we_base_perm::EXIM_IMPORT_SITE, we_base_perm::EXIM_IMPORT_GENERICXML,
							we_base_perm::EXIM_IMPORT_CSV, we_base_perm::EXIM_IMPORT_WXML]),
						// File > Import/Export > Export
						new we_gui_menuEntry(g_l('menu_global', '[export]') . '&hellip;', ['we_' . we_modules_info::EXPORT . '_edit'], [we_base_perm::EXIM_EXPORT_GENERICXML,
							we_base_perm::EXIM_EXPORT_CSV, we_base_perm::EXIM_EXPORTNODOWNLOAD, we_base_perm::EXIM_EXPORT, we_base_perm::EXIM_EXPORT_EDIT,
							we_base_perm::EXIM_EXPORT_MAKE], !we_modules_info::isActive(we_modules_info::EXPORT)),
					])
		]);
	}

	private static function getCockpit(): we_gui_menuTop{
		return
				(new we_gui_menuTop(g_l('global', '[cockpit]'), 'fa fa-th-large', [we_base_perm::WORK_QUICKSTART_CAN_SEE]))
				->set([
					// Cockpit > Display
					new we_gui_menuEntry(g_l('menu_global', '[display]'), ['home'], [we_base_perm::WORK_QUICKSTART_CAN_SEE]),
					// Cockpit > new Widget
					(new we_gui_menuSub('cockpit_new', g_l('menu_global', '[new_widget]'), [we_base_perm::WORK_QUICKSTART_CAN_SEE]))
					->set([
// Cockpit > new Widget > shortcuts
						new we_gui_menuEntry(g_l('menu_global', '[widget][shortcuts]'), ['new_widget', 'sct'], [we_base_perm::WORK_QUICKSTART_CAN_SEE]),
						// Cockpit > new Widget > RSS
						new we_gui_menuEntry(g_l('menu_global', '[widget][rss]'), ['new_widget', 'rss'], [we_base_perm::WORK_QUICKSTART_CAN_SEE]),
						// Cockpit > new Widget > Shop
						new we_gui_menuEntry(g_l('menu_global', '[widget][shop]'), ['new_widget', 'shp'], [we_base_perm::WORK_QUICKSTART_CAN_SEE, we_base_perm::SHOP_NEW_ARTICLE,
							we_base_perm::SHOP_DELETE_ARTICLE, we_base_perm::SHOP_EDIT_ORDER, we_base_perm::SHOP_DELETE_ORDER,
							we_base_perm::SHOP_EDIT_PREFS], !we_modules_info::isActive(we_modules_info::SHOP)),
// Cockpit > new Widget > online users
						new we_gui_menuEntry(g_l('menu_global', '[widget][users_online]'), ['new_widget', 'usr'], [we_base_perm::WORK_QUICKSTART_CAN_SEE, we_base_perm::WIDGET_SHOW_USR]),
						// Cockpit > new Widget > lastmodified
						new we_gui_menuEntry(g_l('menu_global', '[widget][last_modified]'), ['new_widget', 'mfd'], [we_base_perm::WORK_QUICKSTART_CAN_SEE]),
						// Cockpit > new Widget > unpublished
						new we_gui_menuEntry(g_l('menu_global', '[widget][unpublished]'), ['new_widget', 'upb'], [we_base_perm::WORK_QUICKSTART_CAN_SEE]),
// Cockpit > new Widget > my Documents
						new we_gui_menuEntry(g_l('menu_global', '[widget][my_documents]'), ['new_widget', 'mdc'], [we_base_perm::WORK_QUICKSTART_CAN_SEE]),
						// Cockpit > new Widget > Notepad
						new we_gui_menuEntry(g_l('menu_global', '[widget][notepad]'), ['new_widget', 'pad'], [we_base_perm::WORK_QUICKSTART_CAN_SEE]),
						new we_gui_menuEntry(g_l('menu_global', '[widget][kv_failedLogins]'), ['new_widget', 'fdl'], [we_base_perm::CUSTOMER_EDIT, we_base_perm::CUSTOMER_NEW], !we_modules_info::isActive(we_modules_info::CUSTOMER) || !we_base_permission::hasPerm([
								we_base_perm::WORK_QUICKSTART_CAN_SEE])),
						new we_gui_menuEntry(g_l('menu_global', '[widget][workflow]'), ['new_widget', 'wfl'], [we_base_perm::WORK_QUICKSTART_CAN_SEE],!we_modules_info::isActive(we_modules_info::WORKFLOW) ),
					]),
					// Cockpit > new Widget > default settings
					new we_gui_menuEntry(g_l('menu_global', '[default_settings]'), ['reset_home'], [we_base_perm::WORK_QUICKSTART_CAN_SEE]),
		]);
	}

	private static function getModules(): we_gui_menuTop{
		$we_menu = new we_gui_menuTop(g_l('menu_global', '[modules]'), 'fa fa-cubes');
		foreach(we_modules_info::showModuleInMenu(self::E_MODULE, '') as $modInfo){
			'@phan-var array{text:string,name:string,perm:array} $modInfo';
			$we_menu->append(new we_gui_menuEntry($modInfo['text'] . '&hellip;', ['we_' . $modInfo['name'] . '_edit'], $modInfo['perm'] ?? []));
		}
		return $we_menu;
	}

	private static function getSettings(): we_gui_menuTop{
		$set = [
			// Extras > change password
			new we_gui_menuEntry(g_l('menu_global', '[change_password]') . '&hellip;', ['we_gui_changePasswd'], [we_base_perm::USER_EDIT_PASSWD]),
			new we_gui_menuSeparator(false, [we_base_perm::USER_EDIT_PASSWD]),
		];


//add basic settings, formarly modules
		foreach(we_modules_info::showModuleInMenu(self::E_SETTINGS, '') as $modInfo){
			'@phan-var array{text:string,name:string,perm:array} $modInfo';
			$set[] = new we_gui_menuEntry($modInfo['text'] . '&hellip;', ['we_' . $modInfo['name'] . '_edit'], $modInfo['perm'] ?? []);
		}

		$modSett = new we_gui_menuSub('moduleSettings', g_l('menu_global', '[moduleSettings]') . '&hellip;', [we_base_perm::ADMINISTRATOR]);
		foreach(we_modules_info::showModuleInMenu('', 'hasSettings') as $modInfo){
			'@phan-var array{text:string,name:string,perm:array} $modInfo';
			$modSett->append(new we_gui_menuEntry($modInfo['text'] . '&hellip;', ['we_' . $modInfo['name'] . '_editSettings'], $modInfo['perm'] ?? []));
		}

		return (new we_gui_menuTop(g_l('menu_global', '[preferences]'), 'fa fa-gears'))
				->set(array_merge($set, [
					// Extras > Metadata fields
					new we_gui_menuEntry(g_l('menu_global', '[metadata]') . '&hellip;', ['editMetadataFields'], [we_base_perm::ADMINISTRATOR], self::$isSeemode),
					new we_gui_menuSeparator(false, [we_base_perm::USER_EDIT_SETTINGS]),
					/* dialog does not work correctly
					 * [// Extras > versioning
					  'text' => g_l('menu_global', '[versioning]') . '&hellip;',
					  'cmd' => 'we_versions_wizard',
					  'perm' => [we_base_perm::ADMINISTRATOR],
					  ], */
					new we_gui_menuEntry(g_l('menu_global', '[common]') . '&hellip;', ['we_gui_openPreferences'], [we_base_perm::USER_EDIT_SETTINGS]),
					$modSett,
					new we_gui_menuSeparator(false, [we_base_perm::ADMINISTRATOR]),
					// Backup
					(new we_gui_menuSub('file_backup', g_l('menu_global', '[backup]'), [we_base_perm::EXIM_BACKUPLOG, we_base_perm::EXIM_IMPORT, we_base_perm::EXIM_EXPORT,
						we_base_perm::EXIM_EXPORTNODOWNLOAD]))
					->set([
// Backup > make
						new we_gui_menuEntry(g_l('menu_global', '[make_backup]') . '&hellip;', ['make_backup'], [we_base_perm::EXIM_EXPORT,
							we_base_perm::EXIM_EXPORTNODOWNLOAD]),
// Backup > recover
						new we_gui_menuEntry(g_l('menu_global', '[recover_backup]') . '&hellip;', ['recover_backup'], [we_base_perm::EXIM_IMPORT], self::$isSeemode),
// Backup > view Log
						new we_gui_menuEntry(g_l('menu_global', '[view_backuplog]') . '&hellip;', ['view_backuplog'], [we_base_perm::EXIM_BACKUPLOG]),
					]),
// rebuild
					new we_gui_menuEntry(g_l('menu_global', '[rebuild]') . '&hellip;', ['rebuild'], [we_base_perm::REBUILD]),
					// File > Browse server
					new we_gui_menuEntry(g_l('menu_global', '[browse_server]') . '&hellip;', ['browse_server'], [we_base_perm::WORK_BROWSE_SERVER], self::$isSeemode),
					// File > Browse database
					new we_gui_menuEntry(g_l('menu_global', '[browse_database]') . '&hellip;', ['browse_database'], [we_base_perm::ADMINISTRATOR], self::$isSeemode),
					new we_gui_menuSeparator(),
					new we_gui_menuEntry(g_l('menu_global', '[update]') . '&hellip;', ['update'], [we_base_perm::SETTINGS_UPDATE, we_base_perm::ADMINISTRATOR]),
					new we_gui_menuEntry(g_l('menu_global', '[sysinfo]') . '&hellip;', ['we_gui_sysinfo']),
					new we_gui_menuEntry(g_l('menu_global', '[showerrorlog]') . '&hellip;', ['we_gui_showerrorlog'], [we_base_perm::SETTINGS_SEE_ERRORLOG, we_base_perm::ADMINISTRATOR]),
// Extras > versioning-log
					new we_gui_menuEntry(g_l('menu_global', '[versioning_log]') . '&hellip;', ['we_versions_log'], [we_base_perm::ADMINISTRATOR]),
					])
		);
	}

	private static function getHelp(): we_gui_menuTop{
		$vers = explode('.', WE_VERSION);
		array_pop($vers);
		$verString = implode('.', $vers);
		return
				(new we_gui_menuTop(g_l('menu_global', '[help]'), 'fa fa-question-circle'))
				->set([
					new we_gui_menuEntry(g_l('menu_global', '[sidebar]') . '&hellip;', ['openSidebar'], [], !(SIDEBAR_DISABLED == 0)),
					new we_gui_menuSeparator(!(SIDEBAR_DISABLED == 0), [we_base_perm::ADMINISTRATOR]),
					new we_gui_menuEntry(g_l('menu_global', '[webEdition_online]') . '&hellip;', ['help_ext_url', 'https://www.webedition.org', 'we_org']),
					new we_gui_menuEntry(g_l('menu_global', '[webEdition_partner]') . '&hellip;', ['help_ext_url', 'https://www.webedition.org/de/partner/', 'partner'], [], WE_LICENSE !== 'GPL'),
					new we_gui_menuEntry(g_l('menu_global', '[onlinehelp_documentation]') . '&hellip;', ['help_ext_url', 'http://documentation.webedition.org/', 'we_documentation'], [
						], self::$isSeemode),
					new we_gui_menuEntry(g_l('menu_global', '[onlinehelp_tagreference]') . '&hellip;', ['help_ext_url', 'https://www.webedition.org/de/dokumentation-community/tag-referenz/', 'we_tags'], [we_base_perm::WORK_TEMPLATES_CAN_SEE], self::$isSeemode),
					new we_gui_menuEntry(g_l('menu_global', '[onlinehelp_forum]') . '&hellip;', ['help_ext_url', 'https://forum.webedition.org', 'we_forum'], [], self::$isSeemode),
					new we_gui_menuEntry(g_l('menu_global', '[onlinehelp_bugtracker]') . '&hellip;', ['help_ext_url', 'https://qa.webedition.org/tracker', 'we_qa'], [], self::$isSeemode),
					new we_gui_menuEntry(g_l('menu_global', '[onlinehelp_reportBug]') . '&hellip;', ['help_ext_url', 'https://qa.webedition.org/tracker/bug_report_page.php?' . http_build_query([
							'product_version' => $verString,
							'additional_info' => 'Branch: ' . WE_VERSION_BRANCH . '
Rev: ' . WE_VERSION_BRANCH . '
PHP: ' . PHP_VERSION . '
SQL: ' . implode(' - ', we_database_base::getMysqlVer()),
							'custom_field_17' => WE_VERSION_BRANCH,
							'custom_field_16' => WE_SVNREV,
							'custom_field_7' => PHP_VERSION,
							'custom_field_8' => implode(' - ', we_database_base::getMysqlVer()),
						]), 'we_qa'], [], self::$isSeemode),
					new we_gui_menuEntry(g_l('menu_global', '[onlinehelp_changelog]') . '&hellip;', ['help_ext_url', 'https://www.webedition.org/de/dokumentation-community/versionshistorie/version-' . $verString,
						'we_changelog'], [], self::$isSeemode),
					new we_gui_menuSeparator(false, [we_base_perm::ADMINISTRATOR]),
					new we_gui_menuEntry(g_l('menu_global', '[info]') . '&hellip;', ['we_gui_infoDialog'])
		]);
	}

}
