<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0b80d828f85e $
 * $Author: Lukas Imhof $
 * $Date: 2021-11-06 23:04 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_html_multiIconBoxContentLine{
	private const CSS_CLASS = 'class';
	private const FOLD = 'fold';
	private const FOLD_NAMEDMODE = 'fold_namedMode';
	private const HIDE = 'hide';
	private const ID = 'id';
	private const NOLINE = 'noline';
	private const PRELINE = 'preline';
	private const SPACE = 'space';
	private const RIGHTHEADLINE = 'rightHeadline';

	/** @var array{class?:string,fold?:bool,hide?:bool,noline?:bool,preline?:bool,space?:string,rightHeadline?:bool,id?:string} */
	private $data = [
	];

	/** @var string */
	private $headline = '';

	/** @var string */
	private $html = '';

	/** @var string */
	private $icon = '';

	public function __construct(string $headline, string $html, string $icon = ''){
		$this->headline = $headline;
		$this->html = $html;
		$this->icon = $icon;
	}

	/**
	 * @return array{headline:string,html:string,icon:string,class?:string,fold?:bool,hide?:bool,noline?:bool,preline?:bool,space?:string,rightHeadline?:bool,fold_namedMode?:bool}
	  public function getLine(): array{
	  return $this->data;
	  } */
	public function getHeadline(): string{
		return $this->headline;
	}

	public function getHTML(): string{
		return $this->html;
	}

	public function getIcon(): string{
		return $this->icon;
	}

	public function isFold(): bool{
		return $this->data[self::FOLD] ?? false;
	}

	public function isFoldNamedMode(): bool{
		return $this->data[self::FOLD_NAMEDMODE] ?? false;
	}

	public function fold(bool $do = true, bool $namedMode = false): self{
		$this->data[self::FOLD] = $do;
		$this->data[self::FOLD_NAMEDMODE] = $namedMode;
		return $this;
	}

	public function isPreline(): bool{
		return $this->data[self::PRELINE] ?? false;
	}

	public function preline(): self{
		$this->data[self::PRELINE] = true;
		return $this;
	}

	public function isHide(): bool{
		return $this->data[self::HIDE] ?? false;
	}

	public function hide(bool $do = true): self{
		$this->data[self::HIDE] = $do;
		return $this;
	}

	public function getClass(): string{
		return $this->data[self::CSS_CLASS] ?? '';
	}

	public function class(string $class): self{
		$this->data[self::CSS_CLASS] = $class;
		return $this;
	}

	public function isNoline(): bool{
		return $this->data[self::NOLINE] ?? false;
	}

	public function noline(bool $do = true): self{
		$this->data[self::NOLINE] = $do;
		return $this;
	}

	public function getSpace(): string{
		return $this->data[self::SPACE] ?? '';
	}

	public function space(string $space): self{
		$this->data[self::SPACE] = $space;
		return $this;
	}

	public function getId(): string{
		return $this->data[self::ID] ?? '';
	}

	public function id(string $id): self{
		$this->data[self::ID] = $id;
		return $this;
	}

	public function isRightHead(): bool{
		return $this->data[self::RIGHTHEADLINE] ?? false;
	}

	public function rightHead(): self{
		$this->data[self::RIGHTHEADLINE] = true;
		return $this;
	}

}
