<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e34127dc4f4b $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-02-17 00:15 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_import_wizard {

	const TYPE_DOCUMENT = 'documents';
	const TYPE_OBJECT = 'objects';
	const INPUT_IMPORTTYPE = 'importType';
	const INPUT_SERVER_FILE = 'fserver';
	const INPUT_FILE_SOURCE = 'source';
	const INPUT_DOPUBLISH = "publish";
	const INPUT_DOSEARCHABLE = "searchable";
	const INPUT_CBDYNAMIC = "is_dynamic";
	const INPUT_EXTENSION = "we_extension";

	/**
	 * Fields from uploaded document
	 * @var array
	 */
	protected /* array */ $csvFields;

	/**
	 * Datefields defined in Template / Class
	 * @var array
	 */
	protected /* array */ $dateFields;

	/**
	 * Fields that may be imported
	 * @var array
	 */
	protected /* array */ $records;

	/** @var we_fileupload_ui_base */
	protected $fileUploader;

	/** @var we_fileupload_resp_base */
	protected $fileUploaderResp;
	protected $what = '';
	protected $step = 0;
	protected $mode = 0;
	protected /*	 * string */ $extraHead = '';

	public function __construct(string $what, int $step, int $mode){
		$this->what = $what;
		$this->step = $step;
		$this->mode = $mode;
	}

	public function getHTML(): string{
		switch($this->what) {
			default:
			case 'wizframeset':
				return $this->getWizFrameset();
			case 'wizbody':
				return $this->getWizBody();
			case 'wizbusy':
				return $this->getWizBusy();
			case 'wizcmd':
				return $this->getWizCmd();
		}
	}

	private function getWizFrameset(): string{
		$body = we_html_element::body(['id' => 'weMainBody']
										, we_html_element::iframe('wizbody', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=import&pnt=wizbody' . (($cmd1 = we_base_request::string('we_cmd', '', 1)) ? '&we_cmd[1]=' . $cmd1 : ''), 'position:absolute;top:0;bottom:40px;left:0;right:0;') .
										we_html_element::iframe('wizbusy', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=import&pnt=wizbusy&mode=' . $this->mode, 'position:absolute;height:40px;bottom:0;left:0;right:0;overflow: hidden;') .
										we_html_element::iframe('wizcmd', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=import&pnt=wizcmd', 'position:absolute;bottom:0;height:0;left:0;right:0;overflow: hidden;')
		);

		return we_html_tools::getHtmlTop(g_l('import', '[title]'), we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_base.js') . we_html_element::jsScript(JS_DIR . 'import_wizardBase.js') . $this->extraHead, $body);
	}

	protected function getWizBody(): string{
		$step0HTML = $this->getStep0();
		return we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'import_wizardWizbody.js') . $this->extraHead, we_html_element::body(['class' => 'weDialogBody'], we_html_element::form([
																'name' => we_base_constants::WE_FORM,
																'enctype' => 'multipart/form-data',
																		], we_html_element::hiddens([
																				'pnt' => 'wizbody',
																				'step' => (string) $this->step,
																				'mode' => (string) $this->mode,
																				'button_state' => '0'
																		]) . $step0HTML
														)
										)
		);
	}

	protected function getWizBusy(): string{
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('set_button_state');
		$pb = '';

		if(we_base_request::int('mode') == 1){
			$WE_PB = new we_gui_progressBar(0, 200);
			$WE_PB->addText(g_l('import', '[import_progress]'), we_gui_progressBar::TOP, 'pb1');
			$pb = we_gui_progressBar::getJSCode() . we_html_element::div(['id' => 'progress'], $WE_PB->getHTML());

// make jsCmd => we need a minimal we_cmd on this frame
			$jsCmd->addCmd('cycle');
			$jsCmd->addCmd('we_import', 1, -2, ((we_base_request::string('type') === we_exim_ExIm::TYPE_WE)));
		}

		$content = (new we_html_table(['class' => 'default', 'width' => '100%'], 1, 2))
						->setCol(0, 0, '')
						->setCol(0, 1, we_html_element::div(['id' => 'standardDiv'], we_html_button::position_yes_no_cancel(
														we_html_button::create_button(we_html_button::BACK, "javascript:WE().util.dialogTop(window).handleEvent('previous');", '', '', true, false) .
														we_html_button::create_button(we_html_button::NEXT, "javascript:WE().util.dialogTop(window).handleEvent('next');", '', '', false, false, '_btn'), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).handleEvent('cancel');", '', '', false, false))) .
						we_html_element::div(['id' => 'closeDiv', 'style' => 'display:none;'], we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).handleEvent('cancel');", '', '', false, false))
						, ['style' => 'text-align:right']
		);

		return we_html_tools::getHtmlTop('', $jsCmd->getCmds() . $pb, we_html_element::body(['class' => "editfooter",
												'style' => 'overflow:hidden;'
														], $content->getHtml()
										)
		);
	}

	private function getWizCmdInitialStep(we_base_jsCmd $jsCmd, array $v, array $records, array $we_flds, array $attrs, array $attributes): string{
		$h = $this->getHdns('v', $v);
		if($v['type'] != '' && $v['type'] != we_exim_ExIm::TYPE_WE){
			$h .= $this->getHdns('records', $records) .
							$this->getHdns('we_flds', $we_flds);
		}
		if($v['type'] == we_exim_ExIm::TYPE_XML){
			$h .= $this->getHdns('attributes', $attributes) .
							$this->getHdns('attrs', $attrs);
		}

		$jsCmd->addCmd('setProgressText_footer', 'pb1', g_l('import', '[prepare_progress]'));
		$jsCmd->addCmd('call_delayed', ['function' => 'we_import', 'delay' => 15, 'param_1' => 1, 'param_2' => -1, 'param_3' => false]);

		return we_html_element::form(['name' => we_base_constants::WE_FORM], $h);
	}

	protected function getWizCmdPrepareStep(we_base_jsCmd $jsCmd, array $v, array $records, array $we_flds, array $attrs, array $attributes): string{
		return $this->getWizCmdPrepareStepBase($jsCmd, '');
	}

	protected function getWizCmdPrepareStepBase(we_base_jsCmd $jsCmd, string $content): string{
		$jsCmd->addCmd('call_delayed', ['function' => 'we_import', 'delay' => 15, 'param_1' => 1, 'param_2' => 0, 'param_3' => false]);

		return we_html_element::form(['name' => we_base_constants::WE_FORM], $content);
	}

	protected function doImport(string $type, array $fields, int $docType, bool $IsSearchable, int $parentID, int $templateID, int $classID, string $rcd_name, bool $doPublish, string $conflictAction): bool{
		switch($type) {
			case self::TYPE_DOCUMENT:
				if(!$templateID){
					return false;
				}
				$IsSearchable = $docType ? ($IsSearchable || fB('SELECT IsSearchable FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . $docType)) : false;
				$isDynamic = \we_base_request::bool(self::INPUT_CBDYNAMIC);
				if(!we_import_functions::importDocument($parentID, $templateID, $fields, $this->dateFields, $docType, /** fime: implement categories array_filter(we_base_request::filterIntList($v['docCategories'])) */ [], $rcd_name, $isDynamic, \we_base_request::string(self::INPUT_EXTENSION), $doPublish, $IsSearchable, $conflictAction)){
					t_e('warning', 'import of entry failed', $fields);
				}
				return true;
			case self::TYPE_OBJECT:
				if(!we_import_functions::importObject($classID, $fields, []/* fixme: implement categories  array_filter(we_base_request::filterIntList($v['objCategories'])) */, $rcd_name, $doPublish, $IsSearchable, $parentID, $conflictAction)){
					t_e('warning', 'import of entry failed', $fields);
				}
				return true;
			default:
				throw new Exception('invalid import type');
		}
	}

	protected function correctDateTime(string $dateFormat, string $manualFormat, array &$fields): void{
		if(!$this->dateFields || !$dateFormat){
			return;
		}
		foreach($this->dateFields as $dateField) {
			$fields[$dateField] = we_import_functions::parseDate((string) $fields[$dateField], $manualFormat);
		}
	}

	/* protected function getWizCmdProgressStepBase(we_base_jsCmd $jsCmd, array $v, string $hiddens, array $fields): string{

	  $rcd_name = (string) ($v['pfx_fn'] == 1) ? $v['rcd_pfx'] : $v['asoc_prefix'];
	  $this->doImport($v['import_type'], $docType );


	  $jsCmd->addCmd('setProgressText_footer', 'pb1', g_l('import', '[import]'));
	  $jsCmd->addCmd('setProgress_footer', (int) (((intval($v['cid']) + 1) / intval($v['numFiles'])) * 100));
	  $jsCmd->addCmd('call_delayed', ['function' => 'we_import', 'delay' => 15, 'param_1' => 1, 'param_2' => ($v['cid'] + 1), 'param_3' => false]);

	  return we_html_element::form(['name' => we_base_constants::WE_FORM], $hiddens);
	  } */

	private function getWizCmd(string $type = 'normal'): string{
		$jsCmd = new we_base_jsCmd();
		$out = '';
		$mode = we_base_request::int('mode', 0);
		$v = we_base_request::_(we_base_request::STRING_LIST, 'v');

		if(isset($v['mode']) && $v['mode'] == 1){
			$records = we_base_request::_(we_base_request::RAW, 'records', []);
			$we_flds = we_base_request::_(we_base_request::RAW, 'we_flds', []);
			$attrs = we_base_request::_(we_base_request::RAW, 'attrs', []);
			$attributes = we_base_request::_(we_base_request::RAW, 'attributes', []);

			switch($v['cid']) {
				case -2:
					$out = $this->getWizCmdInitialStep($jsCmd, $v, $records, $we_flds, $attrs, $attributes);
					break;
				case -1:
					$out = $this->getWizCmdPrepareStep($jsCmd, $v, $records, $we_flds, $attrs, $attributes);
					break;

				case $v['numFiles']:
					self::importFinished($jsCmd, $type);
					break;
				default:
					$out = $this->getWizCmdProgressStep($jsCmd, $v, $records, $we_flds, $attributes);
					break;
			} // end switch
		} else if($mode != 1){
			$out = we_html_element::form(['id' => 'wizardBaseForm', 'name' => we_base_constants::WE_FORM], we_html_element::hiddens([
													'v[mode]' => '',
													'v[cid]' => '',
													'mode' => '',
													'type' => '',
													'cid' => '']));
		}

		return we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'import_wizardBase.js') . $jsCmd->getCmds(), we_html_element::body(['style' => 'overflow:hidden;'], $out));
	}

	private function importFinished(we_base_jsCmd $jsCmd, string $type): void{
		$jsCmd->addCmd('doOnImportFinished', ['progressText' => g_l('import', '[finish_progress]')]);

		if($type == we_exim_ExIm::TYPE_WE){
			$jsCmd->addCmd('addLog_buffered', [we_html_element::b(g_l('import', '[end_import]') . ' - ' . date(g_l('weEditorInfo', '[date_format_sec]')))]);
		} else {
			$jsCmd->addMsg(g_l('import', '[finish_import]'), we_base_util::WE_MESSAGE_NOTICE);
			$jsCmd->addCmd('call_delayed', ['function' => 'close', 'delay' => 100]);
		}
	}

	/**
	 *
	 * @param we_base_jsCmd $jsCmd
	 * @param string $obj
	 * @param int[] $categories
	 * @return string
	 */
	protected function formCategory2(we_base_jsCmd $jsCmd, string $obj, array $categories): string{
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:WE().util.dialogTop(window).we_cmd('we_selector_document',{id:0,table:'" . CATEGORY_TABLE . "',jsCmd:'add_" . $obj . "Cat'})", '', '', (!we_base_permission::hasPerm([
												we_base_perm::WORK_CATEGORY_EDIT])));
		$cats = new we_chooser_multiDirExtended($categories, 'delete_' . $obj . 'Cat', $addbut, [], we_base_ContentTypes::CATEGORY, CATEGORY_TABLE);
		$cats->setRowPrefix($obj);
		$cats->setCatField('v[' . $obj . 'Categories]');
		return $cats->get($jsCmd);
	}

	/**
	 * @return array<int,array{name:string,type:string}>
	 * @param integer $classID
	 * @desc returns an array with all the fields of the class with the given $classID
	 */
	protected static function getClassFields(int $classID): array{
		$db = new DB_WE();
//$dv = we_unserialize(fS('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($classID), '', $db));
		$tableInfo_sorted = we_contents_objectFile::getSortedTableInfo($classID, true, $db);
		$fields = [];
		$regs = [];
		foreach($tableInfo_sorted as $cur) {
// bugfix 8141
			if(preg_match('/(.+?)_(.*)/', $cur['name'], $regs)){
				$fields[] = ['name' => $regs[2], 'type' => $regs[1]];
			}
		}
		return $fields;
	}

	/**
	 * @return boolean
	 * @param string $type
	 * @desc returns true if the field is a text field
	 */
	protected static function isTextField(string $type): bool{
		switch($type) {
			case 'input':
			case 'text':
			case 'meta':
			case 'checkbox': //Bugfix #4733
				return true;
			default:
				return false;
		}
	}

	/**
	 * @return boolean
	 * @param string $type
	 * @desc returns true if the field is a text field
	 */
	protected static function isDateField(string $type): bool{
		return ($type === 'date');
	}

	/**
	 * @return boolean
	 * @param string $type
	 * @desc returns true if the field is numeric
	 */
	protected static function isNumericField(string $type): bool{
		switch($type) {
			case 'int':
			case 'float':
				return true;
			default:
				return false;
		}
	}

	/**
	 * @return string
	 * @param string $v
	 * @param array<string,string> $a
	 * @param string[] $ignore
	 * @desc returns a string of hidden fields
	 */
	protected function getHdns(string $v, array $a, array $ignore = []): string{
		$hdns = '';
		foreach($a as $key => $value) {
			if(!in_array($key, $ignore)){
				$hdns .= we_html_element::hidden($v . '[' . $key . ']', $value);
			}
		}
		return $hdns;
	}

	protected function getStep0(): string{
		$defaultVal = we_exim_ExIm::TYPE_LOCAL_FILES;

		if(!we_base_permission::hasPerm([we_base_perm::EXIM_IMPORT_FILE])){
			$defaultVal = we_exim_ExIm::TYPE_SITE;
			if(!we_base_permission::hasPerm([we_base_perm::EXIM_IMPORT_SITE])){
				$defaultVal = we_exim_ExIm::TYPE_WE;
				if(!we_base_permission::hasPerm([we_base_perm::EXIM_IMPORT_WXML])){
					$defaultVal = we_exim_ExIm::TYPE_XML;
					if(!we_base_permission::hasPerm([we_base_perm::EXIM_IMPORT_GENERICXML])){
						$defaultVal = we_exim_ExIm::TYPE_CSV;
						if(!we_base_permission::hasPerm([we_base_perm::EXIM_IMPORT_CSV])){
							$defaultVal = '';
						}
					}
				}
			}
		}

		$cmd1 = we_base_request::string('we_cmd', $defaultVal, 1);
		//$expat = (function_exists('xml_parser_create'));

		$tblFiles = (new we_html_table(['class' => 'default withSpace'], 2, 1))
						->setCol(0, 0, we_html_forms::radiobuttonV(we_exim_ExIm::TYPE_LOCAL_FILES, $cmd1, 'type', g_l('import', '[file_import]'), true, '', '', !we_base_permission::hasPerm([
																we_base_perm::EXIM_IMPORT_FILE]), g_l('import', '[txt_file_import]'), we_html_tools::TYPE_HELP))
						->setCol(1, 0, we_html_forms::radiobuttonV(we_exim_ExIm::TYPE_SITE, $cmd1, 'type', g_l('import', '[site_import]'), true, '', '', !we_base_permission::hasPerm([
												we_base_perm::EXIM_IMPORT_SITE]), g_l('import', '[txt_site_import]'), we_html_tools::TYPE_HELP));

		//$importFromLocal = we_html_tools::formElementTable('', $this->fileUploader->getHTML());
		//$importFromButton = (we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT])) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server', 'v[fserver]', 'all_Types', document.we_form.elements['v[fserver]'].value)") : '';
		//$inputLServer = we_html_tools::textInput('v[fserver]', ($v['fserver'] ?? '/'), 255, ['readonly' => 'readonly', 'onclick' => "WE().util.dialogTop(window).setFormField('v[rdofloc]', true, 'radio', 0);"], "text", '300px');
		//$importFromServer = we_html_tools::formElementTable('', $inputLServer, $importFromButton);
		//we_html_tools::getHtmlTop('', $uploader->getCss() . $uploader->getJs(), $body);

		$row = -1;
		$tblData = (new we_html_table(['class' => 'default withSpace'], 3, 1))
						//->setCol(0, 0, we_html_forms::radiobuttonV(we_exim_ExIm::TYPE_WE, $cmd1, 'type', g_l('import', '[wxml_import]'), true, '', '', (!we_base_permission::hasPerm([
						//			we_base_perm::EXIM_IMPORT_WXML]) || !$expat), g_l('import', ($expat ? '[txt_wxml_import]' : '[add_expat_support]')), we_html_tools::TYPE_HELP))
						//->setCol(1, 0, we_html_forms::radiobuttonV(we_exim_ExIm::TYPE_XML, $cmd1, 'type', g_l('import', '[gxml_import]'), true, '', '', (!we_base_permission::hasPerm([
						//			we_base_perm::EXIM_IMPORT_GENERICXML]) || !$expat), g_l('import', ($expat ? '[txt_gxml_import]' : '[add_expat_support]')), we_html_tools::TYPE_HELP))
						->setCol(++$row, 0, we_html_forms::radiobuttonV(we_exim_ExIm::TYPE_CSV, $cmd1, 'type', g_l('import', '[csv_import]'), true, '', '', !we_base_permission::hasPerm([
												we_base_perm::EXIM_IMPORT_CSV]), g_l('import', '[txt_csv_import]'), we_html_tools::TYPE_HELP));
		//->setCol(++$row, 0, g_l('import', '[fileselect_local]'))
		//->setCol(++$row, 0, $importFromLocal)
		//->setCol(++$row, 0, g_l('import', '[fileselect_server]'))
		//->setCol(++$row, 0, $importFromServer);

		$parts = (new we_html_multiIconBoxContent())
						->add((new we_html_multiIconBoxContentLine(g_l('import', '[import_file]'), $tblFiles->getHTML()))
										->space(we_html_multiIconBox::SPACE_MED)
										->noline())
						->add((new we_html_multiIconBoxContentLine(g_l('import', '[import_data]'), $tblData->getHTML()))
						->space(we_html_multiIconBox::SPACE_MED)
						->noline());
		return (new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('import', '[title]'))->get();
	}

	public static function getFrameset(): void{
		$what = we_base_request::string('pnt', 'wizframeset');
		$step = we_base_request::int('step', 0);
		$mode = we_base_request::int('mode', 0);
		$type = we_base_request::string('type', '');

		switch($type) {
			case we_exim_ExIm::TYPE_XML:
				$wizard = new we_import_wizardXML($what, $step, $mode);
				break;
			case we_exim_ExIm::TYPE_WE:
				$wizard = new we_import_wizardWE($what, $step, $mode);
				break;
			case we_exim_ExIm::TYPE_CSV:
				$wizard = new we_import_wizardCSV($what, $step, $mode);
				break;
			case we_exim_ExIm::TYPE_SITE:
				$wizard = new we_import_site($what, $step, $mode);
				break;
			case we_exim_ExIm::TYPE_LOCAL_FILES:
				$wizard = new we_import_files($what, $step, $mode);
				break;
			default:
				$wizard = new self($what, $step, $mode);
		}

		echo $wizard->getHTML();
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
								'backup_file_found' => g_l('import', '[backup_file_found]'),
								'backup_file_found_question' => g_l('import', '[backup_file_found_question]'),
								'errorEmptyDateFormat' => (g_l('siteimport', '[errorEmptyDateFormat]')),
								'format_timestamp' => g_l('import', '[format_timestamp]'),
								'invalid_path' => g_l('import', '[invalid_path]'),
								'nameOfTemplateAlert' => g_l('siteimport', '[nameOfTemplateAlert]'),
								'num_elements' => g_l('import', '[num_elements]'),
								'pleaseSelectTemplateAlert' => g_l('siteimport', '[pleaseSelectTemplateAlert]'),
								'root_dir_1' => g_l('importFiles', '[root_dir_1]'),
								'root_dir_2' => g_l('importFiles', '[root_dir_2]'),
								'root_dir_3' => g_l('importFiles', '[root_dir_3]'),
								'select_docType' => g_l('import', '[select_docType]'),
								'select_seperator' => g_l('import', '[select_seperator]'),
								'select_source_file' => g_l('import', '[select_source_file]'),
								'startEndMarkAlert' => g_l('siteimport', '[startEndMarkAlert]'),
								'we_filename_notValid' => g_l('alert', '[we_filename_notValid]'),
		]);
	}

	protected function getWizCmdProgressStep(we_base_jsCmd $jsCmd, array $v, array $records, array $we_flds, array $attributes): string{
		return '';
	}

}
