<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 52f0f1294a70 $
 * $Author: lukasimhof $
 * $Date: 2022-12-19 18:24 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_import_wizardCSV extends we_import_wizard {
	// fixme: replace $v with session or similar means

	const INPUT_DELIM = "delimitator";
	const INPUT_FIELDNAMES = "csv_fieldnames";
	const INPUT_ENCLOSE = "csv_enclosed";
	const CLASS_FIELDOPTION = 'csvFieldOption';
	const INPUT_DOCTYPE = "doctype";
	const INPUT_TEMPLATENAME = "templatename";
	const INPUT_STORE_TO_PATH = "store_to_path";
	const OBJ_PATH = "obj_path";
	const INPUT_ASOC_PREFIX = "asoc_prefix";
	const INPUT_AUTOFILENAME = "rdo_filename";
	const SELECT_FILENAME_SOURCE = "rcd_pfx";
	const INPUT_TIMEFORMAT = "rdo_timestamp";
	const INPUT_ASOC_FIELDS = "we_flds";
	const INPUT_MANUAL_TIMEFORMAT = "timestamp";
	const INPUT_CONFLICT = "nameconflict";

	protected /*we_import_CSV*/ $cp;
	// csv options from request
	private /*string*/ $delimitator;
	private /*bool*/ $hasFieldNames;
	private /*string*/ $enclosure;

	public function initCSVOptions() {
		$source = \we_base_request::string(self::INPUT_FILE_SOURCE, 'SESSION');
		$fileName = $this->handleFile($source);


		if($source !== 'SESSION') {
			$this->correctFileEncoding($fileName);
		}

		$this->delimitator = \we_base_request::string(self::INPUT_DELIM);
		if(!$this->delimitator) {
			$peek = we_base_file::loadPart($fileName);
			$nKomma = substr_count($peek, ',');
			$nSemi = substr_count($peek, ';');
			$this->delimitator = $nKomma > $nSemi ? ',' : ';';
		}
		switch(\we_base_request::string(self::INPUT_ENCLOSE)) {
			case 'single_quote':
				$this->enclosure = '\'';
				break;
			case 'none':
				$this->enclosure = '';
				break;
			case 'double_quote':
			//falls through
			default:
				$this->enclosure = '"';
		}
		$this->hasFieldNames = \we_base_request::bool(self::INPUT_FIELDNAMES, false);
	}

	protected function setFileUploader(): void {
		switch($this->step) {
			case 1:
				$this->fileUploader = new we_fileupload_ui_base('uploaded_csv_file', [], we_base_ContentTypes::getExtension(we_base_ContentTypes::TEXT));
				//$this->fileUploader->setCmd('cmdFileselectOnclick', 'fileupload_doOnFileSelect');
				$this->fileUploader->setCmd('cmdUploadFinished', 'fileupload_doOnFileSelect');
				$this->extraHead .= $this->fileUploader->getCss() . $this->fileUploader->getJs();
				break;
			case 2:
				$this->fileUploaderResp = new we_fileupload_resp_base();
				break;
		}
	}

	protected function getWizBody(): string {
		$formAttribs = [
				'name' => we_base_constants::WE_FORM,
				'enctype' => 'multipart/form-data'
		];

		$jsCmd = new we_base_jsCmd();
		$hiddens = [
				'pnt' => 'wizbody',
				'type' => we_exim_ExIm::TYPE_CSV,
				'step' => (string) $this->step,
				'mode' => (string) $this->mode,
				'button_state' => '0'
		];
		$this->setFileUploader();
		switch($this->step) {
			default:
			case 1:
				$content = $this->getStep1();
				break;
			case 2:
				$content = $this->getStep2();
				break;
			case 3:
				$content = $this->getStep3();
				break;

		}
		$doOnLoad = !we_base_request::bool('noload');
		return we_html_tools::getHtmlTop('', ($this->fileUploader ? $this->fileUploader->getCss() . $this->fileUploader->getJs() : '') .
										we_html_element::jsScript(JS_DIR . 'import_wizardWizbody.js') .
										$jsCmd->getCmds(), we_html_element::body(['class' => 'weDialogBody',
												'onload' => $doOnLoad ? "parent.wiz_next('wizbusy', WE().consts.dirs.WEBEDITION_DIR+'we_cmd.php?we_cmd[0]=import&pnt=wizbusy&mode=" . $this->mode . "&type=" . we_exim_ExIm::TYPE_CSV . "'); self.focus();" : ""
														], we_html_element::form($formAttribs, we_html_element::hiddens($hiddens) .
																		$content
														)
										)
		);
	}

	private function getWizardCSVOptions(): string {
		$iptDel = we_html_tools::textInput(self::INPUT_DELIM, ($this->delimitator ?: ';'), 2, ['class' => self::CLASS_FIELDOPTION, 'onchange' => "we_cmd('we_import_getCSVPreview')",], 'text', '30px');
		$fldDel = (new we_html_select(['name' => 'select_delim', 'class' => 'weSelect', 'onchange' => "this.form.elements['" . self::INPUT_DELIM . "'].value=this.options[this.selectedIndex].value;this.selectedIndex=options[0];we_cmd('we_import_getCSVPreview');",
								'style' => "width: 130px"
										])) // FIXME: register change listener
						->addOptions(we_exim_ExportCSV::getCSVDelimeters());

		$txtDel = (new we_html_select(['name' => self::INPUT_ENCLOSE, 'class' => 'weSelect ' . self::CLASS_FIELDOPTION, 'style' => 'width: 300px', 'onchange' => "we_cmd('we_import_getCSVPreview');",]))
						->addOptions([
								'double_quote' => g_l('import', '[double_quote]'),
								'single_quote' => g_l('import', '[single_quote]'),
								'none' => g_l('import', '[none]')
						])
						->selectOption($this->enclosure ?: 'double_quote');

		$rowDef = we_html_forms::checkbox('1', $this->hasFieldNames, self::INPUT_FIELDNAMES, g_l('import', '[contains]'), true, 'defaultfont ' . self::CLASS_FIELDOPTION, "we_cmd('we_import_getCSVPreview');");

		$csvSettings = (new we_html_table(['class' => 'default withSpace'], 4, 1))
						//->setCol(0, 0, g_l('import', '[file_format]') . we_html_element::br(2) . $charSet->getHtml())
						->setCol(1, 0, g_l('import', '[field_delimiter]') . we_html_element::br(2) . $iptDel . ' ' . $fldDel->getHtml())
						->setCol(2, 0, g_l('import', '[text_delimiter]') . we_html_element::br(2) . $txtDel->getHtml())
						->setCol(3, 0, $rowDef);
		return $csvSettings->getHTML();
	}

	private function correctFileEncoding(string $csvFile): void {
		if(!(file_exists($csvFile) && is_readable($csvFile))) {
			return;
		}
		//fixme: what if file too big?
		$data = we_base_file::load($csvFile);
		$encoding = mb_detect_encoding($data, 'UTF-8,ISO-8859-1,ISO-8859-15') ?: 'ISO-8859-15';
		$data = str_replace(["\r\n", "\r"], ["\n", "\n"], $data);
		if($encoding !== 'UTF-8') {
			$data = mb_convert_encoding(we_base_file::load($csvFile), 'UTF-8', $encoding . ',UTF-8');
		}
		if(!$data){
			throw new Exception('nothing to save');
		}
		we_base_file::save($csvFile, $data);
	}

	public function initCSVParser(string $csvFile): void {

		$this->cp = (new we_import_CSV())
						->setDelim($this->delimitator)
						->setEnclosure($this->enclosure);
		$this->cp->setFile($csvFile);
		$this->cp->setHeader($this->hasFieldNames);
		$isValid = $this->cp->parseCSV();
		if(!$isValid) {
			//throw error
			return;
		}

		// get CSV fields (+names)
		$recs = $this->cp->CSVFetchFieldNames();
		foreach($recs as $i => $val) {
			if($this->hasFieldNames && $val !== '') {
				$this->csvFields[] = $val;
			} else {
				$this->csvFields[] = g_l('import', '[record_field]') . ($i + 1);
			}
		}
	}

	private function populateDBFields(int $templateID, int $objTargetID): void {
		$this->dateFields = [];
		$this->records = [];
		if($templateID) {
			$templateCode = fS('SELECT c.Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix(TEMPLATES_TABLE) . '" AND c.DID=' . $templateID . ' AND c.nHash=unhex(md5("' . we_contents_template::FULLCODE . '"))');
			$tp = new we_weTag_parser($templateCode);

			foreach($tp->getTagsWithAttributes() as $tag) {
				if(empty($tag['attribs']['name'])) {
					continue;
				}
				switch($tag['name']) {
					case 'field':
					case 'var':
						continue 2;
					// tags with text content, links and hrefs
					case 'input':
						if(!empty($tag['attribs']['type']) && $tag['attribs']['type'] === 'date') {
							$this->dateFields[] = $tag['attribs']['name'];
						}
					case 'textarea':
					case 'href':
					case 'link':
						$this->records[] = $tag['attribs']['name'];
						break;
				}
			}
			$this->records = array_unique($this->records) ?: ['Title', 'Description', 'Keywords'];
		}
		if($objTargetID) {
			$obj = new we_contents_objectFile($objTargetID);
			$class = $obj->TableID;

			$classFields = self::getClassFields((int) $class);
			foreach($classFields as $classField) {
				if(self::isTextField($classField['type']) || self::isNumericField($classField['type']) || self::isDateField($classField['type'])) {
					$this->records[] = $classField['name'];
				}
				//dateFields have to match input for now
				if(self::isDateField($classField['type'])) {
					$this->dateFields[] = $classField['name'];
				}
			}
		}
	}

	public function hasDateField(): bool {
		return (bool) $this->dateFields;
	}

	public function tryParseCSV(int $objTargetID = 0, int $templateID = 0): string {

		$tr = new we_html_tr([], ['class' => 'boxHeader', 'style' => 'background:lightblue;']);
		foreach($this->csvFields as $field) {
			$tr->td($field);
		}

		$th = [g_l('import', '[we_flds]'), g_l('import', '[rcd_flds]')];
		$rows = [];

		$this->populateDBFields($templateID, $objTargetID);
		foreach($this->records as $record) {
			//$hdns .= we_html_element::hidden('records[' . $i . ']', $record);
			$sct_we_fields = (new we_html_select(['name' => self::INPUT_ASOC_FIELDS . '[' . $record . ']',
									'class' => "weSelect",
											]))
							->addOption('', g_l('import', '[any]'));

			foreach($this->csvFields as $value => $text) {
				'@phan-var string $value';
				$b64_value = (string) (we_base_request::bool('v', true, 'startCSV') ? $value : base64_encode($value));
				$sct_we_fields->addOption($b64_value, htmlspecialchars((string) $text));
/*
 * 				if(isset($recs[$record])) {
					if($value == base64_decode($recs[$record])) {
						$sct_we_fields->selectOption($b64_value);
					}
				} elseif($value == $record) {
					$sct_we_fields->selectOption($b64_value);
				}*/
			}

			switch($record) {
				case 'Title':
					$new_record = g_l('import', '[we_title]');
					break;
				case 'Description':
					$new_record = g_l('import', '[we_description]');
					break;
				case 'Keywords':
					$new_record = g_l('import', '[we_keywords]');
					break;
				default:
					$new_record = '';
			}
			$rows[] = [
					$new_record ?: $record,
					$sct_we_fields->getHTML()
			];
		}

		$colChooser = $rows ? we_html_tools::dialogBorder(450, $rows, $th, '') : '';

		$table = new we_html_table(['class' => 'dialogBorderBox',], 0, count($this->csvFields), [$tr]);
		$row = 0;
		while(($data = $this->cp->CSVFetchRow()) !== null && $row < 10) {
			$row++;
			$tr = new we_html_tr();
			foreach($data as $val) {
				$tr->td($val);
			}
			$table->addRow($tr);
		}
		return $table->getHtml() . $colChooser;
	}

	private function handleFile(string $source): string {
		if($source === 'SESSION') {
			return (string) $_SESSION['weS']['module_sessions']['csvImport']['fileName'];
		}
		if($source === 'lLocal') {
			try {
				$importFrom = $this->fileUploaderResp->commitUploadedFile('we_csv_' . we_fileupload::REPLACE_BY_UNIQUEID . '.csv');
			} catch(we_exception_file $e) {
				// fixme: implement some error reporting
				//$upload_error = $e->getMessage();
			}
		} else {
			$importFrom = \we_base_request::string('fserver');
			$realPath = realpath(WE_DOCUMENT_ROOT . $importFrom) ?: '';
			if(!str_contains($realPath, WE_DOCUMENT_ROOT)) {
				t_e('warning', 'Acess outside document_root forbidden!', $realPath);
			} else {
				$contents = we_base_file::load($realPath, 'r');
				$importFrom = TEMP_DIR . 'we_csv_' . we_base_file::getUniqueId() . '.csv';
				we_base_file::save(WE_DOCUMENT_ROOT . $importFrom, $contents, 'w+');
			}
		}
		/*
		$parts = new we_html_multiIconBoxContent();
		if((file_exists(WE_DOCUMENT_ROOT . $v['import_from']) && is_readable(WE_DOCUMENT_ROOT . $v['import_from']))) {

		} elseif(!file_exists(WE_DOCUMENT_ROOT . $v['import_from'])) {
			$parts
							->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('import', '[file_exists]') . WE_DOCUMENT_ROOT . $v['import_from'] . $upload_error, we_html_tools::TYPE_ALERT, 530)))
											->noline());
			$btnStateNext = 'disabled';
			$btnStateBack = 'enabled';
		} else if(!is_readable(WE_DOCUMENT_ROOT . $v['import_from'])) {
			$parts
							->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('import', '[file_readable]') . $upload_error, we_html_tools::TYPE_ALERT, 530)))
											->noline());
			$btnStateNext = 'disabled';
		}*/
		return $_SESSION['weS']['module_sessions']['csvImport']['fileName'] = WE_DOCUMENT_ROOT . $importFrom;
	}


	private function getStep1(): string {
		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', $this->getCSVWizardFileSelect()))
										->space(we_html_multiIconBox::SPACE_SMALL)
										->noline());

		return we_html_element::hiddens([
								'v[btnState_next]' => 'enabled',
								'v[btnState_back]' => 'enabled'
						]) .
										(new we_html_multiIconBox($parts, 'csv'))
										->marginLeft()
										->headline(g_l('import', '[csv_import]'))
										->get();
	}


	private function getCSVWizardFileSelect(): string{
		$this->setFileUploader();
		$file_on_server = \we_base_request::string(self::INPUT_SERVER_FILE, '/');
		$importFromButton = (we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT])) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server', '" . self::INPUT_SERVER_FILE . "', 'all_Types', document.we_form.elements['" . self::INPUT_SERVER_FILE . "'].value)") : '';
		$inputLServer = we_html_tools::textInput(self::INPUT_SERVER_FILE, $file_on_server, 255, ['readonly' => 'readonly', 'onclick' => "WE().util.dialogTop(window).setFormField('" . self::INPUT_FILE_SOURCE . "', true, 'radio', 0);"], "text", '300px');
		$importFromServer = we_html_tools::formElementTable('', $inputLServer, $importFromButton);

		$fileSource = \we_base_request::string(self::INPUT_FILE_SOURCE);
		$inputLLocal = $this->fileUploader->getHTML();
		$importFromLocal = we_html_tools::formElementTable('', $inputLLocal);
		$rdoLServer = we_html_forms::radiobuttonV('lServer', $fileSource, self::INPUT_FILE_SOURCE, g_l('import', '[fileselect_server]'));
		$rdoLLocal = we_html_forms::radiobuttonV('lLocal', $fileSource, self::INPUT_FILE_SOURCE, g_l('import', '[fileselect_local]'));
		$tblRow = 0;
		$importLocs = (new we_html_table(['class' => 'default'], 6, 1))
			->setCol($tblRow++, 0, $rdoLServer)
			->setCol($tblRow++, 0, $importFromServer)
			->setCol($tblRow++, 0, $rdoLLocal, ['style' => 'padding-top:4px;'])
			// FIXME: still need condition?
			//$importLocs->setCol($tblRow++, 0, $this->fileUploader->getHtmlAlertBoxes());
			->setCol($tblRow++, 0, $importFromLocal);
		return $importLocs->getHTML();
	}

	private function getStep2(): string {
		$this->initCSVOptions();
		$this->initCSVParser($_SESSION['weS']['module_sessions']['csvImport']['fileName']);
		$preview = $this->tryParseCSV();

		$radioDocs = we_html_forms::radiobuttonV(self::TYPE_DOCUMENT, self::TYPE_DOCUMENT, self::INPUT_IMPORTTYPE, g_l('import', '[documents]'), true, self::CLASS_FIELDOPTION, "we_cmd('showHideOptions', this.value);");
		$radioObjs = we_html_forms::radiobuttonV(self::TYPE_OBJECT, '', self::INPUT_IMPORTTYPE, g_l('import', '[objects]'), true, self::CLASS_FIELDOPTION, "we_cmd('showHideOptions', this.value);", (!we_modules_info::isActive(we_modules_info::OBJECT)));

		// search / publish / cats
		//fixme: add categories $categories = (we_modules_info::isActive(we_modules_info::CATEGORY) ? $this->formCategory2($jsCmd, 'doc', $v['docCategories'] ? we_base_request::filterIntList($v['objCategories']) : []) : '');
		$seaPu = (new we_html_table(['class' => 'default'], 2, 1))
						->setCol(1, 0, we_html_forms::checkboxWithHidden(false, self::INPUT_DOSEARCHABLE, g_l('weClass', '[IsSearchable]'), false, 'defaultfont'))
						->setCol(0, 0, we_html_forms::checkboxWithHidden(true, self::INPUT_DOPUBLISH, g_l('button', '[publish][value]'), false, 'defaultfont'));

		$weSuggest = we_gui_suggest::getInstance();
		$parts = (new we_html_multiIconBoxContent())
						->add((new we_html_multiIconBoxContentLine(	g_l('import','[csv_settings]'), $this->getWizardCSVOptions()))
										->space(we_html_multiIconBox::SPACE_MED)
										->noline()
										->fold())
						->add((new we_html_multiIconBoxContentLine(g_l('import', '[field_options]') . $radioDocs . $radioObjs, $this->getDocSettingSelector($weSuggest) . $this->getObjSettingSelector($weSuggest)))
										->space(we_html_multiIconBox::SPACE_MED)
										->noline())
						->add((new we_html_multiIconBoxContentLine(g_l('import','[settings]'), $seaPu->getHtml()))
										->space(we_html_multiIconBox::SPACE_MED)
										->noline())
						->add((new we_html_multiIconBoxContentLine(g_l('import', '[name]'), $this->getFilenameOptionsSelector(self::TYPE_DOCUMENT)))
										->space(we_html_multiIconBox::SPACE_MED)
										->noline())
						->add((new we_html_multiIconBoxContentLine(g_l('import', '[preview]'), we_html_element::div(['id' => 'previewTable'], $preview)))
										->space(we_html_multiIconBox::SPACE_MED)
										->noline())
						->add((new we_html_multiIconBoxContentLine(g_l('import', '[format_date]'), $this->getDateFormatSelector()))
										->space(we_html_multiIconBox::SPACE_MED))
						->add((new we_html_multiIconBoxContentLine(g_l('import', '[name_collision]'), $this->getNameConflictSelector()))
						->space(we_html_multiIconBox::SPACE_MED));

		return we_html_element::hiddens([
								'v[btnState_next]' => 'enabled',
								'v[btnState_back]' => 'enabled'
						]) .
										(new we_html_multiIconBox($parts, 'csv'))
										->marginLeft()
										->headline(g_l('import', '[csv_import]'))
										->get();
	}

	private function getDocSettingSelector(we_gui_suggest $weSuggest, int $docType = -1, int $templateID = 0): string {
		$optid = 0;
		$DTselect = (new we_html_select(['name' => self::INPUT_DOCTYPE,
								'class' => 'weSelect',
								'onchange' => "WE().util.dialogTop(window).setFormField('doctypeChanged', 1, 'hidden'); WE().util.dialogTop(window).weChangeDocType(this);",
								'style' => 'width: 300px'
										]))
						->insertOption($optid, '-1', g_l('import', '[none]'));

		$dts = DB_WE::$inst->getAllFirstq('SELECT dt.ID,dt.Text FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.IsFolder=0 ORDER BY dt.Text', false);
		'@phan-var array<int,string> $dts';
		foreach($dts as $id => $dt) {
			$optid++;
			$DTselect->insertOption($optid, (string) $id, $dt);
			if($docType == $id) {
				$DTselect->selectOption($id);
			}
		}

		$doctypeElement = we_html_tools::formElementTable(g_l('import', '[doctype]'), $DTselect->getHTML());

		/*		 * * templateElement *************************************************** */
		/* $ueberschrift = (we_base_permission::hasPerm([we_base_perm::CAN_SEE_TEMPLATES]) ?
		  '<a href="javascript:goTemplate(document.we_form.elements[\'' . $idname . '\'].value)">' . g_l('import', '[template]') . '</a>' :
		  g_l('import', '[template]')); */



		$TPLselect = new we_html_select([
				'name' => 'docTypeTemplateId',
				'class' => 'weSelect',
				'style' => "width: 300px"
		]);

		if($docType !== -1) {
			$displayDocType = $displayNoDocType = '';
			$foo = fS('SELECT Templates FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.ID=' . $docType);
			$TPLselect->addOptions(id2pathA(we_base_request::filterIntList($foo), TEMPLATES_TABLE));
			if($templateID) {
				$TPLselect->selectOption($templateID);
			}
		} else {
			$displayDocType = 'display:none';
			$displayNoDocType = 'display:block';
		}
		$weSuggest->setAcId('TmplPath')
						->setContentType([we_base_ContentTypes::FOLDER_TEMPLATE, we_base_ContentTypes::TEMPLATE])
						->setInput(self::INPUT_TEMPLATENAME, '/', ['class' => self::CLASS_FIELDOPTION])
						->setMaxResults(10)
						->setJsCommandOnItemSelect('we_import_getCSVPreview')
						->setResult('noDocTypeTemplateId', (string) $templateID)
						->setSelector(we_gui_suggest::DocSelector)
						->setTable(TEMPLATES_TABLE)
						->setWidth(300)
						->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:top.wizbody.document.we_form.elements.noDocTypeTemplateId.value,table:WE().consts.tables.TEMPLATES_TABLE,targetID:'noDocTypeTemplateId',targetName:'" . self::INPUT_TEMPLATENAME . "',jsCmd:'reload_editpage',filter:WE().consts.contentTypes.TEMPLATE,open_doc:true})"))
						->setLabel(g_l('import', '[template]'));

		$templateElement = "<div id='docTypeLayer' style='" . $displayDocType . "'>" . we_html_tools::formElementTable(g_l('import', '[template]'), $TPLselect->getHTML()) . "</div>
<div id='noDocTypeLayer' style='" . $displayNoDocType . "'>" . $weSuggest->getHTML() . '</div>';

		$weSuggest->setAcId('DirPath')
						->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT])
						->setInput(self::INPUT_STORE_TO_PATH, (/*$v['store_to_path'] ??*/ '/'))
						->setMaxResults(10)
						->setRequired(true)
						->setResult(self::INPUT_STORE_TO_PATH . '_id', (string) (/*$v['store_to_id'] ??*/ 0))
						->setSelector(we_gui_suggest::DirSelector)
						->setWidth(300)
						->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:WE().util.dialogTop(window).wizbody.document.we_form.elements['" . self::INPUT_STORE_TO_PATH . '_id' . "'].value,table:'" . FILE_TABLE . "',targetID:'" . self::INPUT_STORE_TO_PATH . '_id' . "',targetName:'" . self::INPUT_STORE_TO_PATH . "',rootDirID:0})"))
						->setLabel(g_l('import', '[import_dir]'));

		$storeTo = $weSuggest->getHTML();

		$specifyDoc = (new we_html_table(['class' => 'default'], 1, 2))
						->setCol(0, 1, we_html_forms::checkbox('3',(/*$v['is_dynamic'] ?? */false), self::INPUT_CBDYNAMIC, g_l('import', '[isDynamic]'), true, "defaultfont"), [
								'style' => 'vertical-align:bottom;'])
						->setCol(0, 0, we_html_tools::formElementTable(g_l('import', '[extension]'), we_html_tools::getExtensionPopup(self::INPUT_EXTENSION, (/*$v['we_Extension'] ??*/ '.html'), we_base_ContentTypes::getExtension(we_base_ContentTypes::WEDOCUMENT), 100)), [
				'style' => 'padding-right:20px;']);

		$row = 0;
		$table = (new we_html_table(['class' => 'default', 'id' => 'docSelTable',], 4, 1))
						->setCol(++$row, 0, $doctypeElement)
						->setCol(++$row, 0, $templateElement)
						->setCol(++$row, 0, $storeTo)
						->setCol(++$row, 0, $specifyDoc->getHtml());

		return $table->getHtml();
	}

	private function getObjSettingSelector(we_gui_suggest $weSuggest): string {
		$firstID = 0;
		$first = '/';
		if(we_modules_info::isActive(we_modules_info::OBJECT)) {

			$weSuggest->setAcId('ObjPath')
							->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE])
							->setInput(self::OBJ_PATH,  ($first), ['class' => self::CLASS_FIELDOPTION,])
							->setMaxResults(10)
							->setJsCommandOnItemSelect('we_import_getCSVPreview')
							->setRequired(true)
							->setResult(self::OBJ_PATH . '_id', (string) (($firstID)))
							->setSelector(we_gui_suggest::DirSelector)
							->setTable(OBJECT_FILES_TABLE)
							->setWidth(300)
							->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements['" . self::OBJ_PATH . "'].value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'" . self::OBJ_PATH . '_id' . "',targetName:'" . self::OBJ_PATH . "'})")) //must be jsCmd: document.we_form.elements['v[classID]'].value.split('_')[1]
							->setLabel(g_l('import', '[import_dir]'));

			$objStoreTo = $weSuggest->getHTML();
		} else {
			$objStoreTo = '';
		}
		$row = -1;
		$table = (new we_html_table(['class' => 'default', 'style' => 'display:none;', 'id' => 'objSelTable',], 1, 1))
						->setCol(++$row, 0, $objStoreTo);

		return $table->getHtml();
	}

	private function getStep3(): string {
		$this->initCSVOptions();
		$this->initCSVParser($_SESSION['weS']['module_sessions']['csvImport']['fileName']);
		$type = \we_base_request::string(self::INPUT_IMPORTTYPE);

		$countImported = $this->handleImport($type);

		$msg = '';
		if(!$countImported){
			$msg = g_l('modules_messaging','[error_occured]');
		}
		$parts = (new we_html_multiIconBoxContent())
						->add((new we_html_multiIconBoxContentLine(g_l('modules_customer','[search_result]'), ($msg ?: g_l('import', '[finished_success]') . ' ' . g_l('modules_glossary','[number_of_entries]') . ': ' . $countImported)))
										->space(we_html_multiIconBox::SPACE_MED)
										->noline());
		return (new we_html_multiIconBox($parts, 'csv'))
										->marginLeft()
										->headline(g_l('import', '[csv_import]'))
										->get();
	}

	private function getFilenameOptionsSelector(string $importType): string {
		// Associated prefix selector.
		$asocPfx = (new we_html_table(['class' => 'default'], 1, 1))
						->setCol(0, 0, g_l('import', '[pfx]') . we_html_element::br(2) .
						we_html_tools::textInput(self::INPUT_ASOC_PREFIX, (/*$v['asoc_prefix'] ?? */g_l('import', ($importType === self::TYPE_DOCUMENT ? '[pfx_doc]' : '[pfx_obj]'))), 255, [
								'onclick' => "WE().util.dialogTop(window).setFormField('" . self::INPUT_AUTOFILENAME . "', true, 'radio', 0);"], 'text', '150px'));

		// Assigned record or attribute field selectors.
		$rcdPfxSelect = (new we_html_select(['name' => self::SELECT_FILENAME_SOURCE,
								'class' => "weSelect",
								'onclick' => "WE().util.dialogTop(window).setFormField('" . self::INPUT_AUTOFILENAME . "', true, 'radio', 1);",
								'style' => "width: 150px"
										]))
						->addOptions($this->csvFields)
						->selectOption(we_base_request::string(self::SELECT_FILENAME_SOURCE));

		$asgndFld = (new we_html_table(['class' => 'default'], 1, 1))
						->setCol(0, 0, g_l('import', '[rcd_fld]') . we_html_element::br(2) . $rcdPfxSelect->getHTML());

		// Filename selector.
		$fn = (new we_html_table(['class' => 'default'], 5, 1))
						->setCol(0, 0, we_html_forms::radiobutton('0', /*(!isset($v['rdo_filename']) ? true : ($v['rdo_filename'] == 0))*/ true, self::INPUT_AUTOFILENAME, g_l('import', '[auto]'), true, '', "self.document.we_form.elements['v[pfx_fn]'].value=0;"), [
								'colspan' => '2'])
						->setCol(1, 0, $asocPfx->getHTML(), ['style' => 'padding-left:25px;'])
						->setColAttributes(2, 0, ['height' => 5])
						->setCol(3, 0, we_html_forms::radiobutton('1', /*(!isset($v['rdo_filename']) ? false : ($v['rdo_filename'] == 1))*/ false, self::INPUT_AUTOFILENAME, g_l('import', '[asgnd]'), true, '', "self.document.we_form.elements['v[pfx_fn]'].value=1;"), [
								'colspan' => '2'])
						->setCol(4, 0, $asgndFld->getHTML(), ['style' => 'padding-left:25px;']);
		return $fn->getHtml();
	}

	private function getNameConflictSelector(): string {
		$conflict = we_exim_ExIm::COLLISION_RENAME;
		$fn_colsn = (new we_html_table(['class' => 'default withSpace'], 3, 1))
						->setCol(0, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_RENAME, $conflict, self::INPUT_CONFLICT, g_l('import', '[rename]'), true, 'defaultfont'))
						->setCol(1, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_REPLACE, $conflict, self::INPUT_CONFLICT, g_l('import', '[replace]'), true, 'defaultfont'))
						->setCol(2, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_SKIP, $conflict, self::INPUT_CONFLICT, g_l('import', '[skip]'), true, 'defaultfont'));

		return $fn_colsn->getHTML();
	}

	private function getDateFormatSelector(): string {
		$visible = (bool) $this->dateFields;
		// Timestamp
		$tStamp = (new we_html_table(['class' => 'default withSpace', 'id' => 'dateFormatTable', 'style' => 'display:' . ($visible ? 'block' : 'none')], 4, 1))
						#->setCol(0, 0, we_html_forms::radiobuttonV('Unix', /*$v['rdo_timestamp'] ?? */'', self::INPUT_TIMEFORMAT, g_l('import', '[uts]'), true, '', '', false, g_l('import', '[unix_timestamp]'), 0, 384), [
						#		'colspan' => '2'])
						->setCol(1, 0, we_html_forms::radiobuttonV('GMT', /*$v['rdo_timestamp'] ?? */'', self::INPUT_TIMEFORMAT, g_l('import', '[gts]'), true, '', '', false, g_l('import', '[gmt_timestamp]'), 0, 384), [
								'colspan' => '2']);
						#->setCol(2, 0, we_html_forms::radiobuttonV('Format', /*$v['rdo_timestamp'] ?? */'', self::INPUT_TIMEFORMAT, g_l('import', '[fts]'), true, '', '', false, g_l('import', '[format_timestamp]'), 0, 384), [
						#		'colspan' => '2'])
						#->setCol(3, 0, we_html_tools::textInput(self::INPUT_MANUAL_TIMEFORMAT, (/*$v['timestamp'] ?? */''), 0, ['placeholder' => 'Y-m-d H:i:s', 'onclick' => "WE().util.dialogTop(window).setFormField('" . self::INPUT_TIMEFORMAT . "', true, 'radio', 2);"], 'text', '150px'), [
				#'style' => 'padding-left:25px;']);

		return $tStamp->getHtml();
	}

	private function handleImport(string $importType): int {
		//fixme: what would be the non-deprecated approach?
		$we_flds = \we_base_request::intList(self::INPUT_ASOC_FIELDS);
		$pfx_fn = \we_base_request::bool(self::INPUT_AUTOFILENAME);
		//name from field
		$rcd_pfx = \we_base_request::string(self::SELECT_FILENAME_SOURCE);
		$collisionAction = \we_base_request::string(self::INPUT_CONFLICT);
		$recs = $fields = [];

		$doPublish = \we_base_request::bool(self::INPUT_DOPUBLISH);
		$IsSearchable = \we_base_request::bool(self::INPUT_DOSEARCHABLE);
		$parentID = \we_base_request::int(self::INPUT_STORE_TO_PATH . '_id');
		$templateID = \we_base_request::int('noDocTypeTemplateId') ?: \we_base_request::int('docTypeTemplateId');

		$obj = new we_contents_objectFile(\we_base_request::int(self::OBJ_PATH . '_id'));
		$classID = $obj->TableID;

		$dateFormat = \we_base_request::string(self::INPUT_TIMEFORMAT, 'GMT');
		$manualFormat = \we_base_request::string(self::INPUT_MANUAL_TIMEFORMAT);

		$this->populateDBFields($templateID, \we_base_request::int(self::OBJ_PATH . '_id'));

		// fixme: Error
		if(!$this->csvFields || !$we_flds){
			return 0;
		}

		// create an associative lookup array
		foreach($we_flds as $dbField => $csvFieldIndex) {
			/* if($csvField === 0) {
			  continue;
			  } */
			if(isset($this->csvFields[$csvFieldIndex])) {
				$recs[$dbField] = $csvFieldIndex;
			}
		}

		$entry = 0;
		$countImported = 0;
		while($entry < 15000 && ($data = $this->cp->CSVFetchRow())) {
			++$entry;

			$fields = [];
			foreach($recs as $dbColumnName => $csvColumn) {
				$fields[$dbColumnName] = $data[$csvColumn];
			}


			$rcdName = $this->getName($pfx_fn, $rcd_pfx, $importType, $recs);

			$this->correctDateTime($dateFormat, $manualFormat, $fields);

			$countImported += (int) $this->doImport($importType, $fields, $entry, $IsSearchable, $parentID ?: $obj->ID, $templateID, $classID, $rcdName ?: 'obj', $doPublish, $collisionAction);
		}
		return $countImported;
	}
	/**
	 *
	 * @param bool $pfx_fn Prefix-Mode
	 * @param string $rcd_pfx Record Prefix: CSV Field name
	 * @return string
	 */
	private function getName(bool $pfx_fn, string $rcd_pfx, string $importType, array $recs):string{
		if(!$pfx_fn) {
			return \we_base_request::string(self::INPUT_ASOC_PREFIX);
		}
		$newName = $recs[$rcd_pfx] ?: g_l('import', ($importType === self::TYPE_DOCUMENT ? '[pfx_doc]' : '[pfx_obj]'));
		return (string) $newName;
	}


}
