<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3b1bcdab4c4d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-11-23 23:14 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_listview_shopItems extends we_listview_objectBase{
	/** @var string */
	var $type;

	/** @var we_shop_Basket */
	private $ShoppingCart;

	/** @var array<int,array{id:int,type:string,variant:string,quantity:float,serial:array,customFields:array}> */
	private $ShoppingCartItems;

	/** @var string */
	private $ShoppingCartKey = '';

	/** @var array{id:int,type:string,variant:string,quantity:float,serial:array,customFields:array} */
	var $ActItem;

	/** @var string[] */
	var $IDs = [];

	public function __construct(we_shop_Basket $shoppingCart){
		parent::__construct();
		$this->ShoppingCart = $shoppingCart;
		$this->ShoppingCartItems = $shoppingCart->getShoppingItems();

		$this->IDs = array_keys($this->ShoppingCartItems);
		$this->anz = count($this->IDs);
	}

	public function next_record(): bool{
		if($this->count >= $this->anz){
			return false;
		}

		$cartKey = $this->IDs[$this->count];
		$shoppingCartItems = $this->ShoppingCartItems[$cartKey];
		$this->ShoppingCartKey = $cartKey;
		$this->ActItem = $shoppingCartItems;

		$this->Record = [];
		foreach($shoppingCartItems['serial'] as $key => $value){
			$this->Record[mb_strtolower((string) $key)] = $value;
		}
		$this->count++;
		return true;
	}

	public function f(string $key, bool $stripWE = false): string{
		return parent::f($key, $stripWE);
	}

	public function getTableID(): int{
		switch($this->getActItemField('type')){
			default:
			case we_shop_shop::DOCUMENT:
				return 0;
			case we_shop_shop::OBJECT:
				return intval($this->f(we_listview_base::PROPPREFIX . 'tableid'));
		}
	}

	private function getCustomFieldsAsRequest(): array{
		return $this->ActItem['customFields'] ?
			[we_shop_shop::ARTICLE_CUSTOM_FIELD => $this->ActItem['customFields']] :
			[];
	}

	function getActItemField(string $field): string{
		return (string) $this->ActItem[$field];
	}

	public function getShoppingCartKey(): string{
		return $this->ShoppingCartKey;
	}

	/**
	 *
	 * @return array{id:int,type:string,req:array}
	 */
	public function getShopArticleID(): array{
		return ['id' => (int) $this->getActItemField('id'), 'type' => $this->getActItemField('type'), 'req' => $this->getCustomFieldsAsRequest()];
	}

	public function getHrefArray(string $name): array{
		switch($this->getActItemField('type')){
			case we_shop_shop::DOCUMENT:
				return [
					'int' => (int) $this->f($name . we_base_link::MAGIC_INT_LINK),
					'intID' => (int) $this->f($name . we_base_link::MAGIC_INT_LINK_ID),
					'extPath' => $this->f($name)
				];
			case we_shop_shop::OBJECT:
				$ret = we_unserialize($this->f($name));
				assert(is_array($ret));
				return $ret;
			default:
				return [];
		}
	}

}
