<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3b1bcdab4c4d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-11-23 23:14 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * @desc    class for tag <we:listview type="variant">
 *
 */
class we_listview_variants extends we_listview_base{
	/** @var array */
	private $VariantData = [];

	/** @var int */
	var $Position = 0;

	/** @var int */
	public $id = 0;

	/** @var int */
	var $ObjectId = 0;

	/** @var string */
	var $DefaultName = 'default';

	/** @var we_contents_objectFile|we_document_webEdition */
	var $Model;

	/** @var bool */
	var $IsObjectFile = false;

	/** @var bool */
	var $hidedirindex = false;

	/** @var bool */
	var $objectseourls = false;

	/** @var string */
	private $firstKey = '';

	function __construct(string $name, int $rows, string $defaultname, int $documentid, int $objectid, int $offset, bool $hidedirindex, bool $objectseourls){
		parent::__construct($name, $rows, $offset);

		$this->DefaultName = $defaultname;
		$this->hidedirindex = $hidedirindex;
		$this->objectseourls = $objectseourls;
		try{
			$obj = we_base_WE::getWeObj();
		} catch (we_exception_base $e){
			$obj = null;
		}

		// we have to init a new document and look for the given field
		// get id of given document and check if it is a document or an objectfile
		if($documentid){
			$this->id = $documentid;
			$this->Model = new we_document_webEdition($this->id);
		} else if($objectid && we_modules_info::isActive(we_modules_info::OBJECT)){
			$this->IsObjectFile = true;
			$this->id = $objectid;
			$this->Model = new we_contents_objectFile($this->id);
			// check if its a document or a objectFile
		} elseif(we_base_WE::$we_doc instanceof we_document_triggerDocument){ // is an objectFile can this happen??!
			$this->Model = we_base_WE::getWeDoc();
			$this->id = $this->Model->ID;
			$this->IsObjectFile = true;
		} elseif($obj){
			$this->id = $obj->ID;
			$this->IsObjectFile = true;
			$this->Model = $obj;
		} else {
			$this->Model = we_base_WE::getWeDoc();
			$this->id = $this->Model->ID;
		}

		// store model in listview object
		$this->VariantData['Record'] = we_base_variants::getVariantData($this->Model, $this->DefaultName);
		$this->anz_all = count($this->VariantData['Record']);
		$this->anz = min($this->rows, $this->anz_all);
	}

	private function updateObjectPath(array $path_parts, string $varUrl): void{
		if($this->objectseourls && show_SeoLinks()){
			$Url = fS('SELECT Url FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $this->id);
			if($Url != ''){
				$this->Record[self::PATH_FIELD] = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') .
					( $this->hidedirindex && seoIndexHide($path_parts['basename']) ?
					'' : '/' . $path_parts['filename']
					) . '/' . $Url . ($varUrl ? "?$varUrl" : '');
			} else {
				$this->Record[self::PATH_FIELD] = ($this->hidedirindex && seoIndexHide($path_parts['basename']) ?
					($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . '?' . WEOBJ_PARAM . '=' . $this->id . ($varUrl ? "&amp;$varUrl" : '') :
					we_base_WE::getWeDoc()->Path . '?' . WEOBJ_PARAM . '=' . $this->id . ($varUrl ? '&amp;' . $varUrl : '')
					);
			}
		} elseif($this->hidedirindex && seoIndexHide($path_parts['basename'])){
			$this->Record[self::PATH_FIELD] = we_base_WE::getWeDoc()->Path = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . ($varUrl ? "?$varUrl" : '');
		} else {
			$this->Record[self::PATH_FIELD] = we_base_WE::getWeDoc()->Path . '?' . WEOBJ_PARAM . '=' . $this->id . ($varUrl ? "&amp;$varUrl" : '');
		}
	}

	public function next_record(): bool{
		$this->Position = ($this->count + $this->start);
		if(!isset($this->VariantData['Record'][$this->Position])){
			$this->Record = [];
			return false;
		}
		$this->Record = $this->VariantData['Record'][$this->Position];

		$vardata = reset($this->Record);
		$key = key($this->Record);
		$this->firstKey = $key;
		foreach($vardata as $name => $value){

			$this->Record[mb_strtolower($name)] = (isset($value['type']) && $value['type'] === 'img' ?
				// there is a difference between objects and webEdition Documents
				($value[we_contents_base::ETYPE_NUMBER] ?? $value[we_contents_base::ETYPE_TEXT]) :
				($value[we_contents_base::ETYPE_TEXT] ?? '')
				);
		}

		$this->Record[self::VARIANT_FIELD . '_name'] = $key;

		if($key != $this->DefaultName){
			$varUrl = we_base_variants::REQUEST . '=' . $key;
			$this->Record[self::VARIANT_FIELD] = $key;
		} else {
			$varUrl = $this->Record[self::VARIANT_FIELD] = '';
		}

		$this->Record[self::ID_FIELD] = $this->id;
		$path_parts = pathinfo(/* $this->IsObjectFile ? we_base_WE::$we_doc->Path : */$this->Model->Path);
		if($this->IsObjectFile){ // objectFile
			$this->updateObjectPath($path_parts, $varUrl);
		} else // webEdition Document

		if($this->hidedirindex && seoIndexHide($path_parts['basename'])){
			$this->Record[self::PATH_FIELD] = $this->Model->Path = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . ($varUrl ? '?' . $varUrl : '');
		} else {
			$this->Record[self::PATH_FIELD] = $this->Model->Path . ($varUrl ? '?' . $varUrl : '');
		}

		$this->count++;
		return true;
	}

	public function f(string $key, bool $stripWE = false): string{
		return parent::f($key, $stripWE);
	}

	function getFirstKey(): string{
		return (string) $this->firstKey;
	}

	/**
	 *
	 * @return array{id:int,type:string,req:array}
	 */
	public function getShopArticleID(): array{
		return $this->Model->getShopArticleID();
	}

	public function getDID(): int{
		return $this->id;
	}

	public function useSeoUrls(): bool{
		return $this->objectseourls;
	}

}
