<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0ea8c0c04490 $
 * $Author: lukasimhof $
 * $Date: 2023-01-16 12:20 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_navigation_view extends we_modules_view{
	/** @var we_navigation_navigation */
	public $Model;

	public function __construct(){
		parent::__construct(we_modules_info::NAVIGATION);
		$this->page = intval(we_navigation_frames::TAB_PROPERTIES);
		$this->Model = new we_navigation_navigation();
	}

	function getJSTop(): string{
		return parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigation_view.js', "parent.document.title='" . $this->getModTitle() . "'");
	}

	function getJSProperty(array $jsVars = []): string{
		$objFields = [];
		if(we_modules_info::isActive(we_modules_info::OBJECT) && $this->Model->DynamicSelection === we_navigation_navigation::DYN_CLASS){
			$class = new we_contents_object();
			$class->initByID($this->Model->ClassID);
			$fields = $class->getAllVariantFields();

			foreach(array_keys($fields) as $key){
				[, $tmp] = explode('_', $key, 2);
				$objFields[] = '"' . $tmp . '": "' . $key . '"';
			}
		}

		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigation_view_prop.js', '', ['id' => 'loadVarViewProp', 'data-prop' => setDynamicVar(array_merge(
						$jsVars, [
					'weNavTitleField' => $objFields,
					'IsFolder' => intval($this->Model->IsFolder),
					'selfNaviPath' => $this->Model->Path,
					'selfNaviId' => $this->Model->ID,
		]))]);
	}

	function getEditNaviPosition(): array{
		$values = $this->db->getAllFirstq('SELECT Ordn,Text FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . $this->Model->ParentID . ' ORDER BY Ordn', false);
		$values[-1] = g_l('navigation', '[end]');
		return $values;
	}

	function processCommands(we_base_jsCmd $jscmd): void{
		switch(we_base_request::string('cmd')){
			case 'we_navigation_new':
			case 'we_navigation_new_group':
				if(!we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT])){
					$jscmd->addMsg(g_l('navigation', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$this->Model = new we_navigation_navigation();
				$this->Model->IsFolder = we_base_request::string('cmd') === 'we_navigation_new_group';
				$this->Model->ParentID = we_base_request::int('ParentID', 0);
				$jscmd->addCmd('editLoad', $this->Model->Text);
				break;
			case 'we_navigation_edit':
				if(!we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT])){
					$jscmd->addMsg(g_l('navigation', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$this->Model = new we_navigation_navigation(we_base_request::int('cmdid'));

				if(!$this->Model->isAllowedForUser()){
					$jscmd->addMsg(g_l('navigation', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					$this->Model = new we_navigation_navigation();
					$_REQUEST['home'] = true;
					break;
				}
				$jscmd->addCmd('editLoad', $this->Model->Text, $this->Model->ID);
				break;
			case 'we_navigation_save':
				if(!we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT])){
					$jscmd->addMsg(g_l('navigation', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if(we_navigation_navigation::filenameNotValid($this->Model->Text)){
					$jscmd->addMsg(g_l('navigation', '[wrongtext]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if(!trim($this->Model->Text)){
					$jscmd->addMsg(g_l('navigation', '[name_empty]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$oldpath = $this->Model->Path;
				// set the path and check it
				$this->Model->setPath();
				if($this->Model->pathExists($this->Model->Path)){
					$jscmd->addMsg(g_l('navigation', '[name_exists]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if($this->Model->isSelf() || !$this->Model->isAllowedForUser()){
					$jscmd->addMsg(g_l('navigation', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if($this->Model->DynamicSelection === we_navigation_navigation::DYN_CLASS && !$this->Model->TitleField){
					$classFields = we_unserialize(fS('SELECT DefaultValues FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($this->Model->ClassID)));
					if(is_array($classFields) && !empty($classFields)){
						$fieldsByNamePart = [];
						foreach(array_keys($classFields) as $key){
							if(($pos = strpos((string) ($key), '_')) && (substr((string) $key, 0, $pos) != 'object')){
								$fieldsByNamePart[substr((string) $key, $pos + 1)] = $key;
							}
						}
						if(!key_exists($this->Model->TitleField, $fieldsByNamePart) && !key_exists($this->Model->TitleField, $classFields)){
							$jscmd->addMsg(g_l('navigation', '[wrongTitleField]'), we_base_util::WE_MESSAGE_ERROR);
							break;
						}
					} else {
						$jscmd->addMsg(g_l('navigation', '[wrongTitleField]'), we_base_util::WE_MESSAGE_ERROR);
						break;
					}
				}

				$newone = $this->Model->ID == 0;

				$this->Model->save();

				if($this->Model->IsFolder && $oldpath != '' && $oldpath != '/' && $oldpath != $this->Model->Path){
					$this->db->query('UPDATE ' . NAVIGATION_TABLE . ' SET
Path=REPLACE(Path,"' . $this->db->escape($oldpath) . '/","' . $this->db->escape($this->Model->Path) . '/")
WHERE
	Path LIKE "' . $this->db->escape($oldpath) . '/%" AND
	ID!=' . intval($this->Model->ID), true);
				}

				if($newone){
					$jscmd->addCmd('makeTreeEntry', [
						'id' => $this->Model->ID,
						'parentid' => $this->Model->ParentID,
						'text' => $this->Model->Text,
						'open' => false,
						'contenttype' => ($this->Model->IsFolder ? we_base_ContentTypes::FOLDER_COMMON : we_base_ContentTypes::NAVIGATION),
						'table' => NAVIGATION_TABLE,
						'published' => 1,
						'order' => $this->Model->Ordn
					]);
				} else {
					$jscmd->addCmd('updateTreeEntry', [
						'id' => $this->Model->ID,
						'parentid' => $this->Model->ParentID,
						'text' => $this->Model->Text,
						'order' => $this->Model->Ordn,
						'tooltip' => $this->Model->ID
					]);
				}

				if($this->Model->IsFolder){
					switch($this->Model->Selection){
						case we_navigation_navigation::SELECTION_DYNAMIC:

							$old_items = [];
							if($this->Model->hasDynChilds()){
								$old_items = $this->Model->depopulateGroup();
								foreach($old_items as $id){
									$jscmd->addCmd('deleteTreeEntry', $id['ID']);
								}
							}
							$items = $this->Model->populateGroup($old_items);
							foreach($items as $k => $item){
								$jscmd->addCmd('makeTreeEntry', [
									'id' => $item['id'],
									'parentid' => $this->Model->ID,
									'text' => $item['text'],
									'open' => false,
									'contenttype' => we_base_ContentTypes::NAVIGATION,
									'table' => NAVIGATION_TABLE,
									'published' => 1,
									'order' => $k
								]);
							}
							break;
						case we_navigation_navigation::SELECTION_STATIC:
							if($this->Model->hasDynChilds()){
								$old_items = $this->Model->depopulateGroup();
								foreach($old_items as $id){
									$jscmd->addCmd('deleteTreeEntry', $id['ID']);
								}
							}
							break;
					}
				}
				$delaycmd = we_base_request::string('delayCmd');

				$jscmd->addMsg(g_l('navigation', ($this->Model->IsFolder == 1 ? '[save_group_ok]' : '[save_ok]')), we_base_util::WE_MESSAGE_NOTICE);
				$jscmd->addCmd('saveReload', $this->Model->IsFolder == 1);
				if($delaycmd){
					$jscmd->addCmd('we_cmd', $delaycmd);
					unset($_REQUEST['delayCmd']);
				}
				break;
			case 'we_navigation_delete':
				if(!$this->Model->delete()){
					$jscmd->addMsg($this->Model->errMsg, we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$jscmd->addCmd('loadHome');
				$jscmd->addCmd('deleteTreeEntry', $this->Model->ID);
				$jscmd->addMsg(g_l('navigation', ($this->Model->IsFolder == 1 ? '[group_deleted]' : '[navigation_deleted]')), we_base_util::WE_MESSAGE_NOTICE);
				$this->Model = new we_navigation_navigation();
				break;
			case 'switchPage':
				break;
			case 'move_abs':
				if($this->Model->reorderAbs(we_base_request::int('pos'))){
					$posVals = $this->getEditNaviPosition();
					$posText = '';
					foreach($posVals as $val => $text){
						$posText .= '<option value="' . $val . '"' . ($val == $this->Model->Ordn ? ' selected="selected"' : '') . '>' . $text . '</option>';
					}
					$jscmd->addCmd('we_cmd', ['moveAbs', $this->Model->Ordn, $this->Model->ParentID, $posText]);
				}
				break;
			case 'move_up' :
				if($this->Model->reorderUp()){
					$posVals = $this->getEditNaviPosition();
					$posText = '';
					foreach($posVals as $val => $text){
						$posText .= '<option value="' . $val . '"' . ($val == $this->Model->Ordn ? ' selected="selected"' : '') . '>' . $text . '</option>';
					}
					$jscmd->addCmd('we_cmd', ['moveUp', $this->Model->Ordn, $this->Model->ParentID, $posText]);
				}
				break;
			case 'move_down' :
				if($this->Model->reorderDown()){
					$posVals = $this->getEditNaviPosition();
					$posText = '';
					foreach($posVals as $val => $text){
						$posText .= '<option value="' . $val . '"' . ($val == $this->Model->Ordn ? ' selected="selected"' : '') . '>' . $text . '</option>';
					}
					$jscmd->addCmd('we_cmd', ['moveDown', $this->Model->Ordn, $this->Model->ParentID, $posText]);
				}
				break;
			case 'populate':
				$items = $this->Model->populateGroup();
				foreach($items as $k => $item){
					$jscmd->addCmd('deleteTreeEntry', $item['id']);
					$jscmd->addCmd('makeTreeEntry', [
						'id' => $item['id'],
						'parentid' => $this->Model->ID,
						'text' => $item['text'],
						'open' => false,
						'contenttype' => we_base_ContentTypes::NAVIGATION,
						'table' => NAVIGATION_TABLE,
						'published' => 1,
						'order' => $k
					]);
				}
				$jscmd->addMsg(g_l('navigation', '[populate_msg]'), we_base_util::WE_MESSAGE_NOTICE);
				break;
			case 'depopulate':
				$items = $this->Model->depopulateGroup();
				foreach($items as $id){
					$jscmd->addCmd('deleteTreeEntry', $id['ID']);
				}
				$jscmd->addMsg(g_l('navigation', '[depopulate_msg]'), we_base_util::WE_MESSAGE_NOTICE);
				$this->Model->Selection = we_navigation_navigation::SELECTION_STATIC;
				$this->Model->saveField('Selection');
				break;
			case 'create_template':
				$jscmd->addCmd('we_cmd', ['new', TEMPLATES_TABLE, '', we_base_ContentTypes::TEMPLATE, '', base64_encode($this->Model->previewCode)]);
				break;
			case 'populateFolderWs':
				$values = we_navigation_dynList::getWorkspacesForObject($this->Model->LinkID);

				if($values){
					$jscmd->addCmd('we_cmd', ['doPopulateFolderWs', 'values', $values]);
				} elseif(we_navigation_dynList::getWorkspaceFlag($this->Model->LinkID)){
					$jscmd->addCmd('we_cmd', ['doPopulateFolderWs', 'workspace']);
				} else {
					$jscmd->addCmd('we_cmd', ['doPopulateFolderWs', "noWorkspace"]);
				}

				break;
			case 'populateWorkspaces':
				/* $objFields = '';
				  if($this->Model->DynamicSelection === we_navigation_navigation::DYN_CLASS){
				  if(we_modules_info::isActive(we_modules_info::OBJECT)){

				  $class = new we_contents_object();
				  $class->initByID($this->Model->ClassID);
				  $fields = $class->getAllVariantFields();
				  foreach(array_keys($fields) as $key){
				  list(, $tmp) = explode('_', $key, 2);
				  $objFields .= 'top.content.editor.edbody.weNavTitleField["' . $tmp . '"]="' . $key . '";';
				  }
				  }
				  } */

				if($this->Model->Selection === we_navigation_navigation::SELECTION_DYNAMIC){
					$values = we_navigation_dynList::getWorkspacesForClass($this->Model->ClassID);
					$prefix = 'Class';
				} else {
					$values = we_navigation_dynList::getWorkspacesForObject($this->Model->LinkID);
					$prefix = '';
				}

				if($values){ // if the class has workspaces
					$jscmd->addCmd('doPopulateWs', 'values', $prefix, $values);
				} elseif(we_navigation_dynList::getWorkspaceFlag($this->Model->LinkID)){ // if the class has no workspaces
					$jscmd->addCmd('populateWorkspaces', 'workspace', $prefix);
				} else {
					$jscmd->addCmd('populateWorkspaces', 'noWorkspace', $prefix);
				}
				break;
			case 'populateText':
				if(!$this->Model->Text && $this->Model->Selection === we_navigation_navigation::SELECTION_STATIC && $this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK){
					$cat = new we_category_category();
					$cat->load($this->Model->LinkID);
					$jscmd->addCmd('setTitle', $cat->Title);
				}
				break;

			default:
		}

		$_SESSION['weS']['module_sessions']['navigation'] = $this->Model;
	}

	function processVariables(): void{
		if(!empty($_SESSION['weS']['module_sessions']['navigation'])){
			$this->Model = $_SESSION['weS']['module_sessions']['navigation'];
		}

		$categoryPaths = [];

		if(($name = we_base_request::string('CategoriesControl'))){
			for($i = 0; ($cat = we_base_request::string($name . '_variant0_' . $name . '_item' . $i)); $i++){
				$categoryPaths[] = $cat;
			}
			$this->Model->CategoryPaths = $categoryPaths;
		}

		if(($field = we_base_request::string('SortField'))){
			$this->Model->Sort = [[
				'field' => $field,
				'order' => we_base_request::string('SortOrder')
			]];
		} else {
			$this->Model->Sort = [];
		}

		foreach($this->Model->persistent_slots as $key => $type){
			$type = (in_array($type, [we_base_request::JSON, we_base_request::SERIALIZED_JSON], true) ? we_base_request::STRING : $type);

			if(!in_array(($val = we_base_request::_($type, $key, '-1')), ['-1', -1], true)){//FIXME: check if we really must consider int -1
				$this->Model->$key = $val;
			}
		}

		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$this->Model->customerFilter = $this->Model->customerFilter ?: new we_navigation_customerFilter($this->Model->ID);
			$this->Model->customerFilter->getCustomerFilterFromRequest($this->Model->ID);
			$this->Model->UseDocumentFilter = $this->Model->customerFilter->getUseDocumentFilter();
		}


		if($this->Model->Selection === we_navigation_navigation::SELECTION_DYNAMIC){
			if(($wid = we_base_request::int('WorkspaceIDClass', -1)) > -1){
				$this->Model->WorkspaceID = $wid;
			}

			if(($par = we_base_request::_(we_base_request::STRING, 'dynamic_Parameter')) !== we_base_request::TYPE_DEFAULT){
				$this->Model->Parameter = $par;
			}

			if($this->Model->DynamicSelection === we_navigation_navigation::DYN_CATEGORY && ($url = we_base_request::_(we_base_request::URL, 'dynamic_Url', we_base_request::NOT_VALID_S)) !== we_base_request::NOT_VALID_S){
				$this->Model->Url = $url;
				$this->Model->UrlID = we_base_request::int('dynamic_UrlID', 0);
				$this->Model->LinkSelection = we_base_request::string('dynamic_LinkSelection');
				$this->Model->CatParameter = we_base_request::string('dynamic_CatParameter');
			}
		}

		if(($code = we_base_request::_(we_base_request::RAW_CHECKED, 'previewCode'))){
			$this->Model->previewCode = $code;
		}

		if(($page = we_base_request::int('page', -1)) > -1){
			$this->page = ($this->Model->IsFolder && $page != we_navigation_frames::TAB_PROPERTIES && $page != we_navigation_frames::TAB_CUSTOMER ? 1 : $page);
		}
	}

	public function getPage(): string{
		return (string) ($this->page);
	}

	function getItems(int $id): array{
		return DB_WE::$inst->getAllFirstq('SELECT ID,Text FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . $id . ' AND Depended=1 ORDER BY Ordn', false);
	}

	public function getHomeScreen(we_base_jsCmd $jscmd): string{
		$hiddens = ['cmd' => 'home'];

		$content = we_html_button::create_button('new_item', "javascript:we_cmd('we_navigation_new');", '', "", !we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT])) . '<br/>' .
			we_html_button::create_button('new_folder', "javascript:we_cmd('we_navigation_new_group');", '', "", !we_base_permission::hasPerm([we_base_perm::NAVIGATION_EDIT]));

		return parent::getActualHomeScreen(we_modules_info::NAVIGATION, $jscmd, $content, we_html_element::form(['name' => we_base_constants::WE_FORM], $this->getCommonHiddens($hiddens) . we_html_element::hidden('home', '0')));
	}

}
