<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 22fdfd9bca91 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-12-02 12:31 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_selectorSuggest extends we_rpc_cmd{

	/** @var we_database_base */
	private $db;

	/** @var string[] */
	var $fields = ['ID', 'Path'];

	/** @var array<int,array<string,string>> */
	var $condition = [];

	function execute(): we_rpc_response{
		$this->db = DB_WE::$inst;
		$query = we_base_request::_(we_base_request::FILE, 'we_cmd', '', 'query');
		$table = we_base_request::_(we_base_request::TABLE, 'we_cmd', false, 'table');
		$id = we_base_request::_(we_base_request::STRING, 'we_cmd', '', 'query');
		$doctypes = we_base_request::intList('we_cmd', [], 'documentTypes');
		$query = $query ?: (preg_match('|#(?:\d+)$|', $id) ? $id : '');
		if(!$query || !$table){
			exit();
		}

		$contentTypes = explode(',', we_base_request::string('we_cmd', '', 'contenttypes'));
		$currentDocumentType = we_base_request::string('we_cmd', '', 'currentDocumentType');
		$currentDocumentID = we_base_request::int('we_cmd', 0, 'currentDocumentID');
		if($currentDocumentType && $currentDocumentID){
			if($table === TEMPLATES_TABLE && $currentDocumentType === we_base_ContentTypes::TEMPLATE){
				$this->addCondition('AND', '!=', 'ID', (string) $currentDocumentID);
			}
		}
		$suggests = $this->search($query, $table, $contentTypes, we_base_request::int('we_cmd', 0, 'max'), we_base_request::_(we_base_request::FILE, 'we_cmd', '', 'basedir'), $doctypes);

		$ret = [];

		foreach($suggests as $sug){
			//$isFolder = !empty($sug['IsFolder']) || in_array($sug['ContentType'], we_base_ContentTypes::FOLDER_TYPES);
			'@phan-var array<string,mixed> $sug';
			$short = strrchr($sug['Path'], '/');
			$ret[] = [
				'ID' => $sug['ID'],
				'label' => ($short !== $sug['Path'] ? '/...' : '') . $short,
				'value' => $sug['Path'],
				'contenttype' => ($sug['ContentType'] ?? (empty($sug['IsFolder']) ? '' : we_base_ContentTypes::FOLDER_COMMON))
			];
		}

		$this->resp->setData('suggest', $ret);

		return $this->resp;
	}

	/**
	 * Die Funktion 'search' führt die Suche nach den Anfangszeichen für den übergebenen Selektor-Typ durch. Mit dem Parameter 'limit' kann man die Anzahl der Suchergebnisse begrenzen.
	 *
	 * @param string $search
	 * @param string $table
	 * @param string[] $types
	 * @param int $limit
	 * @param string $rootDir
	 * @param int[] $doctypes
	 *
	 * @return array<string,mixed>
	 */
	private function search(string $search, string $table, array $types, int $limit, string $rootDir, array $doctypes): array{
		$search = strtr($search, [
			'[' => '\\\[',
			']' => '\\\]',
			'"' => ''
		]);
		$userExtraSQL = $this->getUserExtraQuery($table);
		switch($table){
			case USER_TABLE:
				$this->fields[] = 'IsFolder';
				$typeField = 'Type';
				$this->fields[] = $typeField . ' AS ContentType';
				$folderType = [we_users_user::TYPE_USER_GROUP];
				break;
			case CUSTOMER_TABLE :
				$typeField = 'ContentType';
				$this->fields[] = $typeField;
				$folderType = [we_base_ContentTypes::FOLDER_COMMON];
				$userExtraSQL = '';
				break;
			case VFILE_TABLE:
			case CATEGORY_TABLE:
			case NEWSLETTER_TABLE:
			case NAVIGATION_TABLE:
				$typeField = '';
				$folderType = [];
				break;
			default:
				$typeField = 'ContentType';
				$this->fields[] = $typeField;
				$folderType = we_base_ContentTypes::FOLDER_TYPES;
		}

		$rootOnly = $rootDir && ($search === '/' || str_starts_with($rootDir, $search));
		$matches = [];
		$where = (preg_match('|#(\d+)$|', $search, $matches) ?
				'ID=' . $matches[1] : ($rootOnly ?
				'Path LIKE "' . $rootDir . '"' :
				'Path REGEXP "^' . strtr(preg_quote($search), ['\\' => '\\\\']) . '[^/]*$"' . (
				($rootDir !== '/' ?
				' AND (Path LIKE "' . $this->db->escape($rootDir) . '" OR Path LIKE "' . $this->db->escape($rootDir) . '/%")' :
				''))));

		$isFolder = false;

		$q = [];
		if($typeField && $types){
			$addCT = false;
			foreach(array_unique($types) as $type){
				$type = str_replace(' ', '', $type);
				if(in_array($type, we_base_ContentTypes::FOLDER_TYPES)){
					$q[] = '"' . implode('","', $folderType) . '"';
				} else {
					$q[] = '"' . $this->db->escape($type) . '"';
					$isFolder = false;
					$addCT = true;
				}
			}
			if($addCT){
				$this->fields[] = $typeField;
			}
		}

		if(!$q){
			$isFolder = true;
		}

		switch($table){
			case CATEGORY_TABLE:
				$isFolder = false;
				break;
			case FILE_TABLE:
				if($doctypes){
					$where .= ' AND (IsFolder=1 OR DocType IN (' . implode(',', $doctypes) . '))';
				}
		}

		$where .= ($q ? ' AND (' . $typeField . ' IN(' . implode(',', $q) . '))' : '') .
				$userExtraSQL .
				($table === NAVIGATION_TABLE ?
				//FIXME: what about other workspacequeries?!
				we_navigation_navigation::getWSQuery() :
				''
				);

		foreach($this->condition as $val){
			$where .= ' ' . $val['queryOperator'] . ' ' . $val['field'] . $val['conditionOperator'] . '"' . $this->db->escape($val['value']) . '"';
		}

		return $this->db->getAllq('SELECT ' . implode(', ', $this->fields) . ' FROM ' . $this->db->escape($table) . ' WHERE ' . $where . ' ORDER BY ' . ($isFolder ? 'isFolder DESC, Path' : 'Path') . ' ASC ' . ($limit ? ' LIMIT ' . $limit : ''));
	}

	/**
	 * Fügt die übergeben Abfragebedingung hinzu.
	 */
	private function addCondition(string $qop, string $op, string $field, string $value): void{
		$this->condition[] = [
			'queryOperator' => $qop,
			'conditionOperator' => $op,
			'field' => $field,
			'value' => $value
		];
	}

	/**
	 * getUserExtraQuery
	 * Erzeugt ein Bedingungen zur Filterung der Arbeitsbereiche
	 * @param string $table
	 * @return string
	 */
	private function getUserExtraQuery(string $table): string{
		switch($table){
			case NAVIGATION_TABLE:
			case CATEGORY_TABLE:
				return '';
		}
		$userExtraSQL = ' AND((1 ' . we_users_util::makeOwnersSql(false) . ') ';

		if(we_users_util::get_ws($table)){
			$userExtraSQL .= we_selector_file::getWsQuery($table);
		} else if($table === OBJECT_FILES_TABLE && (!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]))){
			$ac = we_users_util::getAllowedClasses();
			if($ac){
				$userExtraSQL .= ' AND TableID IN(' . implode('|', $ac) . ')';
			}
		} else {
			switch($table){
				case (we_modules_info::isActive(we_modules_info::NEWSLETTER) ? NEWSLETTER_TABLE : 'NEWSLETTER_TABLE'):
				case USER_TABLE:
					break;
				default:
					$userExtraSQL .= ' OR RestrictOwners=0 ';
			}
		}
		return $userExtraSQL . ')';
	}

}
