<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3b8044c13b9c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-12-16 00:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_search_pagination{
	/** @var string */
	public $setview = 'list';

	/** @var int */
	public $mode = 0;

	/** @var string */
	public $order = '';

	/** @var int */
	public $searchstart = 0;

	/** @var int */
	public $number = 10;

	/** @var bool */
	public $disableBack = false;

	/** @var bool */
	public $disableNext = false;

	/** @var string */
	public $text = '';

	/** @var int */
	public $page = 1;

	/** @var array<int,int> */
	public $pages = [];

	/** @var array<int,int> */
	public $pageValue = [];

	/** @var array<int,int> */
	public $pageText = [];

	/**
	 *
	 * @param string $setview
	 */
	public function setSetview(string $setview): self{
		$this->setview = $setview;
		return $this;
	}

	/**
	 *
	 * @param int $mode
	 */
	public function setMode(int $mode): self{
		$this->mode = $mode;
		return $this;
	}

	/**
	 *
	 * @param string $order
	 */
	public function setOrder(string $order): self{
		$this->order = $order;
		return $this;
	}

	/**
	 *
	 * @param int $searchstart
	 */
	public function setSearchstart(int $searchstart): self{
		$this->searchstart = $searchstart;
		return $this;
	}

	/**
	 *
	 * @param int $number
	 */
	public function setNumber(int $number): self{
		$this->number = $number;
		return $this;
	}

	/**
	 *
	 * @param bool $disableBack
	 */
	public function setDisableBack(bool $disableBack): self{
		$this->disableBack = $disableBack;
		return $this;
	}

	/**
	 *
	 * @param bool $disableNext
	 */
	public function setDisableNext(bool $disableNext): self{
		$this->disableNext = $disableNext;
		return $this;
	}

	/**
	 *
	 * @param string $text
	 */
	public function setText(string $text): self{
		$this->text = $text;
		return $this;
	}

	/**
	 *
	 * @param int $page
	 */
	public function setPage(int $page): self{
		$this->page = $page;
		return $this;
	}

	/**
	 *
	 * @param array<int,int> $pages
	 */
	public function setPages(array $pages): self{
		$this->pages = $pages;
		$this->pageValue = array_keys($pages);
		$this->pageText = array_values($pages);
		return $this;
	}

	/**
	 *
	 * @param array<int,int> $pageValue
	 */
	public function setPageValue(array $pageValue): self{
		$this->pageValue = $pageValue;
		return $this;
	}

	/**
	 *
	 * @param array<int,int> $pageText
	 */
	public function setPageText(array $pageText): self{
		$this->pageText = $pageText;
		return $this;
	}

	public function getHtml(string $whichSearch): string{
		$btnBack = we_html_button::create_button(we_html_button::BACK, "javascript:weSearch.back();", '', '', $this->disableBack, true, '', false, '', 'btnSearchBack');
		$btnNext = we_html_button::create_button(we_html_button::NEXT, "javascript:weSearch.next();", '', '', $this->disableNext, true, '', false, '', 'btnSearchNext');
		$select = we_html_tools::select('page', $this->pages, 1, $this->page, false, ['onchange' => "this.form.elements.searchstart" . $whichSearch . ".value = this.value; weSearch.search(false);"], true, 0, 'selectSearchPages');

		$tbl = (new we_html_table(['style' => 'margin-top:-2px;'], 1, 4))
			->setCol(0, 0, $btnBack)
			->setCol(0, 1, we_html_element::span(['class' => 'spanSearchText bold'], $this->text), ['class' => 'defaultfont', 'style' => 'padding-top:6px;'])
			->setCol(0, 2, $btnNext)
			->setCol(0, 3, $select);

		return $tbl->getHtml();
	}

}
