<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f860e3d063da $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-12-22 22:11 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_search_view extends we_modules_view{

	const VIEW_LIST = 'list'; // FIXME: change to VIEW_PROPS
	const VIEW_FIELDS = 'fields'; // FIXME: change to VIEW_PROPS
	const VIEW_ICONS = 'icons';
	const SEARCH_DOCS = 'DocSearch';
	const SEARCH_MEDIA = 'MediaSearch';
	const SEARCH_TMPL = 'TmplSearch';
	const SEARCH_ADV = 'AdvSearch';
	const SEARCH_CLASS = 'classSearch';
	const SEARCH_VERSION = 'VersionSearch';
	const RESULT_SIZE = [10 => 10, 25 => 25, 50 => 50, 100 => 100, 500 => 500, 1000 => 1000, 5000 => 5000, 10000 => 10000, 50000 => 50000, 100000 => 100000];

	/** @var we_search_modelBase */
	public $Model;

	/** @var we_search_search */
	protected $searchclass;

	/** @var int */
	private $searchMediaOptFieldIndex = 0;

	public function __construct(we_search_modelBase $model){
		parent::__construct(we_modules_info::SEARCH);
		$this->page = 1;
		$this->Model = $model;
		$this->searchclass = $this->searchclass ?: new we_search_search($this);
	}

	function getJSTop(): string{
		return parent::getJSTop() .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'search/search_view2.js', "parent.document.title='" . $this->getModTitle() . "'");
	}

	/**
	 * @param string $cmd
	 * @return array{int,int}
	 */
	private function getDefaultTabTables(string $cmd): array{
		switch($cmd){
			default:
			case 'weSearch_new_forDocuments' :
				return [we_search_search::TABLE_FILE, we_search_search::TAB_DOCUMENTS];
			case 'weSearch_new_forTemplates' :
				return [we_search_search::TABLE_TEMPLATES, we_search_search::TAB_TEMPLATES];
			case 'weSearch_new_forMedia' :
				return [we_search_search::TABLE_FILE, we_search_search::TAB_MEDIA];
			case 'weSearch_new_forObjects' :
				return [we_search_search::TABLE_OBJECTFILES, we_search_search::TAB_ADVANCED];
			case 'weSearch_new_forClasses' :
				return [we_search_search::TABLE_OBJECT, we_search_search::TAB_ADVANCED];
			case 'weSearch_new_forVersions' :
				return [we_search_search::TABLE_VERSION, we_search_search::TAB_ADVANCED];
			case 'weSearch_new_advSearch' :
			case 'weSearch_new' :
				return [0, we_base_request::int('tab', we_search_search::TAB_ADVANCED)];
		}
	}

	public function processCommands(we_base_jsCmd $jscmd): void{
		$cmdid = we_base_request::int('cmdid');
		switch(($cmd = we_base_request::string('cmd', we_base_request::string('we_cmd', '', 0)))){
			case 'search_submit':
				$this->searchSubmit($jscmd, we_base_WE::$transaction);
				break;
			case 'weSearch_new' :
			case 'weSearch_new_forDocuments' :
			case 'weSearch_new_forTemplates' :
			case 'weSearch_new_forObjects' :
			case 'weSearch_new_forClasses' :
			case 'weSearch_new_forMedia' :
			case 'weSearch_new_advSearch' :
				assert($this->Model instanceof we_search_model);
				[$tables, $tab] = $this->getDefaultTabTables($cmd);
				$keyword = we_base_request::string('keyword');
				$this->Model->setPredefinedSearch($tab, $keyword, $tables);
				$this->Model->prepareModelForSearch();
				$this->Model->setIsFolder(false);
				if(!$this->Model->isAllowedForUser()){
					$jscmd->addMsg(g_l('tools', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					$this->Model = new we_search_model();
					$_REQUEST['home'] = true;
					break;
				}
				$jscmd->addCmd('loadHeaderFooter', $tab, $this->Model->Text, $cmdid, $this->Model->ID);
				break;
			case 'we_search_edit' : // get model from db
				//FIXME: is this used anymore???
				$this->Model = new we_search_model($cmdid);
				$this->Model->prepareModelForSearch();
				$tab = we_base_request::int('tab', 3);
				if(!$this->Model->isAllowedForUser()){
					$jscmd->addMsg(g_l('tools', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					$this->Model = new we_search_model();
					$_REQUEST['home'] = true;
					break;
				}
				$_SESSION['weS']['module_sessions']['search'] = $this->Model;
				$jscmd->addCmd('loadHeaderFooter', $tab, $this->Model->Text, $cmdid, $this->Model->ID);
				break;
			case 'weSearch_save' :
				$this->Model->Text = we_base_request::string('savedSearchName', $this->Model->Text);
				if(strlen($this->Model->Text) > 30){
					$jscmd->addMsg(g_l('searchtool', '[nameTooLong]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				if(stristr($this->Model->Text, "'") || stristr($this->Model->Text, '"')){
					$jscmd->addMsg(g_l('searchtool', '[no_hochkomma]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				if($this->Model->filenameNotValid($this->Model->Text)){
					$jscmd->addMsg(g_l('tools', '[wrongtext]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$this->Model->activTab = we_base_request::int('tabnr', 1); // TODO: have activeTab always active (initByHttp)!!

				if(!trim($this->Model->Text)){
					$jscmd->addMsg(g_l('tools', '[name_empty]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				$oldpath = $this->Model->Path;
				// set the path and check it
				$this->Model->setPath();
				if($this->Model->pathExists($this->Model->Path)){
					$jscmd->addMsg(g_l('tools', '[name_exists]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}
				if($this->Model->isSelf()){
					$jscmd->addMsg(g_l('tools', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$newone = $this->Model->ID === 0;

				if($this->Model->save()){
					$this->Model->updateChildPaths($oldpath);

					if($newone){
						$jscmd->addCmd('makeTreeEntry', [
							'id' => $this->Model->ID,
							'parentid' => $this->Model->ParentID,
							'text' => $this->Model->Text,
							'open' => false,
							'contenttype' => ($this->Model->IsFolder ? we_base_ContentTypes::FOLDER_DOCUMENT : 'we/search'),
							'table' => SEARCH_TABLE,
							'published' => 0
						]);
					} else {
						$jscmd->addCmd('updateTreeEntry', [
							'id' => $this->Model->ID,
							'parentid' => $this->Model->ParentID,
							'text' => $this->Model->Text,
							'order' => 0,
							'tooltip' => $this->Model->ID
						]);
					}
					$jscmd->addCmd('loadHeaderFooter', $this->Model->activTab, $this->Model->Text, '', $this->Model->ID);
					$jscmd->addMsg(g_l('searchtool', ($this->Model->IsFolder == 1 ? '[save_group_ok]' : '[save_ok]')), we_base_util::WE_MESSAGE_NOTICE);

					if(($delay = we_base_request::string('delayCmd'))){
						$jscmd->addCmd('we_cmd', $delay);
						unset($_REQUEST['delayCmd']);
					}
				} else {
					$jscmd->addCmd('loadHeaderFooter', $this->Model->activTab, $this->Model->Text, '', $this->Model->ID);
					$jscmd->addMsg(g_l('searchtool', ($this->Model->IsFolder == 1 ? '[save_group_failed]' : '[save_failed]')), we_base_util::WE_MESSAGE_ERROR);
				}

				break;
			case 'weSearch_delete':
				if($this->Model->delete()){
					$jscmd->addCmd('deleteTreeEntry', $this->Model->ID);
					$jscmd->addMsg(g_l('tools', ($this->Model->IsFolder == 1 ? '[group_deleted]' : '[item_deleted]')), we_base_util::WE_MESSAGE_NOTICE);
					$jscmd->addCmd('we_search_edit');
					$this->Model = new we_search_model();
					$_REQUEST['pnt'] = 'edbody';
				}
				break;
			default:
		}

		$_SESSION['weS']['module_sessions']['search'] = $this->Model;
	}

	private function searchSubmit(we_base_jsCmd $jscmd, string $we_transaction): void{
		//FIXME: some vars are not set and werent before move

		$we_doc = we_contents_root::getSessionDocument($we_transaction);
		assert($we_doc instanceof we_folder_objectFile);

		$we_doc->searchclass->objsearch = we_base_request::_(we_base_request::RAW, 'objsearch', '');
		$we_doc->searchclass->objsearchField = we_base_request::_(we_base_request::RAW, 'objsearchField', '');
		$we_doc->searchclass->objlocation = we_base_request::_(we_base_request::RAW, 'objlocation', 0);

		switch(we_base_request::string('todo')){
			case 'add':
				$we_doc->searchclass->height++;
				$we_doc->searchclass->searchname = $we_doc->searchclass->objsearch;
				$we_doc->searchclass->searchfield = $we_doc->searchclass->objsearchField;
				$we_doc->searchclass->searchlocation = $we_doc->searchclass->objlocation;
				$we_doc->searchclass->start = 0;
				$we_doc->searchclass->searchstart = 0;
				$we_doc->searchclass->setLimit();
				$we_doc->saveInSession($we_transaction);
				$jscmd->addCmd('we_cmd', ['switch_edit_page', $we_doc->EditPageNr]);
				return;
			case 'delete':
				$we_doc->searchclass->removeFilter(we_base_request::int('position'));
				$we_doc->searchclass->searchname = $we_doc->searchclass->objsearch;
				$we_doc->searchclass->searchfield = $we_doc->searchclass->objsearchField;
				$we_doc->searchclass->searchlocation = $we_doc->searchclass->objlocation;
				$we_doc->searchclass->start = 0;
				$we_doc->searchclass->searchstart = 0;
				$we_doc->searchclass->setLimit();
				$we_doc->saveInSession($we_transaction);
				$jscmd->addCmd('we_cmd', ['switch_edit_page', $we_doc->EditPageNr]);
				return;
			case 'search':
				$we_doc->searchclass->searchname = $we_doc->searchclass->objsearch;
				$we_doc->searchclass->searchfield = $we_doc->searchclass->objsearchField;
				$we_doc->searchclass->searchlocation = $we_doc->searchclass->objlocation;
				$we_doc->searchclass->start = 0;
				$we_doc->searchclass->searchstart = 0;
				$we_doc->searchclass->setLimit();
				$we_doc->SearchStart = 0;
				$we_doc->saveInSession($we_transaction);
				$jscmd->addCmd('we_cmd', ['switch_edit_page', $we_doc->EditPageNr]);
				return;
			case 'changemeta':
				$we_doc->saveInSession($we_transaction);
				$jscmd->addCmd('we_cmd', ['reload_editpage']);
				return;
			case 'changedate':
				$we_doc->saveInSession($we_transaction);
				$jscmd->addCmd('we_cmd', ['reload_editpage']);
				return;
			case 'changecheckbox':
				$we_doc->saveInSession($we_transaction);
				$jscmd->addCmd('we_cmd', ['reload_editpage']);
				return;
			default:
				$objsf = we_base_request::string('obj_searchField', '');
				if($objsf === ''){
					return;
				}
				$we_doc->searchclass->height = 0;
				$we_doc->searchclass->show = 'AB';
				$obj_search = we_base_request::string('obj_search');
				$we_doc->searchclass->searchname = [0 => $obj_search];
				$we_doc->searchclass->searchfield = [0 => $objsf];
				$we_doc->searchclass->searchlocation = [0 => we_search_base::OP_CONTAIN];
				$we_doc->searchclass->start = 0;
				$we_doc->searchclass->searchstart = 0;
				$we_doc->searchclass->setLimit();

				$we_doc->searchclass->objsearch = [0 => $obj_search];
				$we_doc->searchclass->objsearchField = [0 => $objsf];
				$we_doc->searchclass->objlocation = [0 => we_search_base::OP_CONTAIN];

				$we_doc->saveInSession($we_transaction);

				$jscmd->addCmd('setStart', 0);
				$jscmd->addCmd('we_cmd', ['switch_edit_page', we_base_constants::WE_EDITPAGE_FIELDS]);
		}
	}

	function getSearchJS(string $whichSearch = ''): string{
		// FIXME: take actualTab from model
		$tab = we_base_request::int('tab', we_base_request::int('tabnr', 1));

		//$showSelects = '';
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'search/search_view.js', '', ['id' => 'loadVarSearch_view', 'data-searchConf' => setDynamicVar([
						'conf' => [
							'whichSearch' => $whichSearch,
							'autostart' => $whichSearch === we_search_view::SEARCH_MEDIA,
							'editorBody' => 'top.content.editor.edbody',
							'tab' => $tab,
							'modelID' => $this->Model->ID,
							'modelIsFolder' => ($this->Model->IsFolder ? 1 : 0),
							'showSelects' => 0, //($showSelects ? 1 : 0),
							'rows' => ($whichSearch === self::SEARCH_MEDIA ? $this->searchMediaOptFieldIndex : (count($this->Model->getProperty('currentSearchFields')) - ($whichSearch === self::SEARCH_ADV ? 1 : 0))),
							//rows: ' . (count($this->Model->getProperty('currentSearchFields')) - ($whichSearch == self::SEARCH_ADV ? 1 : 0) /* : ($whichSearch == self::SEARCH_MEDIA ? $this->searchMediaOptFieldIndex : (count($this->Model->getProperty('currentSearchFields')) - ($whichSearch == self::SEARCH_ADV ? 1 : 0))) */) . ',
							'we_transaction' => we_base_WE::$transaction,
						],
						'elems' => [
							'btnTrash' => we_html_button::create_button(we_html_button::TRASH, "javascript:weSearch.delRow(__we_new_id__)"),
							'btnSelector' => we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('__we_selector__', {id:document.we_form.elements['search" . $whichSearch . "ParentID[__we_new_id__]'].value, table:'__we_sel_table__', targetID:'search" . $whichSearch . "ParentID[__we_new_id__]', targetName:'search" . $whichSearch . "[__we_new_id__]', filter:'__we_content_types__'});"),
							'fieldSearch' => we_html_tools::textInput('search' . $whichSearch . '[__we_new_id__]', '', 0, ['id' => 'search' . $whichSearch . '[__we_new_id__]'], 'text', '170px'),
							'fieldDate' => we_html_tools::textInput('search' . $whichSearch . '[__we_new_id__]', '', 0, ['id' => 'search' . $whichSearch . '[__we_new_id__]', 'required' => 'required'], 'date', '170px'),
							'selStatus' => we_html_tools::select('search' . $whichSearch . '[__we_new_id__]', $this->searchclass->getFieldsStatus(), 1, "", false, ['class' => "defaultfont",
								'style' => "width:170px;", 'id' => "search" . $whichSearch . '[__we_new_id__]']),
							'selSpeicherart' => we_html_tools::select('search' . $whichSearch . '[__we_new_id__]', $this->searchclass->getFieldsSpeicherart(), 1, "", false, ['class' => "defaultfont",
								'style' => "width:170px;", 'id' => "search" . $whichSearch . '[__we_new_id__]']),
							'selLocation' => we_html_tools::select('location' . $whichSearch . '[__we_new_id__]', we_search_search::getLocation(), 1, "", false, ['class' => "defaultfont",
								'style' => 'width:150px', 'id' => 'location' . $whichSearch . '[__we_new_id__]']),
							'selLocationDate' => we_html_tools::select('location' . $whichSearch . '[__we_new_id__]', we_search_search::getLocation('date'), 1, "", false, ['class' => "defaultfont",
								'style' => 'width:150px', 'id' => 'location' . $whichSearch . '[__we_new_id__]']),
							'selLocationText' => we_html_tools::select('location' . $whichSearch . '[__we_new_id__]', we_search_search::getLocation('text'), 1, "", false, ['class' => "defaultfont",
								'style' => 'width:150px', 'id' => 'location' . $whichSearch . '[__we_new_id__]']),
							'selModFields' => we_html_tools::select('search' . $whichSearch . '[__we_new_id__]', we_versions_search::getModFields(), 1, "", false, ['class' => "defaultfont",
								'style' => "width:170px;", 'id' => "search" . $whichSearch . '[__we_new_id__]']),
							'selUsers' => we_html_tools::select('search' . $whichSearch . '[__we_new_id__]', $this->searchclass->getUsers(), 1, "", false, ['class' => "defaultfont",
								'style' => "width:170px;", 'id' => 'search' . $whichSearch . '[__we_new_id__]']),
							'searchFields' => we_html_tools::select('searchFields' . $whichSearch . '[__we_new_id__]', $this->searchclass->getFields($whichSearch), 1, "", false, [
								'class' => "defaultfont", 'id' => "searchFields" . $whichSearch . '[__we_new_id__]', 'onchange' => "weSearch.changeit(this.value, __we_new_id__);"]),
						],
						'result' => $this->getResultData($whichSearch),
							]
		)]);
	}

	public static function getJSConsts(): string{
		return we_base_util::setLangString([
					'MEDIA_CONTENTTYPES_CSV' => [we_base_ContentTypes::APPLICATION, we_base_ContentTypes::AUDIO, we_base_ContentTypes::IMAGE, we_base_ContentTypes::VIDEO,
						we_base_ContentTypes::PDF],
					'SEARCH_ADV' => self::SEARCH_ADV,
					'SEARCH_DOCS' => self::SEARCH_DOCS,
					'SEARCH_MEDIA' => self::SEARCH_MEDIA,
					'SEARCH_TMPL' => self::SEARCH_TMPL,
					'DOCLIST_DOCUMENTS' => we_doclist_view::DOCLIST_DOCUMENTS,
					'DOCLIST_TEMPLATES' => we_doclist_view::DOCLIST_TEMPLATES,
					'DOCLIST_OBJECTFILES' => we_doclist_view::DOCLIST_OBJECTFILES,
					'DOCLIST_COLLECTION' => we_doclist_view::DOCLIST_COLLECTION,
		]);
	}

	public function getNextPrev(int $we_search_anzahl, string $whichSearch): string{
		return $this->getNextPrevData($we_search_anzahl)->getHtml($whichSearch);
	}

	public function getNextPrevData(int $we_search_anzahl): we_search_pagination{
		$currentSearchstart = $this->Model->getPropertyInt('currentSearchstart');
		$currentAnzahl = $this->Model->getPropertyInt('currentAnzahl') ?: 1;
		$disableBack = $currentSearchstart ? false : true;
		$disableNext = ($currentSearchstart + $currentAnzahl) >= $we_search_anzahl;
		$text = (($we_search_anzahl) ? $currentSearchstart + 1 : 0) . '-' .
				(($we_search_anzahl - $currentSearchstart) < $currentAnzahl ? $we_search_anzahl : $currentSearchstart + $currentAnzahl) .
				' ' . g_l('global', '[from]') . ' ' . $we_search_anzahl;
		$pages = [];
		for($i = 0; $i < ceil($we_search_anzahl / $currentAnzahl); $i++){
			$pages[(string) ($i * $currentAnzahl)] = ($i + 1);
		}
		$page = ceil($currentSearchstart / $currentAnzahl) * $currentAnzahl;
		$pages = $pages ?: ['0' => 1];

		return (new we_search_pagination())
						->setSetview($this->Model->getProperty('currentSetView'))
						->setMode($this->Model->mode)
						->setOrder($this->Model->getProperty('currentOrder'))
						->setSearchstart($currentSearchstart)
						->setNumber($currentAnzahl)
						->setDisableBack($disableBack)
						->setDisableNext($disableNext)
						->setText($text)
						->setPage((int) $page)
						->setPages($pages);
	}

	function getSortImage(string $for): string{
		$currentOrder = $this->Model->getProperty('currentOrder');
		if($currentOrder){
			if(str_starts_with($currentOrder, $for)){
				if(str_contains($currentOrder, 'DESC')){
					return '<i class = "fa fa-sort-desc fa-lg"></i>';
				}
				return '<i class = "fa fa-sort-asc fa-lg"></i>';
			}
			return '';
		}
		return '<i class = "fa fa-sort fa-lg"></i>';
	}

	function getSearchDialogOptions(string $whichSearch): string{
		$table = new we_html_table(['class' => 'searchOptions',], 2, 2);
		$row = 0;
		$currentSearchForField = $this->Model->getProperty('currentSearchForField');

		switch($whichSearch){
			case self::SEARCH_DOCS :
				$table
						->setCol($row, 0, we_html_forms::checkboxWithHidden(!empty($currentSearchForField['text']), 'searchForText' . $whichSearch, g_l('searchtool', '[onlyFilename]')))
						->setCol($row++, 1, we_html_forms::checkboxWithHidden(!empty($currentSearchForField['title']), 'searchForTitle' . $whichSearch, g_l('searchtool', '[onlyTitle]')))
						->setCol($row, 0, we_html_forms::checkboxWithHidden(!empty($currentSearchForField['content']), 'searchForContent' . $whichSearch, g_l('searchtool', '[Content]')));
				break;
			case self::SEARCH_TMPL :
				$table
						->setCol($row++, 0, we_html_forms::checkboxWithHidden(!empty($currentSearchForField['text']), 'searchForText' . $whichSearch, g_l('searchtool', '[onlyFilename]')))
						->setCol($row, 0, we_html_forms::checkboxWithHidden(!empty($currentSearchForField['content']), 'searchForContent' . $whichSearch, g_l('searchtool', '[Content]')));
				break;
			case self::SEARCH_MEDIA :
				$table
						->setCol($row, 0, we_html_forms::checkboxWithHidden(!empty($currentSearchForField['text']), 'searchForText' . $whichSearch, g_l('searchtool', '[onlyFilename]')))
						->setCol($row++, 1, we_html_forms::checkboxWithHidden(!empty($currentSearchForField['title']), 'searchForTitle' . $whichSearch, g_l('searchtool', '[onlyTitle]')))
						->setCol($row, 0, we_html_forms::checkboxWithHidden(!empty($currentSearchForField['meta']), 'searchForMeta' . $whichSearch, g_l('searchtool', '[onlyMetadata]')));

				return $table->getHtml();
		}
		//$table->setCol($row++, 2, we_html_button::create_button(we_html_button::SEARCH, "javascript:weSearch.search(true);"), ['style' => 'text-align:right']);

		return $table->getHtml();
	}

	function getSearchDialogMediaType(string $whichSearch): string{
		$table = new we_html_table(['class' => 'searchMediaType',], 2, 2);
		$currentSearchForContentType = $this->Model->getProperty('currentSearchForContentType');

		switch($whichSearch){
			case self::SEARCH_MEDIA :
				$n = 1;
				$table->setCol(0, 0, we_html_element::hiddens([
									'searchFields' . $whichSearch . '[' . $n . ']' => 'ContentType',
									'search' . $whichSearch . '[' . $n . ']' => '1',
									'location' . $whichSearch . '[' . $n++ . ']' => 'IN'
								]) . we_html_forms::checkboxWithHidden($currentSearchForContentType['image'] ? true : false, 'searchForImage' . $whichSearch, g_l('contentTypes', '[image/*]'), false, 'withSpace'))
						->setCol(0, 1, we_html_forms::checkboxWithHidden(!empty($currentSearchForContentType['audio']), 'searchForAudio' . $whichSearch, g_l('contentTypes', '[audio/*]')))
						->setCol(1, 1, we_html_forms::checkboxWithHidden(!empty($currentSearchForContentType['video']), 'searchForVideo' . $whichSearch, g_l('contentTypes', '[video/*]')))
						->setCol(1, 0, we_html_forms::checkboxWithHidden(!empty($currentSearchForContentType['other']), 'searchForOther' . $whichSearch, g_l('contentTypes', '[media/*]')));
				break;
			default:
				return '';
		}

		return $table->getHtml();
	}

	function getSearchDialogFilter(string $whichSearch): string{
		$table = new we_html_table(['width' => '400'], 4, 4);
		$currentSearch = $this->Model->getProperty('currentSearch');
		/*
		  $currentSearch = array_map(function($v){
		  return $v === we_base_request::NOT_VALID_S ? '' : $v;
		  }, $this->Model->getProperty('currentSearch'));
		 *
		 */

		switch($whichSearch){
			case self::SEARCH_MEDIA :
				$n = 2;
				$table
						->setCol(0, 0, g_l('searchtool', '[usage_state]') . ': ')
						->setCol(0, 1, we_html_element::hiddens(['searchFields' . $whichSearch . '[' . $n . ']' => 'IsUsed',
									'location' . $whichSearch . '[' . $n . ']' => we_search_base::OP_IS]) .
								we_html_tools::select('search' . $whichSearch . '[' . $n . ']', [-1 => g_l('searchtool', '[all]'),
									1 => g_l('searchtool', '[only_unsed]'),
									2 => g_l('searchtool', '[only_unused]')], 1, $currentSearch[$n] ?? '', false, [], true, 228), ['colspan' => '2'])
						->setCol(0, 3, we_html_tools::alertAttentionBox(g_l('searchtool', '[media_do_rebuild]'), we_html_tools::TYPE_HELP))
						->setCol(1, 0, g_l('searchtool', '[protection]') . ': ')
						->setCol(1, 1, we_html_element::hiddens(['searchFields' . $whichSearch . '[' . ++$n . ']' => 'IsProtected',
									'location' . $whichSearch . '[' . $n . ']' => we_search_base::OP_IS]) .
								we_html_tools::select('search' . $whichSearch . '[' . $n . ']', [-1 => g_l('searchtool', '[all]'),
									1 => g_l('searchtool', '[only_protected]'),
									2 => g_l('searchtool', '[only_unprotected]')], 1, $currentSearch[$n] ?? '', false, [], true, 228), ['colspan' => '2']);

				$this->searchMediaOptFieldIndex = ++$n; // FIXME: do we need this (or can we handle it as simple param?)
				break;
			default:
				return '';
		}
		$table->setCol(2, 0, $this->getSearchDialogOptionalFields($whichSearch), ['colspan' => '4'])
				->setCol(3, 3, we_html_button::create_button(we_html_button::SEARCH, "javascript:weSearch.search(true);"));

		return $table->getHtml();
	}

	function getSearchDialogCheckboxesAdvSearch(): string{
		/* => if we need this move it to model init
		  if(($table = we_base_request::_(we_base_request::TABLE, 'table'))){
		  $search_tables_advSearch = $table;
		  $this->Model->search_tables_advSearch[$search_tables_advSearch] = 1;
		  }
		 *
		 */

		$table = new we_html_table(['style' => 'width:550px',], 4, 3);
		$currentSearchTables = $this->Model->getProperty('currentSearchTables');

		if(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){
			$table->setCol(0, 0, we_html_forms::checkboxWithHidden(in_array(FILE_TABLE, $currentSearchTables, true), 'search_tables_advSearch[' . FILE_TABLE . ']', g_l('searchtool', '[documents]'), false, 'defaultfont', ''));
		}

		if(we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE]) && we_editor_functions::isNormalMode()){
			$table->setCol(1, 0, we_html_forms::checkboxWithHidden(in_array(TEMPLATES_TABLE, $currentSearchTables, true), 'search_tables_advSearch[' . TEMPLATES_TABLE . ']', g_l('searchtool', '[templates]'), false, 'defaultfont', ''));
		}

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			if(we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
				$table->setCol(0, 1, we_html_forms::checkboxWithHidden(in_array(OBJECT_FILES_TABLE, $currentSearchTables, true), 'search_tables_advSearch[' . OBJECT_FILES_TABLE . ']', g_l('searchtool', '[objects]'), false, 'defaultfont', ''));
			}
			if(we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE]) && we_editor_functions::isNormalMode()){
				$table->setCol(1, 1, we_html_forms::checkboxWithHidden(in_array(OBJECT_TABLE, $currentSearchTables, true), 'search_tables_advSearch[' . OBJECT_TABLE . ']', g_l('searchtool', '[classes]'), false, 'defaultfont', ''));
			}
		}

		if(we_base_permission::hasPerm([we_base_perm::VERSIONS_CAN_SEE])){
			$table->setCol(0, 2, we_html_forms::checkboxWithHidden(in_array(VERSIONS_TABLE, $currentSearchTables, true), 'search_tables_advSearch[' . VERSIONS_TABLE . ']', g_l('versions', '[versions]'), false, 'defaultfont', ''));
		}

		$table->setCol(1, 2, we_html_button::create_button(we_html_button::SEARCH, "javascript:weSearch.search(true);"), ['style' => 'text-align:right']);

		return $table->getHtml();
	}

	function getSearchDialog(string $whichSearch = ''): string{
		$searchInput = '';
		$currentSearch = $this->Model->getProperty('currentSearch');
		/*
		  $currentSearch = array_map(function($v){
		  return $v === we_base_request::NOT_VALID_S ? '' : $v;
		  }, $this->Model->getProperty('currentSearch'));
		 *
		 */
		$keyword = $currentSearch[0] ?? '';

		switch($whichSearch){
			case self::SEARCH_TMPL :
				$searchTables = 'search_tables_' . $whichSearch . '[' . TEMPLATES_TABLE . ']';
				break;
			case self::SEARCH_MEDIA :
				$currentFields = $this->Model->getProperty('currentSearchFields');
				$keyword = $currentFields && $currentFields[0] === 'keyword' ? $keyword : '';
				$searchInput .= we_html_element::hidden('searchFields' . $whichSearch . '[0]', 'keyword');
				$searchTables = 'search_tables_' . $whichSearch . '[' . FILE_TABLE . ']';
				break;
			default:
			case self::SEARCH_DOCS :
				$searchTables = 'search_tables_' . $whichSearch . '[' . FILE_TABLE . ']';
				break;
		}
		$searchInput .= we_html_tools::textInput('search' . $whichSearch . '[0]', (string) $keyword, 0, [], 'search', $whichSearch == self::SEARCH_MEDIA ? '348px' : '380px');

		return '<table class="default"><tbody>
<tr>
 <td>' . $searchInput . ($whichSearch == self::SEARCH_MEDIA ? ' ' . we_html_tools::alertAttentionBox("Ohne Suchbegriff werden alle Medien-Dokumente ausgegeben.", we_html_tools::TYPE_HELP) : '') . '</td>
 <td>' . we_html_button::create_button(we_html_button::SEARCH, "javascript:weSearch.search(true);") . '</td>
 <td>' . we_html_element::hiddens([
					'location' . $whichSearch . '[0]' => we_search_base::OP_CONTAIN,
					$searchTables => '1'
				]) . '</td>
 <td></td>
 <td></td>
</tr></tbody></table>';
	}

	/* TODO: move somewhere else... */

	/**
	 *
	 * @param string $whichSearch
	 * @return array{elements:we_search_resultBase[],pagination:we_search_pagination}
	 */
	public function getResultData(string $whichSearch): array{
		$entries = $this->searchclass->searchProperties($this->Model, $whichSearch);

		return [
			'elements' => $entries,
			'pagination' => $this->getNextPrevData($this->searchclass->founditems),
		];
	}

	/**
	 *
	 * @param string $whichSearch
	 * @return string
	 */
	public function makeResultContainer(string $whichSearch): string{
		$view = (string) $this->Model->getProperty('currentSetView');

		$fieldColumns = [];
		switch($whichSearch){
			case we_search_view::SEARCH_DOCS:
				$listColumns = we_search_resultDocuments::getListColumns();
				break;
			case we_search_view::SEARCH_TMPL:
				$listColumns = we_search_resultTemplates::getListColumns();
				break;
			case we_search_view::SEARCH_MEDIA:
				$listColumns = we_search_resultMedia::getListColumns();
				break;
			case we_search_view::SEARCH_ADV:
				$listColumns = we_search_resultAdvanced::getListColumns();
				break;
			case we_doclist_view::DOCLIST_DOCUMENTS:
				$listColumns = we_doclist_resultDocuments::getListColumns();
				break;
			case we_doclist_view::DOCLIST_TEMPLATES:
				$listColumns = we_doclist_resultTemplates::getListColumns();
				break;
			case we_doclist_view::DOCLIST_OBJECTFILES:
				$listColumns = we_doclist_resultObjectfiles::getListColumns();
				$fieldColumns = we_doclist_resultObjectfiles::getFieldColumns($this->Model->getPropertyInt('TableID'));
				break;
			case we_doclist_view::DOCLIST_COLLECTION:
				$listColumns = we_doclist_resultCollections::getListColumns();
				break;
			default:
				return '';
		}

		$onclick = "weSearch.setSort('%s','" . $whichSearch . "');";
		$propsHeaderCols = $propsEmptyCols = $fieldsHeaderCols = $fieldsEmptyCols = [];
		$gridSort = new we_html_select(['id' => 'gridOrder_' . $whichSearch, 'onchange' => "weSearch.setSort(this.value);"]);
		foreach($listColumns as $col){
			$propsHeaderCols[] = we_html_element::span(($col['sort'] ? ['onclick' => sprintf($onclick, (string) $col['sort'])] : []), ($col['title'] ?: '')) .
					($col['sort'] ? we_html_element::span(['id' => $col['sort'] . '_' . $whichSearch, 'class' => 'listSortArrow'], $this->getSortImage((string) $col['sort'])) : '');
			$propsEmptyCols[] = new we_html_td('', []);
			if($col['sort']){
				$gridSort->addOption((string) $col['sort'], (string) $col['title']);
			}
		}
		foreach($fieldColumns as $col){
			$fieldsHeaderCols[] = we_html_element::span(($col['sort'] ? ['onclick' => sprintf($onclick, $col['sort'])] : []), ($col['title'] ?: '')) .
					($col['sort'] ? we_html_element::span(['id' => $col['sort'] . '_' . $whichSearch, 'class' => 'listSortArrow'], $this->getSortImage($col['sort'])) : '');
			$fieldsEmptyCols[] = new we_html_td('', []);
		}

		$props = new we_html_table(['id' => 'list_' . $whichSearch,
			'class' => 'searchResultList largeicons ' . $whichSearch,
				], 0, count($listColumns));
		$props->addHeaderRow($propsHeaderCols);
		$props->addRow(new we_html_tr($propsEmptyCols, ['class' => /* 'htmlDialogBorder4Cell' */'']));
		$fields = new we_html_table(['id' => 'fields_' . $whichSearch,
			'class' => 'searchResultFields largeicons ' . $whichSearch,
				], 0, count($fieldColumns));
		$fields->addHeaderRow($fieldsHeaderCols);
		$fields->addRow(new we_html_tr($fieldsEmptyCols, ['class' => /* 'htmlDialogBorder4Cell' */'']));
		$gridBtnAsc = we_html_button::create_button(we_html_button::SORT_ASC, '', '', "weSearch.setSort('', true);", false, true, '', false, '', '', 'btnAsc_' . $whichSearch, false, 'ASC');
		$grid = we_html_element::div([], we_html_element::div([], $gridSort->getHtml() . $gridBtnAsc) . we_html_element::div(['id' => 'grid_' . $whichSearch, 'class' => 'searchResultList largeicons']));

		$propsDiv = we_html_element::div(['id' => 'resultList_' . $whichSearch, 'class' => ($view === 'list' ? '' : 'hidden')], $props->getHtml());
		$fieldsDiv = we_html_element::div(['id' => 'resultFields_' . $whichSearch, 'class' => ($view === 'fields' ? '' : 'hidden')], $fields->getHtml());
		$gridDiv = we_html_element::div(['id' => 'resultGrid_' . $whichSearch, 'class' => ($view === 'grid' ? '' : 'hidden')], $grid);

		return we_html_element::div(['id' => 'scrollContent_' . $whichSearch], $propsDiv . $fieldsDiv . $gridDiv);
	}

	function getSearchParameterTop(int $foundItems, string $whichSearch): string{
		$currentOrder = $this->Model->getProperty('currentOrder');
		$currentSetView = $this->Model->getProperty('currentSetView');
		$currentSearchstart = $this->Model->getPropertyInt('currentSearchstart');
		$currentAnzahl = $this->Model->getPropertyInt('currentAnzahl');
		$currentAnzahlMedialinks = $this->Model->getPropertyInt('currentAnzahlMedialinks');
		$currentFolderID = $this->Model->getPropertyInt('currentFolderID');
		$currentSearchTables = $this->Model->getProperty('currentSearchTables');

		$colNewdir = $colImportFile = $numMedialinks = $moreHiddens = '';

		$colSetView = we_html_element::span(['id' => 'viewButtonIcon', 'style' => ($this->Model->getProperty('currentSetView') !== self::VIEW_LIST ? 'display:none;' : '')], we_html_button::create_button(we_html_button::VIEW_ICON, "javascript:weSearch.setView('" . we_search_view::VIEW_ICONS . "');document.getElementById('viewButtonIcon').style.display='none';document.getElementById('viewButtonList').style.display='inline';")) .
				we_html_element::span(['id' => 'viewButtonList', 'style' => ($this->Model->getProperty('currentSetView') !== self::VIEW_ICONS ? 'display:none;' : '')], we_html_button::create_button(we_html_button::VIEW_LIST, "javascript:weSearch.setView('" . we_search_view::VIEW_LIST . "');document.getElementById('viewButtonIcon').style.display='inline';document.getElementById('viewButtonList').style.display='none';"));

		switch($whichSearch){
			case self::SEARCH_MEDIA:
				$numMedialinks = we_html_element::span(['style' => 'font-size:12px; padding:6px 6px 0 12px;'], 'Medialinks: ') .
						we_html_tools::select('anzahlMedialinks' . $whichSearch, [0 => g_l('searchtool', '[no_template]'), 10 => 10, -1 => g_l('searchtool', '[all]')], 1, $currentAnzahlMedialinks, false, [
							'onchange' => "this.form.elements['searchstart" . $whichSearch . "'].value=0;weSearch.search(false);"], true, 0, 'selectSearchNumberMedialinks');
				break;
			case we_doclist_view::DOCLIST_OBJECTFILES:
				$colSetView = we_html_element::span(['id' => 'viewButtonFields', 'style' => ($this->Model->getProperty('currentSetView') !== self::VIEW_LIST ? 'display:none;' : '')], we_html_button::create_button(we_html_button::VIEW_FIELDS, "javascript:weSearch.setView('" . we_search_view::VIEW_FIELDS . "');document.getElementById('viewButtonFields').style.display='none';document.getElementById('viewButtonList').style.display='inline';", '', '', false, true, '', false, g_l('modules_objectClassfoldersearch', '[FELDER]'))) .
						we_html_element::span(['id' => 'viewButtonList', 'style' => ($this->Model->getProperty('currentSetView') !== self::VIEW_FIELDS ? 'display:none;' : '')], we_html_button::create_button(we_html_button::VIEW_LIST, "javascript:weSearch.setView('" . we_search_view::VIEW_LIST . "');document.getElementById('viewButtonFields').style.display='inline';document.getElementById('viewButtonList').style.display='none';", '', '', false, true, '', false, g_l('weClass', '[properties]')));
				$folderType = we_base_ContentTypes::FOLDER_OBJECTFILE;
			/* fall through */
			case we_doclist_view::DOCLIST_DOCUMENTS:
				$folderType = $folderType ?? we_base_ContentTypes::FOLDER_DOCUMENT;
			case we_doclist_view::DOCLIST_TEMPLATES:
				$folderType = $folderType ?? we_base_ContentTypes::FOLDER_TEMPLATE;
			case we_doclist_view::DOCLIST_COLLECTION:
				$folderType = $folderType ?? we_base_ContentTypes::FOLDER_VFILE;
				if($currentFolderID && $whichSearch === we_doclist_view::DOCLIST_DOCUMENTS){
					$colImportFile = we_fileupload_ui_importer::getBtnImportFiles($currentFolderID, true, 'reload_activeEditor_and_tree,' . FILE_TABLE);
				}
				$colNewdir = we_html_button::create_button(we_html_button::NEW_DIR, "javascript:top.we_cmd('new_document','" . $currentSearchTables[0] . "','','" . $folderType . "','','" . $currentFolderID . "')", '', '', false);

				$model = $this->Model;
				'@phan-var we_doclist_model $model';
				$moreHiddens = we_html_element::hiddens(['we_transaction' => $model->transaction,]);
		}
		$selectAnzahl = we_html_tools::select('anzahl' . $whichSearch, self::RESULT_SIZE, 1, $currentAnzahl, false, ['onchange' => "this.form.elements['searchstart" . $whichSearch . "'].value=0;weSearch.search(false);"], true, 0, 'selectSearchNumber');

		$tbl = (new we_html_table(['class' => 'default', 'style' => 'margin:12px 0 12px 19px;'], 1, 9))
				->setCol(0, ($c = 0), g_l('searchtool', '[eintraege_pro_seite]') . ':', ['style' => 'font-size:12px; padding:6px 6px 0 0;'])
				->setCol(0, ++$c, $selectAnzahl, ['class' => 'defaultfont lowContrast', 'style' => 'padding:0 10px 0 0;'])
				->setCol(0, ++$c, $this->getNextPrev($foundItems, $whichSearch))
				->setCol(0, ++$c, $numMedialinks)
				->setCol(0, ++$c, $colSetView)
				->setCol(0, ++$c, $colImportFile, $colImportFile ? ['style' => 'width:50px;'] : [])
				->setCol(0, ++$c, $colNewdir, $colImportFile ? ['style' => 'width:50px;'] : []);

		return we_html_element::hiddens([
					'setView' . $whichSearch => $currentSetView,
					'Order' . $whichSearch => $currentOrder,
					'mode' => (string) $this->Model->mode,
					'searchstart' . $whichSearch => (string) $currentSearchstart,
					'newSearch' => '1',
					'position' => '',
					'do' => '',
				]) . $moreHiddens . $tbl->getHtml();
	}

	function getSearchParameterBottom(int $foundItems, string $whichSearch): string{
		$resetButton = (we_base_permission::hasPerm([we_base_perm::VERSIONS_RESET]) && $whichSearch === self::SEARCH_ADV ?
				we_html_button::create_button(we_html_button::RESET, "javascript:weSearch.resetVersions();", '') :
				'');

		$actionButton = $actionButtonCheckboxAll = '';
		switch($whichSearch){
			case self::SEARCH_DOCS:
			case self::SEARCH_ADV:
				if(we_base_permission::hasPerm([we_base_perm::WORK_CAN_PUBLISH]) && !($whichSearch === we_doclist_view::DOCLIST_TEMPLATES)){ // ??
					$actionButtonCheckboxAll = we_html_button::create_button(we_html_button::TOGGLE, "javascript:weSearch.checkAllActionChecks('" . $whichSearch . "')");

					$actionButton = we_html_button::create_button(we_html_button::FOOTER_PUBLISH, "javascript:weSearch.publishDocs('" . we_folder_base::ACTION_PUBLISH . "','" . $whichSearch . "');");
					break;
				}
				break;
			case self::SEARCH_MEDIA:
				$actionButtonCheckboxAll = we_html_button::create_button(we_html_button::TOGGLE, "javascript:weSearch.checkAllActionChecks('" . $whichSearch . "')");
				$actionButton = we_html_button::create_button(we_html_button::DELETE, "javascript:weSearch.deleteMediaDocs('" . $whichSearch . "');");
				break;
			case we_doclist_view::DOCLIST_TEMPLATES:
			case we_doclist_view::DOCLIST_COLLECTION:
			case we_doclist_view::DOCLIST_DOCUMENTS:
				if(we_base_permission::hasPerm([we_base_perm::WORK_CAN_PUBLISH])){
					$actionButtonCheckboxAll = we_html_button::create_button(we_html_button::TOGGLE, "javascript:weSearch.checkAllActionChecks('" . $whichSearch . "')");
					$actions = array_filter([
						'' => g_l('import', '[option_select]'),
						we_folder_base::ACTION_DELETE => (we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_DELETE]) ? g_l('modules_objectClassfoldersearch', '[action][delete]') : ''),
						we_folder_base::ACTION_PUBLISH => g_l('modules_objectClassfoldersearch', '[action][publish]'),
						we_folder_base::ACTION_UNPUBLISH => g_l('modules_objectClassfoldersearch', '[action][unpublish]'),
						we_folder_base::ACTION_SEARCHABLE => g_l('modules_objectClassfoldersearch', '[action][searchable]'),
						we_folder_base::ACTION_UNSEARCHABLE => g_l('modules_objectClassfoldersearch', '[action][unsearchable]'),
						we_folder_base::ACTION_CATEGORY_ADD => (we_modules_info::isActive(we_modules_info::CATEGORY) ? g_l('modules_objectClassfoldersearch', '[action][category][add]') : ''),
						we_folder_base::ACTION_CATEGORY_DELETE => (we_modules_info::isActive(we_modules_info::CATEGORY) ? g_l('modules_objectClassfoldersearch', '[action][category][delete]') : ''),
					]);
					$actionButton = we_html_tools::select('action', $actions, 1, 0, false, ['onchange' => "weSearch.handleActionSelect(this,'" . $whichSearch . "')"]);
					break;
				}
				$actionButton = '';
				break;
			case we_doclist_view::DOCLIST_OBJECTFILES:
				if(we_base_permission::hasPerm([we_base_perm::WORK_CAN_PUBLISH])){
					$actionButtonCheckboxAll = we_html_button::create_button(we_html_button::TOGGLE, "javascript:weSearch.checkAllActionChecks('" . $whichSearch . "')");
					$actions = array_filter([
						'' => g_l('import', '[option_select]'),
						we_folder_base::ACTION_DELETE => (we_base_permission::hasPerm([we_base_perm::OBJECTFILE_DELETE]) ? g_l('modules_objectClassfoldersearch', '[action][delete]') : ''),
						we_folder_base::ACTION_PUBLISH => g_l('modules_objectClassfoldersearch', '[action][publish]'),
						we_folder_base::ACTION_UNPUBLISH => g_l('modules_objectClassfoldersearch', '[action][unpublish]'),
						we_folder_base::ACTION_SEARCHABLE => g_l('modules_objectClassfoldersearch', '[action][searchable]'),
						we_folder_base::ACTION_UNSEARCHABLE => g_l('modules_objectClassfoldersearch', '[action][unsearchable]'),
						we_folder_base::ACTION_COPY_WS => g_l('modules_objectClassfoldersearch', '[action][copyws]'),
						we_folder_base::ACTION_COPY_TID => g_l('modules_objectClassfoldersearch', '[action][copytid]'),
						we_folder_base::ACTION_CATEGORY_ADD => (we_modules_info::isActive(we_modules_info::CATEGORY) ? g_l('modules_objectClassfoldersearch', '[action][category][add]') : ''),
						we_folder_base::ACTION_CATEGORY_DELETE => (we_modules_info::isActive(we_modules_info::CATEGORY) ? g_l('modules_objectClassfoldersearch', '[action][category][delete]') : ''),
					]);
					$actionButton = we_html_tools::select('action', $actions, 1, 0, false, ['onchange' => "weSearch.handleActionSelect(this,'" . $whichSearch . "')"]);
					break;
				}
				$actionButton = '';
				break;
		}

		$tbl = (new we_html_table(['class' => 'default', 'style' => 'margin-top:10px;'], 2, 4))
				->setCol(($k = 0), ($c = 0), $actionButtonCheckboxAll, ['style' => 'font-size:12px;width:12px;'])
				->setCol($k, ++$c, $actionButton . ($resetButton ? we_html_element::br() . $resetButton : ''), ['style' => 'font-size:12px;width:125px;'])
				->setColAttributes($k, ++$c, ['class' => 'defaultfont lowContrast', 'style' => 'width:48px;height:30px;', 'id' => 'resetBusy' . $whichSearch])
				->setCol($k, ++$c, $this::getNextPrev($foundItems, $whichSearch));

		return $tbl->getHtml();
	}

	// FIXME: will be  obsolete as soon as getSearchDialogOptionalFields() works properly
	function getSearchDialogOptionalFields(string $whichSearch): string{
		if($whichSearch !== self::SEARCH_ADV && $whichSearch !== self::SEARCH_MEDIA){
			return '';
		}
		$currentSearchFields = $this->Model->getProperty('currentSearchFields');
		$currentSearch = $this->Model->getProperty('currentSearch');
		/*
		  $currentSearch = array_map(function($v){
		  return $v === we_base_request::NOT_VALID_S ? '' : $v;
		  }, $this->Model->getProperty('currentSearch'));
		 *
		 */
		$currentLocation = $this->Model->getProperty('currentLocation');
		$this->searchclass->height = count($currentSearchFields);

		$out = '<div ' . /* ($whichSearch === self::SEARCH_MEDIA ? '' : 'style="margin-left:123px;"') . */ '>
<table><tbody id="filterTable' . $whichSearch . '">
<tr>
 <td></td>
 <td></td>
 <td></td>
 <td></td>
 <td></td>
</tr>';

		for($i = ($whichSearch === self::SEARCH_MEDIA ? $this->searchMediaOptFieldIndex : 0); $i < $this->searchclass->height; $i++){
			$button = we_html_button::create_button(we_html_button::TRASH, "javascript:weSearch.delRow(" . $i . ");", '', '', false, false, '', false);
			$locationDisabled = $handle = '';
			$searchInput = we_html_tools::textInput('search' . $whichSearch . '[' . $i . ']', ($currentSearch[$i] ?? ''), 0, ['class' => "wetextinput",
						'id' => "search" . $whichSearch . "[" . $i . "]"], "search", '170px');

			if(isset($currentSearchFields[$i])){
				switch($currentSearchFields[$i]){
					case 'ParentIDDoc':
					case 'ParentIDObj':
					case 'ParentIDTmpl':
					case 'Content':
					case 'Status':
					case 'Speicherart':
					case 'MasterTemplateID':
					case 'HasReferenceToID':
					case 'temp_template_id':
					case 'temp_category':
						$locationDisabled = 'disabled';
				}

				switch($currentSearchFields[$i]){
					case 'meta__Title':
					case 'meta__Description':
					case 'meta__Keywords':
					case 'meta__Autor':
					case 'meta__MIME':
					case 'ID':
					case 'Path':
					case 'CreatorName':
						break;
					case 'allModsIn':
						$searchInput = we_html_tools::select('search' . $whichSearch . '[' . $i . ']', we_versions_search::getModFields(), 1, ($currentSearch[$i] ?? ''), false, [
									'class' => 'defaultfont', 'style' => 'width:170px;', 'id' => 'search' . $whichSearch . '[' . $i . ']']);
						break;
					case 'Status':
						$searchInput = we_html_tools::select('search' . $whichSearch . '[' . $i . ']', $this->searchclass->getFieldsStatus(), 1, ($currentSearch[$i] ?? ''), false, [
									'class' => 'defaultfont', 'style' => 'width:170px;', 'id' => 'search' . $whichSearch . '[' . $i . ']']);
						break;
					case 'Speicherart':
						$searchInput = we_html_tools::select('search' . $whichSearch . '[' . $i . ']', $this->searchclass->getFieldsSpeicherart(), 1, ($currentSearch[$i] ?? ''), false, [
									'class' => 'defaultfont', 'style' => 'width:170px;', 'id' => 'search' . $whichSearch . '[' . $i . ']']);
						break;
					case 'Publish_Date':
					case 'Creation_Date':
					case 'Mod_Date':
						$handle = 'date';
						$searchInput = we_html_tools::getNewDateInput('search' . $whichSearch . '[' . $i . ']', new DateTime($currentSearch[$i] ?? 'now'), we_html_tools::FORMAT_DATE);
						break;
					case 'ParentIDDoc':
					case 'ParentIDObj':
					case 'ParentIDTmpl':
						$linkPath = ($currentSearch[$i] ?? '');

						$rootDirID = 0;
						$cmd = "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements['search" . $whichSearch . "ParentID[" . $i . "]'].value,table:WE().consts.tables.FILE_TABLE,targetID:'search" . $whichSearch . "ParentID[" . $i . "]',targetName:'search" . $whichSearch . "[" . $i . "]',rootDirID:'" . $rootDirID . "'})";
						$button = we_html_button::create_button(we_html_button::SELECT, $cmd);
						$selector = we_html_tools::formElementTable('', we_html_tools::textInput('search' . $whichSearch . '[' . $i . ']', $linkPath, 0, ['readonly' => 'readonly'], 'text', '170px'), we_html_element::hidden('search' . $whichSearch . 'ParentID[' . $i . ']', ""), $button);

						$searchInput = $selector;
						break;
					case 'HasReferenceToID':
						$linkPath = ($currentSearch[$i] ?? '');

						$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_image',{id:document.we_form.elements['search" . $whichSearch . "ParentID[" . $i . "]'].value,table:WE().consts.tables.FILE_TABLE,targetID:'search" . $whichSearch . "ParentID[" . $i . "]',targetName:'search" . $whichSearch . "[" . $i . "]'})", '', '', false);
						$selector = we_html_tools::formElementTable('', we_html_tools::textInput('search' . $whichSearch . '[' . $i . ']', $linkPath, 0, ['readonly' => 'readonly'], 'text', '170px'), we_html_element::hidden('search' . $whichSearch . 'ParentID[' . $i . ']', ''), $button);

						$searchInput = $selector;
						break;
					case 'MasterTemplateID':
					case 'temp_template_id':
						$linkPath = ($currentSearch[$i] ?? '');
						$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['search" . $whichSearch . "ParentID[" . $i . "]'].value,table:WE().consts.tables.TEMPLATES_TABLE,targetID:'search" . $whichSearch . "ParentID[" . $i . "]','search" . $whichSearch . "[" . $i . "]',rootDirID:0})", '', '', false);
						$selector = we_html_tools::formElementTable('', we_html_tools::textInput('search' . $whichSearch . '[' . $i . ']', $linkPath, 0, ['readonly' => 'readonly'], 'text', '170px'), we_html_element::hidden('search' . $whichSearch . 'ParentID[' . $i . ']', ''), $button);

						$searchInput = $selector;
						break;
					case 'temp_category':
						$linkPath = ($currentSearch[$i] ?? '');
						$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['search" . $whichSearch . "ParentID[" . $i . "]'].value,table:WE().consts.tables.CATEGORY_TABLE,targetID:'search" . $whichSearch . "ParentID[" . $i . "]',targetName:'search" . $whichSearch . "[" . $i . "]',rootDirID:0})", '', '', false);
						$selector = we_html_tools::formElementTable('', we_html_tools::textInput('search' . $whichSearch . '[' . $i . ']', $linkPath, 0, ['readonly' => 'readonly'], 'text', '170px'), we_html_element::hidden('search' . $whichSearch . 'ParentID[' . $i . ']', ''), $button);

						$searchInput = $selector;
				}
			}

			$locationValue = ($currentLocation[$i] ?? '');
			$out .= '<tr id="filterRow_' . $i . '">
	<td>' . we_html_element::hidden('hidden_searchFields' . $whichSearch . '[' . $i . ']', $currentSearchFields[$i] ?? '') .
					we_html_tools::select('searchFields' . $whichSearch . '[' . $i . ']', $this->searchclass->getFields($whichSearch), 1, ($currentSearchFields[$i] ?? ''), false, [
						'class' => "defaultfont", 'id' => 'searchFields' . $whichSearch . '[' . $i . ']', 'onchange' => "weSearch.changeit(this.value, " . $i . ");"]) .
					'</td>
	<td id="td_location' . $whichSearch . '[' . $i . ']">' . we_html_tools::select('location' . $whichSearch . '[' . $i . ']', we_search_search::getLocation($handle), 1, $locationValue, false, [
						'class' => "defaultfont", 'style' => 'width:150px', $locationDisabled => $locationDisabled, 'id' => 'location' . $whichSearch . '[' . $i . ']']) . '</td>
	<td id="td_search' . $whichSearch . '[' . $i . ']">' . $searchInput . '</td>
	<td id="td_delButton[' . $i . ']">' . $button . '</td>
	<td id="td_hiddenLocation[' . $i . ']">' . (!$locationDisabled ? '' : we_html_element::hidden('location' . $whichSearch . '[' . $i . ']', $locationValue)) . '</td>
	</tr>';
		}

		$out .= '</tbody></table>' .
				'<table>
<tr>
<td>' . we_html_button::create_button(we_html_button::ADD, "javascript:weSearch.newinput();") . '</td>
<td colspan="8" style="text-align:right"></td>
</tr>
</table></div>';

		return $out;
	}

	function getDirSelector(string $whichSearch): string{
		$weSuggest = we_gui_suggest::getInstance();
		switch($whichSearch){
			default:
			case self::SEARCH_DOCS :
				$nameFolderID = 'folderIDDoc';
				$nameFolderPath = 'folderPathDoc';
				$table = FILE_TABLE;
				$ACname = 'docu';
				break;
			case self::SEARCH_MEDIA :
				$nameFolderID = 'folderIDMedia';
				$nameFolderPath = 'folderPathMedia';
				$table = FILE_TABLE;
				$ACname = 'docu';
				break;
			case self::SEARCH_TMPL :
				$nameFolderID = 'folderIDTmpl';
				$nameFolderPath = 'folderPathTmpl';
				$table = TEMPLATES_TABLE;
				$ACname = 'Tmpl';
				break;
		}
		$currentFolderID = $this->Model->getPropertyInt('currentFolderID');
		$path = id2path($currentFolderID, $table);

		$weSuggest->setAcId($ACname)
				->setWidth(380)
				->setContentType(we_base_ContentTypes::FOLDER_TYPES)
				->setInput($nameFolderPath, $path)
				->setLabel('')
				->setMaxResults(20)
				->setResult($nameFolderID, $nameFolderPath)
				->setSelector(we_gui_suggest::DirSelector)
				->setTable($table)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements['" . $nameFolderID . "'].value,table:'" . $table . "',targetID:'" . $nameFolderID . "',targetName:'" . $nameFolderPath . "'})"));

		return $weSuggest->getHTML();
	}

	function getCommonHiddens(array $cmds = []): string{
		return we_html_element::hiddens(['cmd' => ($cmds['cmd'] ?? ''),
					'cmdid' => ($cmds['cmdid'] ?? ''),
					'pnt' => ($cmds['pnt'] ?? ''),
					'tabnr' => ($cmds['tabnr'] ?? ''),
		]);
	}

	function getJSProperty(array $jsVars = []): string{
		return we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'search/search_view2.js');
	}

	function processVariables(): void{
		if(we_base_request::string('pnt', 'frameset') === 'frameset'){
			$modelid = we_base_request::int('modelid', 0);
			$this->Model = new we_search_model($modelid);
			if($modelid){
				$this->Model->load($modelid);
				$_SESSION['weS']['search']['modelidForTree'] = $modelid;
			}
			$_SESSION['weS']['module_sessions']['search'] = $this->Model;
		} elseif(isset($_SESSION['weS']['module_sessions']['search'])){
			$this->Model = $_SESSION['weS']['module_sessions']['search'];
		}

		$modelVars = array_merge([// some vars are not persistent in db but must be written to session anyway
			'searchstartDocSearch',
			'searchstartTmplSearch',
			'searchstartMediaSearch',
			'searchstartAdvSearch'], $this->Model->persistent_slots);

		$doInitModelByHttp = false;
		foreach($modelVars as $val){
			if(we_base_request::_(we_base_request::RAW, $val, we_base_request::NOT_VALID_RAW) !== we_base_request::NOT_VALID_RAW){
				$doInitModelByHttp = true;
				break;
			}
		}

		if($doInitModelByHttp && ($tab = we_base_request::int('tab', we_base_request::int('tabnr', we_search_search::TAB_DOCUMENTS)))){
			switch($tab){
				case we_search_search::TAB_DOCUMENTS:
					$whichSearch = self::SEARCH_DOCS;
					break;
				case we_search_search::TAB_TEMPLATES:
					$whichSearch = self::SEARCH_TMPL;
					break;
				case we_search_search::TAB_ADVANCED:
					$whichSearch = self::SEARCH_ADV;
					break;
				case we_search_search::TAB_MEDIA:
					$whichSearch = self::SEARCH_MEDIA;
					break;
				default:
					$whichSearch = '';
			}

			$this->Model->initByHttp($whichSearch, false);
		}
	}

}
