<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 7cfe0924a900 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-02-10 14:58 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_selector_image extends we_selector_document{

	public function __construct(int $id, string $JSIDName, string $JSTextName, string $JSCommand, string $order, string $newFolderText, int $rootDirID, bool $open_doc, bool $multiple, int $startID){
		parent::__construct($id, FILE_TABLE, $JSIDName, $JSTextName, $JSCommand, $order, $newFolderText, we_base_ContentTypes::IMAGE, $rootDirID, $open_doc, $multiple, false, $startID, '', []);
	}

	protected function getFrameset(bool $withPreview = false): string{
		return '<body class="selector" onload="startFrameset();showPreview(' . $this->id . ');">' .
				we_html_element::div(['id' => 'fsheader'], $this->printHeaderHTML()) .
				we_html_element::iframe('fsbody', $this->getFsQueryString(we_selector_file::BODY), '', '', "WE().util.dialogTop(window).fsbody.document.body.style.fontSize=WE().util.dialogTop(window).document.getElementsByName('zoom')[0].value+'%';", 'preview') .
				we_html_element::div(['id' => 'fspreviewDiv']) .
				we_html_element::div(['id' => 'fsfooter'], $this->printFooterTable()) .
				we_html_element::div(['id' => 'fspath', 'class' => 'radient']) .
				we_html_element::iframe('fscmd', 'about:blank') .
				'</body>';
	}

	protected function printFooterTable(string $more = ''): string{
		//IE doesn't support slider correctly inside tables, disable this
		return parent::printFooterTable(we_base_browserDetect::isIE() ? '<input name="zoom" type="hidden"/>' : '<input type="range" style="width:120px;height:20px;" name="zoom" min="50" step="25" max="250" value="100" onchange="WE().util.dialogTop(window).fsbody.document.body.style.fontSize=this.value+\'%\';"/>');
	}

	protected function getFramsetJSFile(): string{
		return parent::getFramsetJSFile() . we_html_element::jsScript(JS_DIR . 'selectors/image_selector.js');
	}

	//FIXME: get/set view using tblFile.viewType
	protected function printHeaderTable(string $extra = '', bool $append = true): string{
		return parent::printHeaderTable('<td id="' . we_search_view::VIEW_ICONS . '" style="display:none">' . we_html_button::create_button(we_html_button::VIEW_ICON, "javascript:setview('" . we_search_view::VIEW_ICONS . "');") . '</td>
		<td id="' . we_search_view::VIEW_LIST . '">' . we_html_button::create_button(we_html_button::VIEW_LIST, "javascript:setview('" . we_search_view::VIEW_LIST . "');") . '</td>', true);
	}

	protected function setFramesetJavaScriptOptions(): void{
		parent::setFramesetJavaScriptOptions();
		$this->jsoptions['options']['view'] = we_search_view::VIEW_ICONS;
	}

}
