<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 88e1bb2a67b5 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-12-12 01:07 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_shop_shipping implements JsonSerializable{

	/** @var string */
	var $id = '';

	/** @var string */
	var $text = '';

	/** @var string[] */
	var $countries = [];

	/** @var int[] */
	var $cartValue = [];

	/** @var float[] */
	var $shipping = [];

	/** @var bool */
	var $default = false;

	public function __construct(string $id, string $text, array $countries, array $cartValue, array $shipping, bool $default){
		$this->id = $id;
		$this->text = $text;
		$this->countries = $countries;
		$this->cartValue = $cartValue;
		$this->shipping = array_map('floatval', $shipping);
		$this->default = $default;
	}

	public static function getNewEmptyShipping(): self{
		return new self('weShipping_' . md5(uniqid('', true)), g_l('modules_shop', '[new_entry]'), ['Deutschland'], [10, 20, 100], [15, 5, 0], false);
	}

	/**
	 *
	 * @return array<string,mixed>
	 */
	#[\ReturnTypeWillChange]
	public function jsonSerialize()/* :mixed */{
		return [
			'id' => $this->id,
			'text' => $this->text,
			'countries' => $this->countries,
			'cartValue' => $this->cartValue,
			'shipping' => $this->shipping,
			'default' => (bool) $this->default,
		];
	}

	public static function initFromJson(array $json): self{
		return new self($json['id'], $json['text'], $json['countries'], $json['cartValue'], $json['shipping'], $json['default']);
	}

}
