<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: d16c20cbcaf9 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2023-01-19 12:06 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_customerResetPassword extends we_weTag_tag{

	public static $status = -1;

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['type' => false], __CLASS__);
//cleanup table
		DB_WE::$inst->query('DELETE FROM ' . PWDRESET_TABLE . ' WHERE expires<NOW()');

		$required = array_unique(self::getAttribute('required', $attribs, [], we_base_request::STRING_LIST));
		$loadFields = array_unique(self::getAttribute('loadFields', $attribs, [], we_base_request::STRING_LIST));
		$pwdRegex = self::getAttribute('passwordRule', $attribs, '', we_base_request::RAW);

//set dates
		foreach(we_base_util::convertDateInRequest('s') as $name => $date){
			$_REQUEST['s'][$name] = $date->format(we_database_base::DATE_TIME_FORMAT);
		}

		switch($type = self::getAttribute('type', $attribs, '', we_base_request::STRING)){
			default:
				return self::parseError('no type set', __CLASS__);
			case 'direct':
				if(!isset($_REQUEST['s'])){
					self::$status = we_customer_customer::PWD_FIELD_NOT_SET;
					return '';
				}
				return self::setDirecPassword($required, $loadFields, $pwdRegex);
			case 'emailPassword'://preset Password
				if(!isset($_REQUEST['s'])){
					self::$status = we_customer_customer::PWD_FIELD_NOT_SET;
					return '';
				}
			case 'email'://no preset password
				if(count($required) < 1){
					return self::parseError('For security reasons: in email mode, attribute <b>required</b> needs at least one field!', __CLASS__);
				}
				$customerEmailField = self::getAttribute('customerEmailField', $attribs, '', we_base_request::STRING);
				$pwd = ($type === 'emailPassword'?we_base_request::_(we_base_request::RAW_CHECKED, 's', '', 'Password'):'');
				if(($type === 'emailPassword' && !self::checkPwds($pwdRegex)) || !($uid = self::checkRequired($required, $loadFields, $customerEmailField, $internal))){
					self::$status = self::$status ?: we_customer_customer::PWD_FIELD_NOT_SET;
					return '';
				}
				$_SESSION['webuser']['WE_token'] = substr(md5(uniqid('', true)), 0, 25);
				DB_WE::$inst->query('REPLACE INTO ' . PWDRESET_TABLE . ' SET ' . we_database_base::arraySetter(['ID' => $uid,
							'UserTable' => ($internal ? 'tblUser' : 'tblWebUser'),
							'password' => $pwd ? we_customer_customer::cryptPassword($pwd) : '',
							'expires' => sql_function('NOW()+ INTERVAL ' . self::getAttribute('expireToken', $attribs, 3600, we_base_request::INT) . ' SECOND'),
							'token' => $_SESSION['webuser']['WE_token']
				]));
				self::$status = we_customer_customer::PWD_ALL_OK;

				break;
			case 'resetFromMail'://called from "email" after link is clicked
				//if optional required field is given, check them
				if(!empty($required) && !self::checkRequired($required, $loadFields)){
					self::$status = self::$status ?: we_customer_customer::PWD_FIELD_NOT_SET;
					return '';
				}
				$user = we_base_request::int('user');
				//check token && if password present; expired logins are already deleted
				$data = getHash('SELECT ID,password FROM ' . PWDRESET_TABLE . ' WHERE UserTable="' . ($internal ? 'tblUser' : 'tblWebUser') . '" AND ID=' . $user . ' AND token="' . DB_WE::$inst->escape(we_base_request::string('token')) . '"');

				if(!$data){
					self::$status = we_customer_customer::PWD_TOKEN_INVALID;
					DB_WE::$inst->query('DELETE FROM ' . PWDRESET_TABLE . ' WHERE UserTable="' . ($internal ? 'tblUser' : 'tblWebUser') . '" AND ID=' . $user);
					return '';
				}
				//if no pwd is set, check if passwords are given by request
				if(!$data['password']){
					if(!self::checkPwds($pwdRegex)){
						//error is set by checkPwds
						return '';
					}
					//set password from request, pwd in db was empty
					$data['password'] = ($internal ?
							we_users_user::makeSaltedPassword(we_base_request::_(we_base_request::RAW_CHECKED, 's', '', 'Password')) :
							we_customer_customer::cryptPassword(we_base_request::_(we_base_request::RAW_CHECKED, 's', '', 'Password'))
							);
				}
				if($internal){
					DB_WE::$inst->query('UPDATE ' . USER_TABLE . ' SET passwd="' . DB_WE::$inst->escape($data['password']) . '" WHERE LoginDenied=0 AND ID=' . $data['ID']);
					if(DB_WE::$inst->affected_rows()){
						self::$status = we_customer_customer::PWD_UNKNOWN_ERROR;
						DB_WE::$inst->query('UPDATE ' . FAILED_LOGINS_TABLE . ' SET isValid="false" WHERE UserTable="tblUser" AND Username=(SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . $data['ID'] . ')');
					}
				} else {
					//ok, we have a password, all (optional requirements are met) & token was valid
					DB_WE::$inst->query('UPDATE ' . CUSTOMER_TABLE . ' SET Mod_Date=Mod_Date,Password="' . DB_WE::$inst->escape($data['password']) . '" WHERE LoginDenied=0 AND ID=' . $data['ID']);
					if(DB_WE::$inst->affected_rows()){
						self::$status = we_customer_customer::PWD_UNKNOWN_ERROR;
						DB_WE::$inst->query('UPDATE ' . FAILED_LOGINS_TABLE . ' SET isValid="false" WHERE UserTable="tblWebUser" AND Username=(SELECT Username FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . $data['ID'] . ')');
					}
				}

				DB_WE::$inst->query('DELETE FROM ' . PWDRESET_TABLE . ' WHERE UserTable="' . ($internal ? 'tblUser' : 'tblWebUser') . '" AND ID=' . $data['ID']);
				self::$status = we_customer_customer::PWD_ALL_OK;
				break;
		}
		return '';
	}

	protected static function setDirecPassword(array $required, array $loadFields, string $pwdRegex): string{
		if(count($required) < 2){
			return self::parseError('For security reasons: in direct mode, attribute <b>required</b> needs at least two different fields!', __CLASS__);
		}
		if(!self::checkPwds($pwdRegex) || !($uid = self::checkRequired($required, $loadFields))){
			return '';
		}

		DB_WE::$inst->query('UPDATE ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter(['Password' => we_customer_customer::cryptPassword(we_base_request::string('s', '', 'Password'))]) . ' WHERE ID=' . $uid);
		self::$status = we_customer_customer::PWD_ALL_OK;
//finally remove any entries of failed logins so the user can login
		DB_WE::$inst->query('UPDATE ' . FAILED_LOGINS_TABLE . ' SET isValid="false" WHERE UserTable="tblWebUser" AND Username="' . DB_WE::$inst->escape($_SESSION['webuser']['Username']) . '"');
//what about IP blocks?!
		return '';
	}

	protected static function checkPwds(string $pwRegex): bool{
		$pwd = we_base_request::_(we_base_request::RAW_CHECKED, 's', '', 'Password');
		if(!$pwd){
			self::$status = we_customer_customer::PWD_FIELD_NOT_SET;
			return false;
		}
		if($pwd != we_base_request::_(we_base_request::RAW_CHECKED, 's', '', 'Password2')){
			self::$status = we_customer_customer::PWD_NOT_MATCH;
			return false;
		}
		if($pwRegex && !preg_match('/' . addcslashes($pwRegex, '/') . '/', $pwd)){
			self::$status = we_customer_customer::PWD_NOT_SUFFICIENT;
			return false;
		}

		return true;
	}

	/**
	 * @param string[] $required
	 * @param string[] $loadFields
	 * @param string $emailfield
	 * @param bool $internal
	 * @return int
	 */
	protected static function checkRequired(array $required, array $loadFields, string $emailfield = '', bool $internal = false): int{
		if(!$required){
			self::$status = we_customer_customer::PWD_FIELD_NOT_SET;
			return 0;
		}
		$where = [];
		foreach($required as $cur){
			if(($var = we_base_request::string('s', '', $cur))){
				$where[] = '`' . DB_WE::$inst->escape($cur) . '`="' . DB_WE::$inst->escape($var) . '"';
			} else {
				self::$status = we_customer_customer::PWD_FIELD_NOT_SET;
				return 0;
			}
		}
		$table = ($internal ? USER_TABLE : CUSTOMER_TABLE);
		$loadFields[] = 'ID';
		if(!$internal){
			$loadFields[] = 'username';
		}
		if($emailfield){
			$loadFields[] = $emailfield;
		}

		if(($uid = fI('SELECT ID FROM ' . $table . ' WHERE LoginDenied=0 AND ' . implode(' AND ', $where)))){
			$_SESSION['webuser'] = getHash('SELECT `' . implode('`,`', array_unique($loadFields)) . '` FROM ' . $table . ' WHERE LoginDenied=0 AND ID=' . $uid);
			return $uid;
		}

		self::$status = we_customer_customer::PWD_NO_SUCH_USER;
		return 0;
	}

}
