<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: e3f4c78bab7d $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-02-28 18:36 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_object extends we_weTag_tag{

	public static function parse(array $attribs, string $content): string{
		$name = self::getParserAttribute('name', $attribs, '', we_base_request::STRING);
		if($name && str_contains($name, ' ')){
			return self::parseError(sprintf(g_l('parser', '[name_with_space]'), 'object'), __CLASS__);
		}

		//dont't check if id or name is set, since it is possible to set object ID by request

		return '<?php if(' . we_weTag_parser::printTag('object', $attribs) . '){?>' . $content . '<?php }
		we_tag_listview::post(); ?>';
	}

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		if(!we_modules_info::isActive(we_modules_info::OBJECT)){
			echo self::modulMissingError('Object/DB', __CLASS__);
			return false;
		}
		self::$doc = we_base_WE::getWeDoc();

		$condition = self::getAttribute('condition', $attribs, 0, we_base_request::RAW);
		$classid = self::getAttribute('classid', $attribs, 0, we_base_request::INT);
		$we_oid = self::getAttribute('id', $attribs, 0, we_base_request::INT);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$triggerid = self::getAttribute('triggerid', $attribs, 0, we_base_request::INT);
		$searchable = self::getAttribute('searchable', $attribs, false, we_base_request::BOOL);
		$hidedirindex = self::getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);
		$objectseourls = self::getAttribute('objectseourls', $attribs, TAGLINKS_OBJECTSEOURLS, we_base_request::BOOL);

		if($name){
			//handle listview of documents
			$we_oid = (int) (we_tag_listview::$current && we_tag_listview::$current instanceof we_listview_base && we_tag_listview::$current->f($name) ?
					we_tag_listview::$current->f($name) :
					(self::$doc->getElementI($name) ?:
					(self::$doc->getElementS($name) ?:
					$we_oid)
					));

			if(we_base_WE::$editmode){
				$size = 5 * intval(self::getAttribute('size', $attribs, 30, we_base_request::UNIT));
				$fieldname = self::getAttribute('text', $attribs, self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING), we_base_request::STRING);
				self::edit($name, $fieldname, $we_oid, $classid, $size);
			}
		}

		$we_oid = $we_oid ?:
//Fix #10526 check if objectID is given by request
				(we_base_request::int(WEOBJ_PARAM) ?: we_base_request::int('we_oid', 0));

		//Fix #10609 we need classID now!
		$classid = $classid ?: ($we_oid ? fI('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=0 AND ID=' . $we_oid) : 0);

		if($we_oid && $classid){
			$unique = md5(uniqid(__CLASS__, true));
			$lv = new we_listview_object($unique, 1, 0, '', false, $classid, [], false, 'ID=' . $we_oid . ($condition ? ' AND ' . $condition : ''), $triggerid, 0, $searchable, '', '', new DateTime(), '', [
					], [
					], 'true', 0, [], [], [], $hidedirindex, $objectseourls, true);
			$avail = $lv->next_record();

			if($avail){
				if(we_editor_functions::isSeeMode()){
					echo we_gui_SEEM::getSeemAnchors($we_oid, 'object');
				}
			}
			we_tag_listview::pre($lv);
			return $avail;
		}
		we_tag_listview::pre(new we_listview_pseudo());
		return false;
	}

	protected static function edit(string $name, string $fieldName, int $we_oid, int $classid, int $size): void{
		$rootDirID = ($classid ? fI('SELECT ID FROM ' . OBJECT_FILES_TABLE . ' WHERE IsClassFolder=1 AND TableID=' . $classid) : 0);
		$path = id2path($we_oid, OBJECT_FILES_TABLE);
		$textname = 'we_' . self::$doc->Name . '_' . we_contents_base::INTERNAL_VAR . '[' . $name . '_path]';
		$idname = 'we_' . self::$doc->Name . '_object[' . $name . '#' . we_contents_base::ETYPE_NUMBER . ']';
		$classPath = id2path($classid, OBJECT_TABLE);

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId($name . we_base_file::getUniqueId(), $classPath)
				->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE])
				->setInput($textname, ($path === '/' ? $classPath : $path))
				->setResult($idname, (string) ($path === '/' ? 0 : $we_oid))//non existent objects
				->setMaxResults(10)
				->setSelector(we_gui_suggest::DocSelector)
				->setTable(OBJECT_FILES_TABLE)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'reload_hot_editpage',rootDirID:'" . $rootDirID . "',filter:WE().consts.contentTypes.OBJECT_FILE,open_doc:" . (we_base_permission::hasPerm([
									we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]) ? 0 : 1) . "})"))
				->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:if(parseInt(document.forms[0].elements['" . $idname . "'].value) !== 0){WE().layout.weEditorFrameController.openDocument(WE().consts.tables.OBJECT_FILES_TABLE, document.forms[0].elements['" . $idname . "'].value,'');}", '', '', !$we_oid))
				->setCreateButton($classid ? we_html_button::create_button(we_html_button::ADD, "javascript:top.we_cmd('new',WE().consts.tables.OBJECT_FILES_TABLE,'',WE().consts.contentTypes.OBJECT_FILE," . $classid . ");") : '')
				->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.forms[0].elements['" . $idname . "'].value=0;document.forms[0].elements['" . $textname . "'].value='';_EditorFrame.setEditorIsHot(false);we_cmd('reload_editpage');"))
				->setWidth($size);
		?>
		<table class="we we_object weEditTable">
			<tr>
				<td class="bold"><?= $fieldName; ?></td>
				<td><?= $weSuggest->getHTML(); ?></td>
			</tr>
		</table><?php
	}

}
