<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 5ec688e60ee3 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-12-02 15:41 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_sessionStart extends we_weTag_tag{

	/** @var bool */
	public static $started = false;

	/** @var int */
	public static $logout = 0;

	/** @var string */
	private static $username = '';

	/** @var bool */
	protected static $loginDenied = false;

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		if(we_base_WE::$mainDoc['doc']->InWebEdition ||
				we_base_WE::$editmode ||
				we_base_WE::$schedulerActive ||
				we_base_WE::$versioningActive ||
				self::$started
		){
			return '';
		}
		self::$started = true;

		if(!isset($_SESSION)){
			we_base_WE::$session = we_base_sessionHandler::inst();
		}
		if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
			return '';
		}

		if(!empty($_REQUEST['we_webUser_logout'])){
			self::doLogout();
			return '';
		}
		self::$username = we_base_request::string('s', '', 'Username');

		$SessionAutologin = 0;

		if(!isset($_SESSION['webuser'])){
			$_SESSION['webuser'] = ['registered' => false];
		}
		$persistentlogins = self::getAttribute('persistentlogins', $attribs, false, we_base_request::BOOL);
		if(empty($_SESSION['webuser']['registered']) && isset($_REQUEST['s']['Username']) && isset($_REQUEST['s']['Password']) && !(isset($_REQUEST['s']['ID'])) && !isset($_REQUEST['s']['Password2'])//if set, we assume it is a password reset or use of an forgotten password routine, so we don't try to do an login
		){
			DB_WE::$inst->query('DELETE FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND LoginDate<(NOW() - INTERVAL ' . we_base_constants::LOGIN_FAILED_HOLDTIME . ' DAY)');
			$checkPassword = true;
			$hook = new we_hook_handler('customer_preLogin', ['customer' => &$_REQUEST['s'], 'type' => 'normal', 'tagname' => 'sessionStart', 'checkPassword' => &$checkPassword,]);
			$hook->executeHook();

			if(!self::startdoLogin($persistentlogins, $SessionAutologin, !$checkPassword)){
				self::handleFailedLogin();
			} else {
				DB_WE::$inst->query('UPDATE ' . FAILED_LOGINS_TABLE . ' SET isValid="false" WHERE UserTable="tblWebUser" AND Username="' . DB_WE::$inst->escape(self::$username) . '"');
				//change session ID to prevent session
				we_base_sessionHandler::makeNewID();
				$hook = new we_hook_handler('customer_Login', ['customer' => &$_SESSION['webuser'], 'type' => 'normal', 'tagname' => 'sessionStart']);
				$hook->executeHook();
			}
			unset($_REQUEST['s']['Password']);
		}
		if($persistentlogins && (!$_SESSION['webuser']['registered']) && isset($_COOKIE['_we_autologin'])){
			if(!self::startdoAutoLogin()){
				//in case login with cookie failed, remove the cookie
				if(version_compare(PHP_VERSION, '7.3.0') >= 0){
					$cookie = session_get_cookie_params();
					setcookie('_we_autologin', '', ['expires' => (time() - 3600), 'path' => '/', 'secure' => we_http_util::isHttps(), 'httponly' => true, 'samesite' => $cookie['samesite'] ?: 'strict']);
				} else {
					setcookie('_we_autologin', '', (time() - 3600), '/');
				}
				self::handleFailedLogin();
			} else {
				we_base_sessionHandler::makeNewID();
			}
		}

		if(!empty($_SESSION['webuser']['registered'])){
			$webuser = &$_SESSION['webuser'];
			'@phan-var array{ID:int,Username:string,LastAccess:int} $webuser';
			$webUserID = (int) $webuser['ID'];
			if($webUserID && !empty($webuser['Username'])){
				if(((int) $webuser['LastAccess']) + 60 < time()){
					DB_WE::$inst->query('UPDATE ' . CUSTOMER_TABLE . ' SET Mod_Date=Mod_Date,LastAccess_Date=NOW() WHERE ID=' . $webUserID);
					$_SESSION['webuser']['LastAccess'] = time();
					$_SESSION['webuser']['LastAccess_Date'] = date('Y-m-d H:i:s');
				}
			}

			if(self::getAttribute('onlinemonitor', $attribs, false, we_base_request::BOOL)){
				DB_WE::$inst->query('DELETE FROM ' . CUSTOMER_SESSION_TABLE . ' WHERE LastAccess_Date<(NOW() - INTERVAL 1 HOUR)');
				$monitorgroupfield = self::getAttribute('monitorgroupfield', $attribs, '', we_base_request::STRING);
				$doc = self::getDocForTag(self::getAttribute('monitordoc', $attribs, '', we_base_request::STRING), false);

				$WebUserGroup = ($monitorgroupfield && $_SESSION['webuser'][$monitorgroupfield] ? $_SESSION['webuser'][$monitorgroupfield] : 'we_guest');
				$sessID = (preg_match('|^([a-f0-9]){32,40}$|', session_id()) ? str_pad(session_id(), 40, '0') : '00'/* sessid is invalid */);

				DB_WE::$inst->query('INSERT INTO ' . CUSTOMER_SESSION_TABLE . ' SET ' .
						we_database_base::arraySetter([
							'SessionID' => sql_function('x\'' . $sessID . '\''),
							'SessionIp' => inet_pton(strstr($_SERVER['REMOTE_ADDR'], ':') ? $_SERVER['REMOTE_ADDR'] : '::ffff:' . $_SERVER['REMOTE_ADDR']),
							'WebUserID' => $webUserID,
							'WebUserGroup' => $WebUserGroup,
							'Browser' => $_SERVER['HTTP_USER_AGENT'] ?? '',
							'Referrer' => $_SERVER['HTTP_REFERER'] ?? '',
							'LastLogin_Date' => sql_function('NOW()'),
							'PageID' => $doc->ID,
							'SessionAutologin' => $SessionAutologin
						]) . ' ON DUPLICATE KEY UPDATE ' . we_database_base::arraySetter([
							'PageID' => $doc->ID,
							'WebUserID' => $webUserID,
							'WebUserGroup' => $WebUserGroup,
				]));
			}
		}
		//remove sessions consisting only of webuser[registered]
		if(!empty($_SESSION['webuser']) && count($_SESSION['webuser']) == 1){
			unset($_SESSION['webuser']);
		}
		return '';
	}

	protected static function doLogout(): void{
		if(!empty($_SESSION['webuser']['registered']) && !empty($_SESSION['webuser']['ID'])){
			if(!empty($_SESSION['webuser']['AutoLoginID'])){
				DB_WE::$inst->query('DELETE FROM ' . CUSTOMER_AUTOLOGIN_TABLE . ' WHERE AutoLoginID=x\'' . sha1($_SESSION['webuser']['AutoLoginID']) . '\'');
				if(version_compare(PHP_VERSION, '7.3.0') >= 0){
					$cookie = session_get_cookie_params();
					setcookie('_we_autologin', '', ['expires' => (time() - 3600), 'path' => '/', 'httponly' => true, 'samesite' => $cookie['samesite'] ?: 'strict']);
				} else {
					setcookie('_we_autologin', '', (time() - 3600), '/');
				}
			}
			//we want to know who was logged out, so keep this info
			self::$logout = $_SESSION['webuser']['ID'];
		}
		unset($_SESSION['s'], $_REQUEST['s']);
		if(SECURITY_DELETE_SESSION){
			we_base_sessionHandler::makeNewID(true);
		}
		$_SESSION['webuser'] = [
			'registered' => false
		];
	}

	//keep this public
	public static function handleFailedLogin(): void{
		$_SESSION['webuser'] = [
			'registered' => false,
			'loginfailed' => we_users_user::INVALID_CREDENTIALS
		];
		if(!self::$loginDenied){
			we_users_user::logLoginFailed('tblWebUser', self::$username);
		}
		sleep(SECURITY_DELAY_FAILED_LOGIN);
		//make it harder to guess salt/password
		usleep(rand(0, 1000000));
		we_html_tools::setHttpCode(403);

		if(
				fI('SELECT COUNT(1) FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND Username="' . DB_WE::$inst->escape(self::$username) . '" AND isValid="true" AND LoginDate >(NOW() - INTERVAL ' . intval(SECURITY_LIMIT_CUSTOMER_NAME_HOURS) . ' hour)') >= intval(SECURITY_LIMIT_CUSTOMER_NAME) ||
				fI('SELECT COUNT(1) FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND IP="' . $_SERVER['REMOTE_ADDR'] . '" AND LoginDate >(NOW() - INTERVAL ' . intval(SECURITY_LIMIT_CUSTOMER_IP_HOURS) . ' hour)') >= intval(SECURITY_LIMIT_CUSTOMER_IP)
		){
			//don't serve user
			if(SECURITY_LIMIT_CUSTOMER_REDIRECT){
				$_SESSION['webuser']['loginfailed'] = we_users_user::MAX_LOGIN_COUNT_REACHED;
				unset($_REQUEST['s']);
				if(($path = id2path(SECURITY_LIMIT_CUSTOMER_REDIRECT, FILE_TABLE))){
					include(WEBEDITION_PATH . '../' . $path);
					exit();
				}
			}
			echo 'Dear customer, our service is currently not available. Please try again later. Thank you.<br/>' .
			'Sehr geehrter Kunde, aus Sicherheitsgründen ist ein Login derzeit nicht möglich! Bitte probieren Sie es später noch ein mal. Vielen Dank';

			exit();
		}
	}

	/** Function made available to check externally if userlogin is denied */
	public static function startCheckDenied(): bool{
		if(
				fI('SELECT COUNT(1) FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND Username="' . DB_WE::$inst->escape(self::$username) . '" AND isValid="true" AND LoginDate>(NOW() - INTERVAL ' . intval(SECURITY_LIMIT_CUSTOMER_NAME_HOURS) . ' hour)') >= intval(SECURITY_LIMIT_CUSTOMER_NAME) ||
				fI('SELECT COUNT(1) FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblWebUser" AND IP="' . $_SERVER['REMOTE_ADDR'] . '" AND LoginDate>(NOW() - INTERVAL ' . intval(SECURITY_LIMIT_CUSTOMER_IP_HOURS) . ' hour)') >= intval(SECURITY_LIMIT_CUSTOMER_IP)
		){
			self::$loginDenied = true;
			return true;
		}
		return false;
	}

	public static function checkPassword(string $pwd, array $user = []): bool{
		$user = $user ?: getHash('SELECT ID,Password FROM ' . CUSTOMER_TABLE . ' WHERE Password!="" AND LoginDenied=0 AND Username="' . DB_WE::$inst->escape(self::$username) . '"');
		return $user && we_customer_customer::comparePassword($user['Password'], $pwd, $user['ID']);
	}

	//keep this public!
	public static function startdoLogin(bool $persistentlogins, int &$SessionAutologin, bool $externalPasswordCheck = false): bool{ //FIXME: check for last time =>(cuncurrent logins)
		self::$username = we_base_request::string('s', '', 'Username');
		$pwd = we_base_request::_(we_base_request::RAW, 's', '', 'Password');
		if(!self::$username || !$pwd || self::startCheckDenied()){
			return false;
		}

		$wasRegistered = !empty($_SESSION['webuser']['registered']) && $_SESSION['webuser']['registered'];
		$u = getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE Password!="" ' . ($externalPasswordCheck ? '' : 'AND LoginDenied=0') . ' AND Username="' . DB_WE::$inst->escape(self::$username) . '"');
		if($u && ($externalPasswordCheck || self::checkPassword($pwd, $u))){
			if((SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_DBPASSWORD) == 0){
				unset($u['Password']);
			}
			$u['LastAccess'] = time();

			$_SESSION['webuser'] = array_merge($u, we_customer_customer::getEncryptedFields());
			//keep Password if known
			if(SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_PASSWORD){
				$_SESSION['webuser']['_Password'] = $pwd;
			}
			$_SESSION['webuser']['registered'] = true;
			DB_WE::$inst->query('UPDATE ' . CUSTOMER_TABLE . ' SET Mod_Date=Mod_Date,LastLogin_Date=NOW() WHERE ID=' . intval($_SESSION['webuser']['ID']));
			if($persistentlogins && self::setPersistentLogin()){
				$SessionAutologin = 1;
			}

			we_tag_ifLogin::$login = !$wasRegistered;
			return true;
		}

		return false;
	}

	protected static function setPersistentLogin(): bool{
		if(!we_base_request::bool('s', false, 'AutoLogin') || $_SESSION['webuser']['AutoLoginDenied'] == 1){
			return false;
		}
		$userID = (int) $_SESSION['webuser']['ID'];
		$_SESSION['webuser']['AutoLoginID'] = uniqid(substr(session_id(), 0, 8), true);
		DB_WE::$inst->query('INSERT INTO ' . CUSTOMER_AUTOLOGIN_TABLE . ' SET ' . we_database_base::arraySetter([
					'AutoLoginID' => sql_function('x\'' . sha1($_SESSION['webuser']['AutoLoginID']) . '\''),
					'WebUserID' => $userID,
					'LastIp' => $_SERVER['REMOTE_ADDR']
		]));

		if(version_compare(PHP_VERSION, '7.3.0') >= 0){
			$cookie = session_get_cookie_params();
			setcookie('_we_autologin', $_SESSION['webuser']['AutoLoginID'], ['expires' => (time() + we_customer_customer::CUSTOMER_AUTOLOGIN_LIFETIME), 'path' => '/', 'secure' => we_http_util::isHttps(),
				'httponly' => true, 'samesite' => $cookie['samesite'] ?: 'strict']);
		} else {
			setcookie('_we_autologin', $_SESSION['webuser']['AutoLoginID'], (time() + we_customer_customer::CUSTOMER_AUTOLOGIN_LIFETIME), '/', '', we_http_util::isHttps(), true);
		}
		DB_WE::$inst->query('UPDATE ' . CUSTOMER_TABLE . ' SET AutoLogin=1 WHERE ID=' . $userID);
		$_SESSION['webuser']['AutoLogin'] = 1;
		return true;
	}

	protected static function startdoAutoLogin(): bool{
		$autologinSeek = (string) $_COOKIE['_we_autologin'];
		if(!empty($autologinSeek)){
			$hook = new we_hook_handler('customer_preLogin', ['customer' => &$_REQUEST['s'], 'type' => 'autoLogin', 'tagname' => 'sessionStart']);
			$hook->executeHook();
			self::$username = we_base_request::string('s', '', 'Username');

			$wasRegistered = $_SESSION['webuser']['registered'];
			$u = getHash('SELECT u.* FROM ' . CUSTOMER_TABLE . ' u JOIN ' . CUSTOMER_AUTOLOGIN_TABLE . ' c ON u.ID=c.WebUserID WHERE u.LoginDenied=0 AND u.AutoLoginDenied=0 AND u.Password!="" AND c.AutoLoginID=x\'' . sha1($autologinSeek) . '\'');
			if($u){
				if((SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_DBPASSWORD) == 0){
					unset($u['Password']);
				}
				$u['LastAccess'] = time();

				$_SESSION['webuser'] = array_merge($u, we_customer_customer::getEncryptedFields());
				//try to decrypt password if possible
				if(SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_PASSWORD){
					// is this even needed still?
					$_SESSION['webuser']['_Password'] = $_SESSION['webuser']['_Password'] ?? we_customer_customer::decryptData((string) ($_SESSION['webuser']['Password'] ?? ''));
				}
				$_SESSION['webuser']['registered'] = true;
				$_SESSION['webuser']['AutoLoginID'] = uniqid(substr(session_id(), 0, 8), true);
				DB_WE::$inst->query('UPDATE ' . CUSTOMER_AUTOLOGIN_TABLE . ' SET ' . we_database_base::arraySetter([
							'AutoLoginID' => sql_function('x\'' . sha1($_SESSION['webuser']['AutoLoginID']) . '\''),
							'LastIp' => $_SERVER['REMOTE_ADDR'],
						]) . ' WHERE WebUserID=' . intval($_SESSION['webuser']['ID']) . ' AND AutoLoginID=x\'' . sha1($autologinSeek) . '\''
				);

				if(version_compare(PHP_VERSION, '7.3.0') >= 0){
					setcookie('_we_autologin', $_SESSION['webuser']['AutoLoginID'], ['expires' => (time() + we_customer_customer::CUSTOMER_AUTOLOGIN_LIFETIME), 'path' => '/', 'secure' => we_http_util::isHttps(),
						'httponly' => true, 'samesite' => 'strict']);
				} else {
					setcookie('_we_autologin', $_SESSION['webuser']['AutoLoginID'], (time() + we_customer_customer::CUSTOMER_AUTOLOGIN_LIFETIME), '/', '', we_http_util::isHttps(), true);
				}
				we_tag_ifLogin::$login = $wasRegistered;
				$hook = new we_hook_handler('customer_Login', ['customer' => &$_SESSION['webuser'], 'type' => 'autoLogin', 'tagname' => 'sessionStart']);
				$hook->executeHook();
				return true;
			}
		}

		return false;
	}

	public static function setUserName(string $username): void{
		self::$username = $username;
	}

}
