<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 2bdf51e3c013 $
 * $Author: Lukas Imhof $
 * $Date: 2022-03-09 18:45 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_subscribe extends we_weTag_tag{

	private static $cnt = 0;

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		$type = self::getAttribute('type', $attribs, 'email', we_base_request::STRING);
		$values = self::getAttribute('values', $attribs, [], we_base_request::STRING_LIST);
		$value = self::getAttribute('value', $attribs, '', we_base_request::RAW);
		$checked = self::getAttribute('checked', $attribs, false, we_base_request::BOOL);

		switch($type){
			case 'listCheckbox':
				$nr = ++self::$cnt;
				if($checked || (($list = we_base_request::_(we_base_request::STRING_LIST, 'we_subscribe_list__')) && in_array($nr, $list))){
					$attribs['checked'] = 'checked';
				}

				return self::htmlTag('input', ['type' => 'hidden', 'name' => 'we_use_lists__', 'value' => 1]) .
						self::htmlTag('input', array_merge(self::removeAttribs($attribs, ['name', 'type', 'value']), [
							'type' => 'checkbox',
							'name' => 'we_subscribe_list__[' . $nr . ']',
							'value' => $nr,
				]));

			case 'listSelect':
				if($values){
					$options = '';
					foreach($values as $i => $v){
						$options .= (($list = we_base_request::_(we_base_request::STRING_LIST, 'we_subscribe_list__')) && in_array($i, $list) ?
								self::htmlTag('option', ['value' => $i + 1, 'selected' => 'selected'], htmlspecialchars($v)) :
								self::htmlTag('option', ['value' => $i + 1], htmlspecialchars($v)));
					}
					return self::htmlTag('input', ['type' => 'hidden', 'name' => 'we_use_lists__', 'value' => 1]) .
							self::htmlTag('select', array_merge(self::removeAttribs($attribs, ['name', 'type', 'value', 'values', 'lists', 'maxlength', 'checked']), [
								'name' => 'we_subscribe_list__[]',
								'multiple' => 'multiple'
									]), $options, true);
				}
				return '';

			case 'htmlCheckbox':
			case 'htmlSelect':
				return '';

			case 'firstname':
				return self::htmlTag('input', array_merge(self::removeAttribs($attribs, ['name', 'type', 'value', 'values']), [
							'type' => 'text',
							'name' => 'we_subscribe_firstname__',
							'value' => we_base_request::string('we_subscribe_firstname__', $value)
				]));
			case 'salutation':
				if($values){
					$newAttribs = self::removeAttribs($attribs, ['name', 'type', 'value', 'values', 'maxlength', 'checked']);
					$name = 'we_subscribe_salutation__';
					$value = we_base_request::string('we_subscribe_salutation__', $value);
					return self::selectField($name, $value, $values, '', $newAttribs, true); //same function like <we:sessionField type="select">
				}

				return self::htmlTag('input', array_merge(self::removeAttribs($attribs, ['name', 'type', 'value', 'values']), [
							'name' => 'we_subscribe_salutation__',
							'type' => 'text',
							'value' => we_base_request::string('we_subscribe_salutation__', $value)
				]));

			case 'title':
				if($values){
					$newAttribs = self::removeAttribs($attribs, ['name', 'type', 'value', 'values', 'maxlength', 'checked']);
					$name = 'we_subscribe_title__';
					$value = we_base_request::string('we_subscribe_title__', $value);
					return self::selectField($name, $value, $values, '', $newAttribs, true); //same function like <we:sessionField type="select">
				}
				return self::htmlTag('input', array_merge(self::removeAttribs($attribs, ['name', 'type', 'value', 'values']), [
							'name' => 'we_subscribe_title__',
							'type' => 'text',
							'value' => we_base_request::string('we_subscribe_title__', $value)
				]));
			case 'lastname':
				return self::htmlTag('input', array_merge(self::removeAttribs($attribs, ['name', 'type', 'value', 'values']), [
							'type' => 'text',
							'name' => 'we_subscribe_lastname__',
							'value' => we_base_request::string('we_subscribe_lastname__', $value)
				]));

			case 'email':
			default:
				return self::htmlTag('input', array_merge(self::removeAttribs($attribs, ['name', 'type', 'value', 'values']), [
							'type' => 'email',
							'name' => 'we_subscribe_email__',
							'value' => we_base_request::_(we_base_request::EMAIL, "we_subscribe_email__", $value),
				]));
		}
	}

}
