<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 79d9ce692017 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-04-06 17:58 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category    webEdition
 * @license     http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_url extends we_weTag_tag{

	private static $urls = [];

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['id' => false], __CLASS__);
		$type = self::getAttribute('type', $attribs, 'document', we_base_request::STRING);
		$id = self::getAttribute('id', $attribs, 0, we_base_request::STRING);
		$hidedirindex = self::getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);

		if(is_numeric($id) && (isset(self::$urls[$type . $id]))){ // do only work you have never done before
			return self::$urls[$type . $id];
		}

		if($id !== 'self' && $id !== 'top' && intval($id) === 0){
			return '/';
		}
		switch($type){
			case we_base_ContentTypes::OBJECT :
				$obj = we_base_WE::$we_obj;
				'@phan-var ?we_contents_objectFile $obj';
				$objectseourls = self::getAttribute('objectseourls', $attribs, TAGLINKS_OBJECTSEOURLS, we_base_request::BOOL);
				$triggerid = self::getAttribute('triggerid', $attribs, 0, we_base_request::INT);
				$url = self::getObjectUrl($obj, (string) $id, $triggerid, $objectseourls, $hidedirindex);
				break;
			case 'document':
			default:
				$url = self::getDocumentUrl((string) $id, $hidedirindex);
				break;
		}
		$urlReplace = we_folder_base::getUrlReplacements(DB_WE::$inst, true);
		return (self::$urls[$type . $id] = ($urlReplace ? preg_replace($urlReplace, array_keys($urlReplace), $url) : $url));
	}

	protected static function getDocumentUrl(string $id, bool $hidedirindex): string{
		switch($id){
			case 'self':
			case 'top':
				$doc = self::getDocForTag($id, true);
				$docID = $doc->ID;
				break;
			default:
				$docID = $id;
				break;
		}

		$path = fS('SELECT CONCAT(Path,IF(IsFolder,"/","")' ./* (DISABLE_MEDIA_MODTIME ? '' : ',"?m=",UNIX_TIMESTAMP(Publish_Date)') .*/ ') FROM ' . FILE_TABLE . ' WHERE ID=' . intval($docID));
		if(empty($path)){
			return '';
		}
		$path_parts = pathinfo($path);
		return ($hidedirindex && seoIndexHide($path_parts['basename'])) ? ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' : $path;
	}

	protected static function getObjectUrl(?we_contents_objectFile $obj, string $id, int $triggerid, bool $objectseourls, bool $hidedirindex): string{
		$objectID = ($id === 'self' || $id === 'top') ? $obj->ID : intval($id);
		$getObject = getHash('SELECT Url,TriggerID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . intval($objectID));
		if(!$getObject){
			return '';
		}

		$pathPrefix = rtrim(
				(defined('WE_REDIRECTED_SEO') ?
				substr(WE_REDIRECTED_SEO, 0, strripos((string) WE_REDIRECTED_SEO, $getObject['Url']) ?: 0) :
				$getObject['Url']),
				'/'); //@phan-suppress-current-line PhanParamSuspiciousOrder
		$triggerDocPath = $triggerid ? id2path($triggerid) :
				($getObject['TriggerID'] ? id2path($getObject['TriggerID']) :
				(defined('WE_REDIRECTED_SEO') ? //webEdition object uses SEO-URL
				we_contents_objectFile::getNextDynDoc($pathPrefix . DEFAULT_DYNAMIC_EXT, //@phan-suppress-current-line PhanParamSuspiciousOrder
						path_to_id($pathPrefix), $obj->getWorkspaces()) :
				(string) (parse_url(urldecode($_SERVER['REQUEST_URI']), PHP_URL_PATH) ?? '')
				)
				);

		$path_parts = pathinfo($triggerDocPath);

		if($objectseourls && $getObject['Url'] != '' && show_SeoLinks()){
			return ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' .
					($hidedirindex && seoIndexHide($path_parts['basename']) ?
					'' : $path_parts['filename'] . '/' ) . $getObject['Url'];
		}
		return ($hidedirindex && seoIndexHide($path_parts['basename']) ?
				($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/?' . WEOBJ_PARAM . '=' . $objectID :
				$triggerDocPath . '?' . WEOBJ_PARAM . '=' . $objectID
				);
	}

}
