<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: ca5ca02e784d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-03-04 01:33 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_xmlfeed extends we_weTag_tag{
	/** @var we_xml_browser[] */
	public static $xmlfeeds = [];

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		self::attributeMissing($attribs, ['name' => false, 'url' => false], __CLASS__);

		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$url = self::getAttribute('url', $attribs, '', we_base_request::URL);
		$refresh = abs(self::getAttribute('refresh', $attribs, 30, we_base_request::INT)) * 60;
		$timeout = abs(self::getAttribute('timeout', $attribs, 0, we_base_request::INT));

		$cache = we_cache_file::DOMAIN_XMLFEED . md5($url);
		try{
			$do_refresh = ($refresh ? (we_cache_file::stat($cache) + $refresh) < time() : true);
			if(!$do_refresh){
				self::$xmlfeeds[$name] = we_xml_browser::loadCache($cache);
			}
		} catch (we_exception_cache $e){
			$do_refresh = true;
		}

		try{
			if($do_refresh){
				self::$xmlfeeds[$name] = new we_xml_browser(); //this is equal to we_tag_xmlfeed
				if(self::$xmlfeeds[$name]->getFile($url, $timeout)){
					self::$xmlfeeds[$name]->saveCache($cache, (2 * $refresh)); //keep file longer, in case of timeouts
				}
			}
		} catch (we_exception_http $e){
			return '';
		}

		return '';
	}

}
