<?php
declare(strict_types=1);

/**

 * webEdition CMS
 *
 * $Rev: e9fb15a2aaef $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-15 17:36 +0100 $
 */
class we_tw_form extends we_weTag_twbase{

	public function __construct(string $tagName){
		parent::__construct($tagName, true);
		$this->NeedsEndTag = true;

		$id = new we_weTag_selectorAttribute($this, 'id', '', FILE_TABLE, we_base_ContentTypes::WEDOCUMENT);
		$name = new we_weTag_textAttribute($this, 'name', '', false, '');
		$nameid = new we_weTag_textAttribute($this, 'nameid', '', false, '');
		$method = new we_weTag_selectAttribute($this, 'method', '', [
			new we_weTag_option('get'),
			new we_weTag_option('post'),
			], false, '');
		$target = new we_weTag_choiceAttribute($this, 'target', '', [
			new we_weTag_option('_top'),
			new we_weTag_option('_parent'),
			new we_weTag_option('_self'),
			new we_weTag_option('_blank'),
			], false, false);
		$recipient = new we_weTag_textAttribute($this, 'recipient', '', true, '');
		$onsuccess = new we_weTag_selectorAttribute($this, 'onsuccess', '', FILE_TABLE, we_base_ContentTypes::WEDOCUMENT);
		$onerror = new we_weTag_selectorAttribute($this, 'onerror', '', FILE_TABLE, we_base_ContentTypes::WEDOCUMENT);
		$onmailerror = new we_weTag_selectorAttribute($this, 'onmailerror', '', FILE_TABLE, we_base_ContentTypes::WEDOCUMENT);
		$onrecipienterror = new we_weTag_selectorAttribute($this, 'onrecipienterror', '', FILE_TABLE, we_base_ContentTypes::WEDOCUMENT);
		$from = new we_weTag_textAttribute($this, 'from', '');
		$subject = new we_weTag_textAttribute($this, 'subject', '');
		$order = new we_weTag_textAttribute($this, 'order', '');
		$required = new we_weTag_textAttribute($this, 'required', '');
		$remove = new we_weTag_textAttribute($this, 'remove', '');
		$mimetype = new we_weTag_selectAttribute($this, 'mimetype', '', [
			new we_weTag_option('text/plain'),
			new we_weTag_option('text/html'),
		]);
		$confirmmail = new we_weTag_selectAttribute($this, 'confirmmail', '', we_weTag_selectAttribute::getTrueFalse());
		$forcefrom = new we_weTag_selectAttribute($this, 'forcefrom', '', we_weTag_selectAttribute::getTrueFalse());
		$preconfirm = new we_weTag_textAttribute($this, 'preconfirm', '');
		$postconfirm = new we_weTag_textAttribute($this, 'postconfirm', '');
		$doctype = new we_weTag_sqlRowAttribute($this, 'doctype', '', DOC_TYPES_TABLE, false, 'Text');
		$docTyppeId = new we_weTag_sqlRowAttribute($this, 'doctypeID', '', DOC_TYPES_TABLE, false);
		$categories = new we_weTag_multiSelectorAttribute($this, 'categories', '', CATEGORY_TABLE, '', 'Path', false, we_modules_info::CATEGORY);
		$tid = new we_weTag_selectorAttribute($this, 'tid', '', TEMPLATES_TABLE, we_base_ContentTypes::TEMPLATE);
		$classid = new we_weTag_selectorAttribute($this, 'classid', '', OBJECT_TABLE, we_base_ContentTypes::OBJECT, false, we_modules_info::OBJECT);
		$parentid = new we_weTag_selectorAttribute($this, 'parentid', '', OBJECT_FILES_TABLE, we_weTag_selectorAttribute::FOLDER, false, we_modules_info::OBJECT);
		$enctype = new we_weTag_selectAttribute($this, 'enctype', '', [
			new we_weTag_option('application/x-www-form-urlencoded'),
			new we_weTag_option('multipart/form-data'),
		]);
		$params = new we_weTag_textAttribute($this, 'params', '');

		if(defined('FORMMAIL_VIAWEDOC') && FORMMAIL_VIAWEDOC == 1){
			$this->TypeAttribute = new we_weTag_typeAttribute($this, 'type', [
				new we_weTag_option('-', false, '', [$id, $name, $nameid, $method, $target, $enctype], []),
				new we_weTag_option('document', false, '', [$id, $name, $nameid, $method, $target, $doctype, $docTyppeId, $tid, $enctype, $params], []), //fixme use $doctype || $docTyppeId
				new we_weTag_option('formmail', false, '', [$id, $name, $nameid, $method, $target, $recipient, $onsuccess, $onerror, $onmailerror, $onrecipienterror, $from,
					$subject, $order, $required, $remove, $mimetype, $confirmmail, $forcefrom, $preconfirm, $postconfirm, $params], [$recipient]),
				new we_weTag_option('object', false, we_modules_info::OBJECT, [$id, $name, $nameid, $method, $target, $categories, $classid, $parentid, $enctype, $params], [
					$classid]),
				new we_weTag_option('search', false, '', [$id, $name, $nameid, $method, $target, $params]),
				new we_weTag_option('shopliste', false, we_modules_info::SHOP, [$id, $nameid, $method, $target, $params])
				], false, '');
		} else {
			$this->TypeAttribute = new we_weTag_typeAttribute($this, 'type', [
				new we_weTag_option('-', false, '', [$id, $name, $nameid, $method, $target, $enctype], []),
				new we_weTag_option('document', false, '', [$id, $name, $nameid, $method, $target, $doctype, $docTyppeId, $tid, $enctype, $params], []), //fixme use $doctype || $docTyppeId
				new we_weTag_option('formmail', false, '', [$name, $nameid, $method, $target, $recipient, $onsuccess, $onerror, $onmailerror, $onrecipienterror, $from, $subject,
					$order, $required, $remove, $mimetype, $confirmmail, $forcefrom, $preconfirm, $postconfirm, $params], [$recipient]),
				new we_weTag_option('object', false, we_modules_info::OBJECT, [$id, $name, $nameid, $method, $target, $categories, $classid, $parentid, $enctype, $params], [
					$classid]),
				new we_weTag_option('search', false, '', [$id, $name, $nameid, $method, $target, $params]),
				new we_weTag_option('shopliste', false, we_modules_info::SHOP, [$id, $nameid, $method, $target, $params])
				], false, '');
		}

		$this->Attributes = [
			$id,
			$name,
			$nameid,
			$method,
			$target,
			$params,
			$recipient,
			$onsuccess,
			$onerror,
			$onmailerror,
			$from,
			$subject,
			$order,
			$required,
			$remove,
			$mimetype,
			$confirmmail,
			$forcefrom,
			$preconfirm,
			$postconfirm,
			$doctype,
			$docTyppeId,
			$categories,
			$tid,
			$classid,
			$parentid,
			$enctype,
		];
	}

}
