<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f860e3d063da $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-12-22 22:11 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_versions_version{

	/**
	 * fields from tblFile and tblObjectFiles which can be modified
	 */
	const MOD_FIELDS = [//FIXME: use dbset for this!
		'status',
		'ParentID',
		'Text',
		'IsSearchable',
		'Category',
		'CreatorID',
		'RestrictOwners',
		'Owners',
		'OwnersReadOnly',
		'Language',
		'WebUserID',
		'documentElements',
		'documentScheduler',
		'documentCustomFilter',
		'TemplateID',
		'Filename',
		'Extension',
		'IsDynamic',
		'DocType',
		'Workspaces',
		'Templates',
		'InGlossar'
	];

	/** @var int */
	protected $ID;

	/** @var int */
	protected $documentID;

	/** @var string */
	protected $documentTable;
	protected $documentElements;
	protected $documentScheduler;
	protected $documentCustomFilter;
	protected $logDate;
	protected $status;

	/** @var int */
	protected $version = 1;

	/** @var ?string */
	protected $binaryPath = null;

	/** @var int */
	protected $modifierID = 0;

	/** @var string */
	protected $ContentType = '';

	/** @var int */
	protected $ParentID = 0;

	/** @var DateTime */
	protected $Creation_Date;

	/** @var int */
	protected $CreatorID = 0;

	/** @var string */
	protected $Path;

	/** @var ?int */
	protected $TemplateID = null;

	/** @var ?string */
	protected $Filename = null;

	/** @var ?string */
	protected $Extension = null;

	/** @var bool */
	protected $IsDynamic = false;

	/** @var bool */
	protected $IsSearchable = false;

	/** @var ?int */
	protected $DocType = null;

	/** @var ?string */
	protected $Category = null;

	/** @var bool */
	protected $RestrictOwners = false;

	/** @var ?string */
	protected $Owners = null;

	/** @var ?string */
	protected $OwnersReadOnly = null;

	/** @var string */
	protected $Language = 'de_DE';

	/** @var ?int */
	protected $WebUserID = null;

	/** @var ?string */
	protected $Workspaces = null;

	/** @var ?string */
	protected $Templates = null;

	/** @var ?int */
	protected $MasterTemplateID = null;

	/** @var ?int */
	protected $TableID = null;

	/** @var ?bool */
	protected $IsClassFolder = null;

	/** @var bool */
	protected $active = true;

	/** @var ?int */
	protected $resetFromVersion = null;

	/**
	 * ContentTypes which apply for versioning
	 * all except classes, templates and folders
	 */
	public static function getContentTypesVersioning(): array{
		static $contentTypes = [];
		if(!$contentTypes){
			$contentTypes = ['all'];
			foreach(we_base_ContentTypes::getContentTypes() as $k){
				switch($k){
					case we_base_ContentTypes::OBJECT:
					case we_base_ContentTypes::FOLDER_DOCUMENT:
					case we_base_ContentTypes::FOLDER_OBJECTFILE:
						break;
					default:
						$contentTypes[] = $k;
				}
			}
		}
		return $contentTypes;
	}

	/**
	 * set first document object if no versions exist
	 * for contentType = text/webedition
	 */
	public static function setInitialDocObject(we_contents_root $obj): void{
		if($obj->ID && we_versions_version::CheckPreferencesCtypes($obj->ContentType)){
			$_SESSION['weS']['versions']['versionToCompare'][$obj->Table][$obj->ID] = self::getHashValue(self::removeUnneededCompareFields(self::objectToArray($obj)));

			if(!self::versionsExist($obj->ID, $obj->Table, $obj->ContentType)){
				$_SESSION['weS']['versions']['initialVersions'] = true;
				self::save($obj);
			}
		}
	}

	/**
	 * looks if versions exist for the document
	 */
	private static function versionsExist(int $id, string $table, string $contentType): bool{//FIXME: we need a documenttable!
		return fB('SELECT 1 FROM ' . VERSIONS_TABLE . ' WHERE documentTable="' . stripTblPrefix($table) . '" AND documentId=' . $id . ' AND ContentType="' . DB_WE::$inst->escape($contentType) . '" LIMIT 1');
	}

	/**
	 * get versions of one document / object
	 * @param int $id
	 * @param string $table
	 * @param string $where
	 * @param string $order
	 * @return array<int,array{modifierID:int,modifiedFields:string,status:string,version:int,active:int,src:string,resetFromVersion:int,ID:int,documentID:int,version:int,documentTable:string,ContentType:string,_logDate:string}> of version-records of one document / object
	 */
	static function loadVersionsOfId(int $id, string $table, string $where = '', string $order = ''): array{
		$db = DB_WE::$inst;
		//@phan-suppress-next-line PhanPartialTypeMismatchReturn
		return $db->getAllq('SELECT
			modifierID,
			modifiedFields,
			status,
			version,
			active,
			src,
			resetFromVersion,
			ID,
			documentID,
			version,
			documentTable,
			ContentType,
			DATE_FORMAT(logDate,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS _logDate
FROM ' . VERSIONS_TABLE . '
WHERE
	documentID=' . $id . ' AND
	documentTable="' . $db->escape(stripTblPrefix($table)) . '" ' .
						($where ? ' AND ' . $where : '') . '
ORDER BY ' . ($order ?: 'version DESC'));
	}

	/**
	 * get one version of document / object
	 * @return array{ID:int,documentID:int,documentTable:string,documentElements:string,documentScheduler:string,documentCustomFilter:string,logDate:string,status:string,version:int,binaryPath:string,modifiedFields:string,modifierID:int,ContentType:string,ParentID:int,Creation_Date:string,CreatorID:int,Path:string,TemplateID:int,Filename:string,Extension:string,IsDynamic:int,IsSearchable:int,DocType:int,Category:string,RestrictOwners:int,Owners:string,OwnersReadOnly:string,Language:string,WebUserID:int,Workspaces:string,Templates:string,MasterTemplateID:int,TableID:int,IsClassFolder:int,active:int,src:string,resetFromVersion:int,InGlossar:int,_logDate:string} of version-records of one document / object
	 */
	public static function loadVersion(string $where = ''): array{
		return getHash('SELECT *,DATE_FORMAT(logDate,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS _logDate FROM ' . VERSIONS_TABLE . ' ' . $where);
	}

	/**
	 * cases in which versions are created
	 * 1. if documents are imported
	 * 2. there exists no version-record of a document but in tblfile oder tblobjectsfile (document/object was not created new)
	 * 3. if document / object is saved, published or unpublished
	 */
	public static function save(we_contents_base $docObj, string $status = 'saved'): bool{
		$_SESSION['weS']['versions']['fromImport'] = false;
		$cmd0 = we_base_request::string('we_cmd', '', 0) ?: we_base_request::string('cmd');

		if(we_base_request::string('pnt') === 'wizcmd'){
			switch(we_base_request::string('v', '', 'type')){
				case we_exim_ExIm::TYPE_CSV:
				case we_exim_ExIm::TYPE_XML:
					break;
				default:
					if(!isset($_SESSION['weS']['ExImRefTable']) ||
							!($_SESSION['weS']['ExImRefTable']->exists(['ID' => $docObj->ID]))){
						return false;
					}
			}
			$_SESSION['weS']['versions']['fromImport'] = true;
			return we_versions_version::saveVersion($docObj);
		}
		if((!empty($_SESSION['weS']['versions']['fromScheduler'])) ||
				((we_base_request::string('type') === 'reset_versions') ||
				(!empty($_SESSION['weS']['versions']['initialVersions'])))){
			$cmd0 = 'save_document';
			if(isset($_SESSION['weS']['versions']['initialVersions'])){
				unset($_SESSION['weS']['versions']['initialVersions']);
			}
		}
		switch($cmd0){
			case 'siteImport':
			case 'import_files':
				$_SESSION['weS']['versions']['fromImport'] = true;
				return we_versions_version::saveVersion($docObj);
			default:
			case 'save_document':
			case 'unpublish':
			case 'revert_published':
			case 'ResetVersion':
			case 'PublishDocs':
			case 'ResetVersionsWizard':
				return we_versions_version::saveVersion($docObj, $status);
		}
	}

	/**
	 * apply preferences
	 */
	public static function CheckPreferencesCtypes($ct): bool{

		switch($ct){
//if folder was saved don' make versions (if path was changed of folder)
			case we_base_ContentTypes::FOLDER_DOCUMENT:
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
			case we_base_ContentTypes::FOLDER_TEMPLATE:
			case we_base_ContentTypes::FOLDER_VFILE:
				return false;
			case we_base_ContentTypes::WEDOCUMENT:
				return (bool) VERSIONING_TEXT_WEBEDITION;
			case we_base_ContentTypes::IMAGE:
				return (bool) VERSIONING_IMAGE;
			case we_base_ContentTypes::HTML:
				return (bool) VERSIONING_TEXT_HTML;
			case we_base_ContentTypes::JS:
				return (bool) VERSIONING_TEXT_JS;
			case we_base_ContentTypes::JSON:
				return (bool) VERSIONING_TEXT_JSON;
			case we_base_ContentTypes::CSS:
				return (bool) VERSIONING_TEXT_CSS;
			case we_base_ContentTypes::TEXT:
				return (bool) VERSIONING_TEXT_PLAIN;
			case we_base_ContentTypes::HTACCESS:
				return (bool) VERSIONING_TEXT_HTACCESS;
			case we_base_ContentTypes::TEMPLATE:
				return (bool) VERSIONING_TEXT_WETMPL;
			case we_base_ContentTypes::VIDEO:
				return (bool) VERSIONING_VIDEO;
			case we_base_ContentTypes::AUDIO:
				return (bool) VERSIONING_AUDIO;
			case we_base_ContentTypes::PDF:
				return (bool) VERSIONING_PDF;
			case we_base_ContentTypes::APPLICATION:
				return (bool) VERSIONING_SONSTIGE;
			case we_base_ContentTypes::XML:
				return (bool) VERSIONING_TEXT_XML;
			case we_base_ContentTypes::OBJECT_FILE:
				return (bool) VERSIONING_OBJECT;
			default:
				return false;
		}
	}

	private function CheckPreferencesTime(int $docID, string $docTable): void{
		switch($docTable){
			case TEMPLATES_TABLE:
				$prefTimeDays = (VERSIONS_TIME_DAYS_TMPL != '-1') ? VERSIONS_TIME_DAYS_TMPL : 0;
				$prefTimeWeeks = (VERSIONS_TIME_WEEKS_TMPL != '-1') ? VERSIONS_TIME_WEEKS_TMPL : 0;
				$prefTimeYears = (VERSIONS_TIME_YEARS_TMPL != '-1') ? VERSIONS_TIME_YEARS_TMPL : 0;
				$prefAnzahl = intval(VERSIONS_ANZAHL_TMPL);
				break;
			default:
				$prefTimeDays = (VERSIONS_TIME_DAYS != '-1') ? VERSIONS_TIME_DAYS : 0;
				$prefTimeWeeks = (VERSIONS_TIME_WEEKS != '-1') ? VERSIONS_TIME_WEEKS : 0;
				$prefTimeYears = (VERSIONS_TIME_YEARS != '-1') ? VERSIONS_TIME_YEARS : 0;
				$prefAnzahl = intval(VERSIONS_ANZAHL);
		}

		$prefTime = intval(($prefTimeDays ?: 0) + ($prefTimeWeeks ?: 0) + ($prefTimeYears ?: 0));
		//$dels = [];

		if($prefTime){
			$deletetime = time() - $prefTime;
//initial version always stays
			$max = fI('SELECT MAX(ID) FROM ' . VERSIONS_TABLE . ' WHERE documentId=' . $docID . ' AND documentTable="' . DB_WE::$inst->escape(stripTblPrefix($docTable)) . '"');
			$where = ' documentId=' . $docID . ' AND documentTable="' . DB_WE::$inst->escape(stripTblPrefix($docTable)) . '" AND logDate<FROM_UNIXTIME(' . $deletetime . ') AND Creation_Date!=logDate ' . ($max ? 'AND ID!=' . $max . ' ' : '');
			/* $dels = */self::deleteVersion([], $where);
		}

		$anzahl = fI('SELECT COUNT(1) FROM ' . VERSIONS_TABLE . ' WHERE documentId=' . $docID . ' AND documentTable="' . DB_WE::$inst->escape(stripTblPrefix($docTable)) . '"');

		if($prefAnzahl && $anzahl > $prefAnzahl){
			$toDelete = $anzahl - $prefAnzahl;
			$ids = DB_WE::$inst->getAllq('SELECT ID FROM ' . VERSIONS_TABLE . ' WHERE documentId=' . $docID . ' AND documentTable="' . DB_WE::$inst->escape(stripTblPrefix($docTable)) . '" ORDER BY version ASC LIMIT ' . intval($toDelete), true);
			self::deleteVersion($ids);
			/* $dels = array_merge($dels, self::deleteVersion($ids)); */
		}
		/* don't log autodelete
		 * if($dels){
		  $versionslog = new we_versions_log();
		  $versionslog->saveVersionsLog($dels, we_versions_log::VERSIONS_DELETE);
		  } */
	}

	/**
	 * make new version-entry in DB
	 */
	public static function saveVersion(we_contents_base $documentObj, string $status = 'saved'): bool{
//preferences
		if(!($documentObj instanceof we_document_base) || !self::CheckPreferencesCtypes($documentObj->ContentType)){
			return false;
		}
		$document = self::objectToArray($documentObj);
		$version = new self();

		if(isset($document['documentCustomerFilter']) && is_object($document['documentCustomerFilter'])){
			$document['documentCustomerFilter'] = self::objectToArray($document['documentCustomerFilter']);
		}

//preferences
		if(we_base_request::string('we_cmd', '', 0) === 'save_document' &&
				we_base_request::bool('we_cmd', false, 5)){
			$status = 'published';
		}

		switch($documentObj->ContentType){
			case we_base_ContentTypes::OBJECT_FILE:
			case we_base_ContentTypes::WEDOCUMENT:
			case we_base_ContentTypes::HTML:
				break;
			case we_base_ContentTypes::TEMPLATE:
				if(VERSIONS_CREATE_TMPL){//true if save version button is active
					break;
				}
			default:
				$status = 'saved';
		}
		switch($status){
			case 'unpublished':
			case 'deleted':
				break;
			default:
				if(self::IsScheduler()){
					$status = 'published';
				}
		}
		if(!empty($_SESSION['weS']['versions']['doPublish'])){
			$status = 'published';
		}
		$tblversionsFields = array_flip(DB_WE::$inst->metaFieldNames(VERSIONS_TABLE));

		switch($document['ContentType']){
			case we_base_ContentTypes::TEMPLATE:
				//templates don't have categories
				if(VERSIONS_CREATE_TMPL){
					if($status != 'published' && !we_base_request::bool('we_cmd', true, 5)){ //FIXME: this should not be handled here!
						return false;
					}
				}
				unset($tblversionsFields['Category'], $tblversionsFields['TemplateID'], $tblversionsFields['DocType'], $tblversionsFields['TableID'], $tblversionsFields['IsClassFolder'], $tblversionsFields['InGlossar']);
				break;
			case we_base_ContentTypes::OBJECT:
				unset($tblversionsFields['Extension'], $tblversionsFields['TableID'], $tblversionsFields['IsClassFolder'], $tblversionsFields['InGlossar']);
				break;
			case we_base_ContentTypes::OBJECT_FILE:
				unset($tblversionsFields['Extension'], $tblversionsFields['InGlossar']);
				break;
			case we_base_ContentTypes::HTML:
				unset($tblversionsFields['TemplateID'], $tblversionsFields['DocType'], $tblversionsFields['MasterTemplateID'], $tblversionsFields['TableID'], $tblversionsFields['IsClassFolder'], $tblversionsFields['InGlossar']);
			case we_base_ContentTypes::WEDOCUMENT:
				if((defined('VERSIONS_CREATE') && VERSIONS_CREATE) && $status != 'published' && !we_base_request::bool('we_cmd', true, 5)){ //FIXME: this should not be handled here!
					return false;
				}
				unset($tblversionsFields['MasterTemplateID'], $tblversionsFields['TableID'], $tblversionsFields['IsClassFolder']);
		}
		$tblversionsFields = array_keys($tblversionsFields);
		$docHash = self::getHashValue(self::removeUnneededCompareFields(self::objectToArray($document)));

//look if there were made changes
		if(isset($_SESSION['weS']['versions']['versionToCompare'][$documentObj->Table][$documentObj->ID]) && ($lastEntry = $_SESSION['weS']['versions']['versionToCompare'][$documentObj->Table][$documentObj->ID]) != ''){

			$diffExists = ($docHash != $lastEntry);

			$lastEntry = self::getLastEntry($documentObj->ID, $documentObj->Table);

			switch($status){
				case 'published':
				case 'saved':
					if(isset($lastEntry['status']) && $status == $lastEntry['status'] && !$diffExists && self::versionsExist($documentObj->ID, $documentObj->Table, $documentObj->ContentType)){
						return false;
					}
			}
		}

		$set = [];
		$docFields = $documentObj->getFieldsForDB([], true); //get represantation which is needed to store doc in db
		foreach($tblversionsFields as $fieldName){
			'@phan-var string $fieldName';
			switch($fieldName){
				case 'ID':
					continue 2;
				/* case 'Creation_Date':
				  $set[$fieldName] = $documentObj->$fieldName->format('U');
				  break; */
				case 'Text':
					$docFields['Text'] = $docFields[$fieldName] ?? $docFields['Filename'] . $docFields['Extension'];
				default:
					$set[$fieldName] = $docFields[$fieldName] ?? $version->makePersistentEntry($fieldName, $status, $document);
			}
		}

		if($set){
			DB_WE::$inst->query('INSERT INTO ' . VERSIONS_TABLE . ' SET ' . we_database_base::arraySetter($set));
			$vers = ($document['version'] ?? $version->version);
			DB_WE::$inst->query('UPDATE ' . VERSIONS_TABLE . ' SET active=0 WHERE documentID=' . intval($documentObj->ID) . ' AND documentTable="' . DB_WE::$inst->escape(stripTblPrefix($documentObj->Table)) . '" AND version!=' . intval($vers));
		}
		$_SESSION['weS']['versions']['versionToCompare'][$documentObj->Table][$documentObj->ID] = $docHash;
		$_SESSION['weS']['versions']['versionToComparex'][$documentObj->Table][$documentObj->ID] = $document;
		$version->CheckPreferencesTime($documentObj->ID, $documentObj->Table);
		return true;
	}

	/**
	 *
	 * @return string[]
	 */
	private function getModifiedFields(array $document): array{
		$db = DB_WE::$inst;
		$modifications = [];

		/* get fields which can be changed */
		$fields = DB_WE::$inst->metaFieldNames(VERSIONS_TABLE);

		$vals = getHash('SELECT ' . implode(',', $fields) . ' FROM ' . VERSIONS_TABLE . ' WHERE version<' . intval($this->version) . ' AND status!="deleted" AND documentID=' . intval($document['ID']) . ' AND documentTable="' . $db->escape(stripTblPrefix($document['Table'])) . '" ORDER BY version DESC LIMIT 1');
		foreach($fields as $v){
			$val = (string) $v;
			if(!in_array($val, self::MOD_FIELDS) || !isset($vals[$val])){
				continue;
			}
			$lastEntryField = $vals[$val] ?? '';

			if(isset($document[$val])){
				switch($val){
					case 'DocType':
					case 'IsSearchable':
					case 'WebUserID':
					case 'TemplateID':
						if(!$document[$val]){
							$document[$val] = 0;
						}
						break;
				}
				if(
						(empty($lastEntryField) && empty($document[$val])) ||
						($lastEntryField == $document[$val])){
// do nothing
				} else {
					$modifications[] = $val;
				}
				continue;
			}
			switch($val){
				case 'status':
					if($lastEntryField != $this->status){
						$modifications[] = $val;
					}
					break;
				case 'documentElements':
				case 'documentScheduler':
				case 'documentCustomFilter':
					$diff = [];
					$lastEntryField = (!$lastEntryField ? [] :
							we_unserialize(
									(substr_compare($lastEntryField, 'a%3A', 0, 4) == 0 ?
									html_entity_decode(urldecode($lastEntryField), ENT_QUOTES) :
									$lastEntryField), [], true
					));
					switch($val){
						case 'documentElements':
//TODO: imi: check if we need next-level information from nested arrays
							if(!$document['elements']){
								break;
							}
							$newData = $document['elements'];
							foreach($newData as $k => $vl){
								if(isset($lastEntryField[$k]) && is_array($lastEntryField[$k]) && is_array($vl)){
									if(isset($vl[we_contents_base::ETYPE_TEXT])){
										$vl[we_contents_base::ETYPE_TEXT] = is_array($vl[we_contents_base::ETYPE_TEXT]) ? we_serialize($vl[we_contents_base::ETYPE_TEXT]) : $vl[we_contents_base::ETYPE_TEXT];
									}
									if(isset($lastEntryField[$k][we_contents_base::ETYPE_TEXT])){
										$lastEntryField[$k][we_contents_base::ETYPE_TEXT] = is_array($lastEntryField[$k][we_contents_base::ETYPE_TEXT]) ? we_serialize($lastEntryField[$k][we_contents_base::ETYPE_TEXT]) : $lastEntryField[$k][we_contents_base::ETYPE_TEXT];
									}
									$diffs = array_diff_assoc($vl, $lastEntryField[$k]);
									if(!empty($diffs) && (isset($diffs[we_contents_base::ETYPE_TEXT]) || isset($diffs[we_contents_base::ETYPE_NUMBER]))){
										$diff[] = $diffs;
									}
								}
							}

							break;
						case 'documentScheduler':
//TODO: imi: check if count() is ok (do we allways have two arrays?)
							if(count($document['schedArr']) != count($lastEntryField)){
								$diff['schedArr'] = true;
							} elseif(!empty($document['schedArr'])){
								$newData = $document['schedArr'];
								foreach($newData as $k => $vl){
									if(!empty($lastEntryField[$k]) && is_array($lastEntryField[$k]) && is_array($vl)){
										$tmpArr1 = $tmpArr2 = [];
										foreach($vl as $key => $v){
											$tmpArr1[$key] = is_array($v) ? we_serialize($v) : $v;
										}
										if(!empty($lastEntryField[$k])){
											foreach($lastEntryField[$k] as $key => $v){
												$tmpArr2[$key] = is_array($v) ? we_serialize($v) : $v;
											}
										}
										$diffs = array_diff_assoc($tmpArr1, $tmpArr2);
										if(!empty($diffs)){
											$diff = $diffs;
										}
									}
								}
							}
							break;
						case 'documentCustomFilter':
//TODO: imi: check if we need both foreach
							if(isset($document['documentCustomerFilter']) && is_array($document['documentCustomerFilter']) && is_array($lastEntryField)){
								$tmpArr1 = $tmpArr2 = [];
								foreach($document['documentCustomerFilter'] as $k => $v){
									$tmpArr1[$k] = is_array($v) ? we_serialize($v) : $v;
								}
								foreach($lastEntryField as $k => $v){
									$tmpArr2[$k] = is_array($v) ? we_serialize($v) : $v;
								}
								$diffs = array_diff_assoc($tmpArr1, $tmpArr2);
								if(!empty($diffs)){
									$diff = $diffs;
								}
							}

							break;
					}

					if(!empty($diff)){
						$modifications[] = $val;
					}
			}
		}
		return $modifications;
	}

	/**
	 * give the persistent fieldnames the values if you save, publish or unpublish
	 * persistent fieldnames are fields which are not in tblfile or tblobjectsfile and are always saved
	 * @return mixed value of field
	 */
	private function makePersistentEntry(string $fieldName, string $status, array $document){
		$db = DB_WE::$inst;

		switch($fieldName){
			case 'documentID':
				return $document['ID'];
			case 'documentTable':
				return stripTblPrefix($document['Table']); //FIXME: check if this is tblFile or Prefixed version
			case 'documentElements':
				return (isset($document['elements']) && is_array($document['elements']) ?
						sql_function('x\'' . bin2hex(we_serialize($document['elements'], SERIALIZE_JSON, false, 9)) . '\'') :
						'');
			case 'documentScheduler':
				return (!empty($document['schedArr']) && is_array($document['schedArr']) ?
						sql_function('x\'' . bin2hex(we_serialize($document['schedArr'], SERIALIZE_JSON, false, 9)) . '\'') :
						null);
			case 'documentCustomFilter':
				return (!empty($document['documentCustomerFilter']) && is_array($document['documentCustomerFilter']) ?
						sql_function('x\'' . bin2hex(we_serialize($document['documentCustomerFilter'], SERIALIZE_JSON, false, 9)) . '\'') :
						null);
			case 'logDate':
				return (fB('SELECT 1 FROM ' . VERSIONS_TABLE . ' WHERE documentID=' . intval($document['ID']) . ' AND documentTable="' . $db->escape(stripTblPrefix($document['Table'])) . '" LIMIT 1') ? sql_function('NOW()') : $document['Creation_Date']);
			case 'status':
				return ($this->status = $status);
			case 'version':
				$lastEntryVersion = fI('SELECT MAX(version) FROM ' . VERSIONS_TABLE . ' WHERE documentID=' . intval($document['ID']) . ' AND documentTable="' . $db->escape(stripTblPrefix($document['Table'])) . '"');
				if($lastEntryVersion){
					$this->version = $lastEntryVersion + 1;
				}
				return $this->version;
			case 'binaryPath':
				switch($document['ContentType']){
					case we_base_ContentTypes::COLLECTION:
					case we_base_ContentTypes::OBJECT_FILE:
					case we_base_ContentTypes::TEMPLATE:
						break;
					default:
						$documentPath = substr($document['Path'], 1);
						$siteFile = WE_DOCUMENT_ROOT . WE_SITE_DIR . $documentPath;
						$vers = $this->version;
						$versionName = $document['ID'] . '_' . $vers . (empty($document['Extension']) ? '' : $document['Extension']);
						$this->binaryPath = null;
						if(!empty($document['IsDynamic'])){

							//$this->writePreviewDynFile($siteFile, $binaryFile, $documentObj);
						} elseif(file_exists($siteFile) && $document['Extension'] === '.php' && (in_array($document['ContentType'], [we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML], true))){

						} elseif(!empty($document['TemplatePath']) && $document['ContentType'] === we_base_ContentTypes::WEDOCUMENT){

						} else {
							if(file_exists($siteFile) && filesize($siteFile) < (20 * 1024 * 1024)){//only write file if it is smaller than 20M
								$this->binaryPath = $versionName . '.gz';
								$binaryFile = WE_VERSION_PATH . $this->binaryPath;
								we_base_file::save($binaryFile, gzencode(file_get_contents($siteFile) ?: '', 9) ?: '');
							}
						}
				}
				return $this->binaryPath;
			case 'modifiedFields':
				return implode(',', $this->getModifiedFields($document));
			case 'modifierID':
				return (!we_base_WE::$schedulerActive ? we_users_user::getID() : 0);
			case 'active':
				return 1;
			case 'src':
				return self::IsScheduler() ? 'fromScheduler' : (!empty($_SESSION['weS']['versions']['fromImport']) ? 'fromImport' : null);
			case 'resetFromVersion':
				return (!empty($document['resetFromVersion'])) ? $document['resetFromVersion'] : null;
			case 'IsSearchable':
			case 'IsDynamic':
				return 0;
			case 'TemplateID':
				return null;
			default:
				return null;
		}
	}

	/**
	 * look if scheduler was called
	 * @return boolean
	 */
	private static function IsScheduler(): bool{
		if(isset($_SESSION['weS']['versions']['fromScheduler'])){
			return $_SESSION['weS']['versions']['fromScheduler'];
		}

		return false;
	}

	/**
	 * save version-entry in DB which is marked as deleted
	 */
	public static function setVersionOnDelete(int $docID, string $docTable, string $ct): void{
		$lastEntry = array_merge(self::getLastEntry($docID, $docTable), [
			'logDate' => sql_function('NOW()'),
			'status' => 'deleted',
			'modifiedFields' => 'status',
			'modifierID' => we_users_user::getID(),
			'active' => 1,
			'src' => self::IsScheduler() ? 'fromScheduler' : null,
		]);
		$lastEntry['version'] = (isset($lastEntry['version'])) ? $lastEntry['version'] + 1 : 1;

		unset($lastEntry['ID']);

		//preferences
		$doDelete = self::CheckPreferencesCtypes($ct);

		// always write delete versions, if enabled, so ignore VERSIONS_CREATE
		if($lastEntry && $doDelete){
			DB_WE::$inst->query('INSERT INTO ' . VERSIONS_TABLE . ' SET ' . we_database_base::arraySetter($lastEntry));
			DB_WE::$inst->query('UPDATE ' . VERSIONS_TABLE . ' SET active=0 WHERE documentID=' . $docID . ' AND documentTable="' . DB_WE::$inst->escape(stripTblPrefix($docTable)) . '" AND version!=' . intval($lastEntry['version']));
		}
		$version = new self();
		$version->CheckPreferencesTime($docID, $docTable);
	}

	/**
	 * delete version entry from db and delete version files
	 * @return 	array<int,array{Text:string,ContentType:string,Path:string,Version:int,documentID:int}>
	 */
	public static function deleteVersion(array $IDs = [], string $where = ''): array{
		if(!we_users_user::getID()){
			return [];
		}

		$dels = $delIds = [];
		$w = ($IDs ? 'ID IN(' . implode(',', $IDs) . ')' : $where);
		$data = DB_WE::$inst->getAllq('SELECT ID,documentID,version,CONCAT(Filename,COALESCE(Extension,"")) AS Text,ContentType,documentTable,Path,binaryPath FROM ' . VERSIONS_TABLE . ' WHERE ' . $w);

		foreach($data as $cur){
			$dels[] = $cur['ID'];
			$delIds[$cur['ID']] = [
				'Text' => $cur['Text'],
				'ContentType' => $cur['ContentType'],
				'Path' => $cur['Path'],
				'Version' => $cur['version'],
				'documentID' => $cur['documentID'],
			];

			if($cur['binaryPath']){
				we_base_file::deleteLocalFile(WE_VERSION_PATH . $cur['binaryPath']);
			}
		}
		if($dels){
			DB_WE::$inst->query('DELETE FROM ' . VERSIONS_TABLE . ' WHERE ID IN (' . implode(',', $dels) . ')');
		}
		return $delIds;
	}

	static function getDocumentFromVersion(int $ID): we_contents_root{
		$resetArray = getHash('SELECT * FROM ' . VERSIONS_TABLE . ' WHERE ID=' . $ID);
		if(!$resetArray){
			throw new we_exception_file('not found');
		}
		$resetDoc = we_base_ContentTypes::getObject($resetArray['ContentType']);
		assert($resetDoc instanceof we_document_base);
		$setVals = $resetDoc->getDocumentFromVersionInt($ID);
		foreach($resetArray as $k => $v){
			if(in_array($k, $setVals)){
				//data already set by getDocumentFromVersionInt
				continue;
			}
			switch($k){
				case 'documentID':
					$resetDoc->ID = $v;
					break;
				case 'TemplateID':
					if($v && $resetDoc instanceof we_document_webEdition){
						$resetDoc->TemplateID = $v;
					}
					break;
				case 'documentElements':
					if($v){
						$resetDoc->elements = we_unserialize($v);
					}
					break;
				case 'documentScheduler':
					if($v){
						$resetDoc->schedArr = we_unserialize($v);
					}
					break;
				case 'documentCustomFilter':
					if($v){
						$filter = we_unserialize($v);
						$resetDoc->documentCustomerFilter = new we_customer_documentFilter($resetDoc->Table);
						foreach($filter as $key => $val){
							if(is_string($key) && isset($resetDoc->documentCustomerFilter->$key) && $val){
								$resetDoc->documentCustomerFilter->$key = $val;
							}
						}
					}
					break;
			}
		}
		return $resetDoc;
	}

	/**
	 * reset version
	 * @return 	array<int,array{Text:string,ContentType:string,Path:string,Version:int,documentID:int}>
	 */
	static function resetVersion(int $ID, int $version, bool $publish): array{
		if(!we_users_user::getID()){
			return [];
		}
		$db = DB_WE::$inst;
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', 0);

		$resetArray = getHash('SELECT * FROM ' . VERSIONS_TABLE . ' WHERE ID=' . $ID);
		if(!$resetArray){
			throw new we_exception_version($ID . ' not found');
		}
		$resetIDs = [];
		$resetDoc = self::getDocumentFromVersion($ID);

		if($resetDoc->ContentType == we_base_ContentTypes::IMAGE){
			$lastBinaryPath = fS('SELECT binaryPath FROM ' . VERSIONS_TABLE . ' WHERE documentID=' . ((int) $resetArray['documentID']) . ' AND documentTable="' . $db->escape($resetArray['documentTable']) . '" AND version<=' . $version . ' AND binaryPath!="" ORDER BY version DESC LIMIT 1');
			$resetDoc->setElementS('data', WE_VERSION_PATH . $lastBinaryPath);
		}

		$resetDoc->EditPageNr = $_SESSION['weS']['EditPageNr'];

		$parentID = fI('SELECT ParentID FROM ' . $resetDoc->Table . ' WHERE ID=' . intval($resetDoc->ID), -1);
//if document was deleted

		if($parentID > -1){
			$resetDoc->ParentID = $parentID;
			$resetDoc->Path = $resetDoc->getPath();
			$oldId = 0;
		} else {
//save this id and contenttype to turn the id for the versions
			$oldId = $resetDoc->ID;
			$resetDoc->ID = 0;
		}

		if($resetDoc->ParentID){
//if folder was deleted
			$existsPath = fB('SELECT 1 FROM ' . $resetDoc->Table . ' WHERE ID=' . intval($resetDoc->ParentID) . ' AND IsFolder=1');

			if(!$existsPath){
// create old folder if it does not exists
				$folders = explode('/', $resetArray['Path']);
				$lastPathID = 0;
				foreach($folders as $k => $v){
					if($k != 0 && $k != (count($folders) - 1)){

						$parentID = $lastPathID;
						$folder = we_folder_base::instance(addTblPrefix(stripTblPrefix($resetArray['documentTable'])));

						$folder->we_new($parentID, $v);
						$existsFolderPathID = fI('SELECT ID FROM ' . $resetDoc->Table . ' WHERE Path="' . $db->escape($folder->Path) . '" AND IsFolder=1');
						if($existsFolderPathID){
							$lastPathID = $existsFolderPathID;
						} else {
							$folder->we_save();
							$lastPathID = $folder->ID;
						}
					}
				}

				$resetDoc->ID = 0;
				$resetDoc->ParentID = $lastPathID;
				$resetDoc->Path = $resetArray['Path'];
			}
		}

		$existsFile = fB('SELECT 1 FROM ' . $resetDoc->Table . ' WHERE ID!=' . ((int) $resetArray['documentID']) . ' AND Path="' . $db->escape($resetDoc->Path) . '" LIMIT 1');

		$doPark = false;
		if($existsFile){
			if(!empty($resetDoc->Filename)){
				$resetDoc->Filename = '_' . $resetArray['documentID'] . '_' . $resetDoc->Filename;
				$publish = false;
				$doPark = true;
			}
			$resetDoc->Path = $resetDoc->getPath();
		}

		if($resetDoc instanceof we_document_webEdition){
			we_base_WE::setWeDoc($resetDoc);
		}

		we_base_delete::deleteTemporaryContent($resetDoc->ID, $resetDoc->Table);

		$resetDoc->Mod_Date = new DateTime();
		if($resetDoc instanceof we_document_base){
			$resetDoc->Publish_Date = new DateTime($resetArray['logDate']);
		}

		$wasPublished = fS('SELECT status FROM ' . VERSIONS_TABLE . ' WHERE documentID=' . ((int) $resetArray['documentID']) . ' AND documentTable="' . $db->escape(stripTblPrefix($resetArray['documentTable'])) . '" AND status="published" ORDER BY version DESC LIMIT 1');
		$publishedDoc = WE_DOCUMENT_ROOT . $resetDoc->Path;

		$publishedDocExists = (we_base_ContentTypes::isRealFile($resetArray['ContentType']) ?
				file_exists($publishedDoc) :
				true);
		if($doPark || !$wasPublished || !$publishedDocExists){
			assert($resetDoc instanceof we_document_base);
			$resetDoc->Publish_Date = null;
		}
		if($publish){
			// fixme: what is this for???
			$_SESSION['weS']['versions']['doPublish'] = true;
		}
		//check if file needs to be inserted to a specific ID
		if($resetDoc->ID && !fB('SELECT 1 FROM ' . $resetDoc->Table . ' WHERE ID=' . $resetDoc->ID)){
			$resetDoc->insertID = $resetDoc->ID;
			$resetDoc->ID = 0;
		}
		$resetDoc->we_save(true, true);
		if($publish){
			unset($_SESSION['weS']['versions']['doPublish']);
			$resetDoc->we_publish();
		}

		$resetDoc->resetFromVersion = $version;
		$resetDoc->saveInSession($we_transaction);

		if(we_modules_info::isActive(we_modules_info::WORKFLOW) && $resetDoc->ContentType == we_base_ContentTypes::WEDOCUMENT){
			if(we_workflow_utility::inWorkflow($resetDoc->ID, $resetDoc->Table)){
				we_workflow_utility::removeDocFromWorkflow($resetDoc->ID, $resetDoc->Table, we_users_user::getID(), '');
			}
		}

		$resetIDs[$resetArray['ID']] = [
			'Text' => $resetArray['Filename'] . ($resetArray['Extension'] ?? ''),
			'ContentType' => $resetArray['ContentType'],
			'Path' => $resetArray['Path'],
			'Version' => $resetArray['version'],
			'documentID' => $resetArray['documentID'],
		];

//update versions if id or path were changed
		if($parentID > -1 && $oldId){
			//update the ID of old document
			$db->query('UPDATE ' . FILE_TABLE . ' SET ID=' . intval($oldId) . ' WHERE ID=' . intval($resetDoc->ID));
		}
		return $resetIDs;
	}

	public static function showValueCustomerFilter(string $k, /* mixed */ $v): string{
		switch($k){
//Customer Filter
			case '_id':
				return (string) ($v ?: 0);
			case '_accessControlOnTemplate':
				return g_l('versions', $v ? '[yes]' : '[no]');
			case '_errorDocNoLogin':
				return we_base_util::shortenPathSpace(id2path((int) $v, FILE_TABLE), 30);
			case '_errorDocNoAccess':
				return we_base_util::shortenPathSpace(id2path((int) $v, FILE_TABLE), 30);
			case '_mode':
				switch($v){
					case 0:
						return g_l('modules_customerFilter', '[mode_off]');
					case 1:
						return g_l('modules_customerFilter', '[mode_all]');
					case 2:
						return g_l('modules_customerFilter', '[mode_specific]');
					case 3:
						return g_l('modules_customerFilter', '[mode_filter]');
					default:
						return '';
				}
			case '_specificCustomers':
			case '_blackList':
			case '_whiteList':
				$fieldValueText = [];
				if(is_array($v) && !empty($v)){
					foreach(id2pathA($v, CUSTOMER_TABLE) as $cust){
						$fieldValueText[] = we_base_util::shortenPathSpace((string) $cust, 30);
					}
				}
				return implode(we_html_element::br(), $fieldValueText);
			case '_filter':
				$fieldValueText = '';
				if(is_array($v) && !empty($v)){
					foreach($v as $key => $val){
						$fieldValueText .= $key . ':<br/>';
						if(is_array($val) && !empty($val)){
							foreach($val as $key2 => $val2){
								$fieldValueText .= $key2 . ':' . $val2 . '<br/>';
							}
						}
					}
				}
				return $fieldValueText;
			default:
				return (string) $v;
		}
	}

	/**
	 *
	 * @param string $k
	 * @param mixed $v
	 * @param string $table
	 * @return string
	 */
	public static function showValue(string $k, /* mixed */ $v, string $table = ''): string{
		$val = self::_showValue($k, $v, $table);
		return ($val ?: '&nbsp;');
	}

	/**
	 * return the fieldvalue that has been changed
	 * @staticvar int $months
	 * @staticvar int $days
	 * @param string $k
	 * @param mixed $v
	 * @param string $table
	 * @return string
	 */
	private static function _showValue(string $k, /* mixed */ $v, string $table): string{
		$pathLength = 41;
		static $months = 0;
		$months = $months ?: we_base_country::getTranslationList(we_base_country::MONTH, array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: 'de');
		static $days = 0;
		$days = $days ?: we_base_country::getTranslationList(we_base_country::DAY, array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: 'de');

		switch($k){
			case '_logDate':
				return $v;
			case 'status':
				return g_l('versions', '[' . $v . ']');
			case 'ParentID':
				return id2path($v, addTblPrefix(stripTblPrefix($table)));
			case 'modifierID':
			case 'CreatorID':
				return id2path($v, USER_TABLE);
			case 'MasterTemplateID':
			case 'TemplateID':
				return ($v == 0 ? '' : id2path($v, TEMPLATES_TABLE));
			case 'InGlossar':
			case 'IsDynamic':
			case 'IsSearchable':
				return g_l('versions', ($v == 1 ? '[activ]' : '[notactiv]'));
			case 'DocType':
				return fS('SELECT Text FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($v));
			case 'RestrictOwners':
				return g_l('versions', ($v == 1 ? '[activ]' : '[notactiv]'));
			case 'Language':
				return we_base_WE::$Langs['FrontendLanguages'][$v] ?? '';
			case 'WebUserID':
				return $v ? id2path($v, CUSTOMER_TABLE) : '';
			case 'Workspaces':
				$fieldValueText = '';
				if($v != ''){
					$vals = array_filter(explode(',', $v));
					if(!empty($vals)){
						foreach($vals as $k){
							if($fieldValueText != ''){
								$fieldValueText .= '<br/>';
							}
							$fieldValueText .= we_base_util::shortenPathSpace(id2path((int) $k, FILE_TABLE), $pathLength);
						}
					}
				}
				return $fieldValueText;
			case 'Templates':
				$fieldValueText = '';
				if($v != ''){
					$vals = array_filter(explode(',', $v));
					if(!empty($vals)){
						foreach($vals as $k){
							if($fieldValueText){
								$fieldValueText .= '<br/>';
							}
							$fieldValueText .= we_base_util::shortenPathSpace(id2path((int) $k, FILE_TABLE), $pathLength);
						}
					}
				}
				return $fieldValueText;
			case 'src':
				return (string) $v;
			case 'resetFromVersion':
				return (empty($v) ? '-' : (string) ($v));
			case 'Category':
				$fieldValueText = [];
				$arr = we_base_request::filterIntList($v);
				if(!we_modules_info::isActive(we_modules_info::CATEGORY)){
					return $v ? implode('<br/>', $arr) : '';
				}
				if(!empty($arr)){
					foreach(id2pathA($arr, CATEGORY_TABLE) as $path){
						'@phan-var string $path';
						$fieldValueText[] = we_base_util::shortenPathSpace($path, $pathLength);
					}
				}
				return $fieldValueText ? implode('<br/>', $fieldValueText) : '';
			case 'Owners':
				$fieldValueText = [];
				$arr = we_base_request::filterIntList($v);
				if(!empty($arr)){
					foreach(id2pathA($arr, USER_TABLE) as $path){
						'@phan-var string $path';
						$fieldValueText[] = we_base_util::shortenPathSpace($path, $pathLength);
					}
				}
				return $fieldValueText ? implode('<br/>', $fieldValueText) : '';
			case 'OwnersReadOnly':
				$fieldValueText = [];
				if($v != '' && !is_array($v)){
					$v = we_unserialize($v);
				}
				if(is_array($v) && !empty($v)){
					$paths = id2pathA($v, USER_TABLE);
					foreach($v as $key => $val){
						$stat = g_l('versions', ($val == 1) ? '[activ]' : '[notactiv]');
						$fieldValueText[] = (isset($paths[$key]) ? we_base_util::shortenPathSpace($paths[$key], $pathLength) : g_l('weClass', '[fieldusers]') . ' ' . $key) . ': ' . $stat;
					}
				}
				return $fieldValueText ? implode('<br/>', $fieldValueText) : '';
			case we_base_variants::ELEMENT_NAME:
				$fieldValueText = '';
				if($v != '' && !is_array($v)){
					$v = we_unserialize($v);
				}
				if(is_array($v) && !empty($v)){
					foreach($v as $val){
						if(is_array($val)){
							foreach($val as $k => $vl){
								if($k != ''){
									$fieldValueText .= '<strong>' . $k . '</strong><br/>';
								}
								if(is_array($val)){
									foreach($vl as $key3 => $val3){
										if($key3 != ''){
											$fieldValueText .= $key3 . ': ';
										}
										$fieldValueText .= (empty($val3[we_contents_base::ETYPE_TEXT]) ?
												(empty($val3[we_contents_base::ETYPE_NUMBER]) ? '' :
												$val3[we_contents_base::ETYPE_NUMBER] . '<br/>') :
												$val3[we_contents_base::ETYPE_TEXT] . '<br/>');
									}
								}
							}
						}
					}
				}
				return $fieldValueText . '<br/>';
//Scheduler
			case 'task':
				return ($v ? g_l('modules_schedule', '[task][' . $v . ']') : '');
			case 'type':
				return g_l('modules_schedule', '[type][' . $v . ']');
			case 'active':
				return g_l('versions', ($v == 1) ? '[yes]' : '[no]');
			case 'months':
				$mon = [];
				if(is_array($v) && !empty($v)){
					foreach($v as $k => $val){
						if($val == 1){
							$mon[] = $months['abbreviated'][$k];
						}
					}
				}
				return implode(', ', $mon);
			case 'days':
				$ds = [];
				if(is_array($v) && !empty($v)){
					foreach($v as $k => $val){
						if($val == 1){
							$ds[] = sprintf('%02d', $k + 1);
						}
					}
				}
				return implode(', ', $ds);
			case 'weekdays':
				$weekdays = [];
				if(is_array($v) && !empty($v)){
					foreach($v as $k => $val){
						if($val == 1){
							$weekdays[] = $days['abbreviated'][$k];
						}
					}
				}

				return implode(', ', $weekdays);
			case 'time':
				return is_int($v) ? date('d.m.y - H:i:s', $v) : $v;
			case 'doctypeAll':
				return ($v == 1) ? g_l('versions', '[yes]') : '';
			case 'DoctypeID':
				return fS('SELECT Text FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($v));
			case 'CategoryIDs':
				$fieldValueText = '';
				$v = array_filter(explode(',', $v));
				if(!empty($v)){
					foreach($v as $key){
						if($fieldValueText != ''){
							$fieldValueText .= '<br/>';
						}
						$fieldValueText .= we_base_util::shortenPathSpace(id2path((int) $key, CATEGORY_TABLE), $pathLength);
					}
				}
				return $fieldValueText;
			default:
				return (string) (is_array($v) ? implode(',', $v) : $v);
		}
	}

	/**
	 * convert object to array
	 * @return array<string,mixed>
	 */
	private static function objectToArray($obj): array{
		$arr = is_object($obj) ? get_object_vars($obj) : $obj;
		if(!is_array($arr)){
			return [];
		}
		foreach($arr as $key => $val){
			$arr[$key] = (is_scalar($val) || $val === null ? $val : (is_object($val) && $val instanceof DateTime ? $val->format(we_database_base::DATE_TIME_FORMAT) : self::objectToArray($val)) );
		}

		return $arr;
	}

	private static function removeUnneededCompareFields(array $doc): array{
		unset($doc['Publish_Date'], $doc['Mod_Date'], $doc['EditPageNr'], $doc['DocStream'], $doc['DB_WE'], $doc['hasVariants'], $doc['editorSaves'], $doc['Name'], $doc['wasUpdate'], $doc['InWebEdition'], $doc['PublWhenSave'], $doc['IsTextContentDoc'], $doc['fileExists'], $doc['elements'][we_base_variants::ALLVARIANT_FIELD], $doc['persistent_slots'], $doc['versionsModel']);
		return $doc;
	}

	private static function getHashValue(array $a, bool $inner = false): string{
		$tmp = '';
		$keys = array_keys($a);
		sort($keys);
		foreach($keys as $key){
			$tmp .= $key . (is_array($a[$key]) ? self::getHashValue($a[$key], true) : $a[$key]);
		}
		return ($inner ? $tmp : md5($tmp));
	}

	/**
	 * get last record of $docID which was saved or published
	 * @return mixed[] with fields and values
	 */
	private static function getLastEntry(int $docID, string $docTable): array{
		return getHash('SELECT * FROM ' . VERSIONS_TABLE . ' WHERE documentID=' . $docID . ' AND documentTable="' . DB_WE::$inst->escape(stripTblPrefix($docTable)) . '" AND status IN ("saved","published","unpublished","deleted") ORDER BY version DESC LIMIT 1');
	}

	public static function versionExists(int $docID, string $docTable): bool{
		return fB('SELECT 1 FROM ' . VERSIONS_TABLE . ' WHERE documentID=' . $docID . ' AND documentTable="' . DB_WE::$inst->escape(stripTblPrefix($docTable)) . '" AND status IN ("saved","published","unpublished","deleted") LIMIT 1');
	}

	public static function updateLastVersionPath(int $docID, string $docTable, int $parentId, string $path): void{
		DB_WE::$inst->query('UPDATE ' . VERSIONS_TABLE . ' SET ' . we_database_base::arraySetter([
					'ParentID' => $parentId,
					'Path' => $path
				]) . '
WHERE
	documentID=' . $docID . ' AND
	documentTable="' . DB_WE::$inst->escape(stripTblPrefix($docTable)) . '"
ORDER BY
	ID DESC
LIMIT 1');
	}

	public static function todo(array $data): bool{
		switch($data['type']){
			case 'version_delete':
				$_SESSION['weS']['versions']['logDeleteIds'] = array_merge($_SESSION['weS']['versions']['logDeleteIds'], self::deleteVersion([$data['ID']]));
				break;
			case 'version_reset' :
				$publish = we_base_request::bool('reset_doPublish');
				$_SESSION['weS']['versions']['logResetIds'] = array_merge($_SESSION['weS']['versions']['logResetIds'], self::resetVersion($data['ID'], $data['version'], $publish));

				break;

			default:
				return false;
		}
		return true;
	}

	public static function showPreviewDoc(int $id): void{
		if(!$id){
			return;
		}
		$doc = self::getDocumentFromVersion($id);
		if(!($doc instanceof we_document_webEdition)){
			return;
		}
		//FIXME this works only for we-documents!
		we_base_WE::$versioningActive = true;
		echo (new we_editor_showTempDocumentData($doc))->setContent($doc->getDocumentData(), true)->show();
		return;
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
					'all' => g_l('logging', '[all]'),
					'defaultView' => g_l('logging', '[defaultView]'),
					'deleteDateVersionsOK' => g_l('versions', '[deleteDateVersionsOK]'),
					'deleteNothingFound' => g_l('versions', '[deleteNothingFound]'),
					'deleteVersions' => g_l('versions', '[deleteVersions]'),
					'mark' => g_l('versions', '[mark]'),
					'notChecked' => (g_l('versions', '[notChecked]')),
					'notMark' => g_l('versions', '[notMark]'),
					'resetAllVersionsOK' => g_l('versions', '[resetAllVersionsOK]'),
					'resetVersions' => g_l('versions', '[resetVersions]'),
		]);
	}

}
