<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: cdc2c0325715 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-12-02 22:36 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_voting_tree extends we_tree_base{

	protected function customJSFile(): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'voting/voting_tree.js', 'initTree();');
	}

	public static function getItems(int $ParentID, int $offset = 0, int $segment = 500, bool $sort = false): array{
		$items = [];
		$owners_sql = we_voting_voting::getOwnersSql();

		$prevoffset = max(0, $offset - $segment);
		if($offset && $segment){
			$items[] = [
				'id' => 'prev_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $prevoffset . '-' . $offset . ')',
				'contenttype' => self::CT_UP,
				'table' => VOTING_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => false,
				'published' => 0,
				'disabled' => false,
				'tooltip' => '',
				'offset' => $prevoffset
			];
		}

		$where = ' WHERE ParentID=' . $ParentID . ' ' . $owners_sql;

		foreach(DB_WE::$inst->getAllq('SELECT ID,ParentID,Path,Text,IsFolder,RestrictOwners,Owners,Active,ActiveTime,Valid FROM ' . VOTING_TABLE . $where . ' ORDER BY IsFolder DESC,(text REGEXP "^[0-9]") DESC,ABS(text),Text' . ($segment ? ' LIMIT ' . abs($offset) . "," . abs($segment) : '' )) as $item){
			'@phan-var array{ID:int,ParentID:int,Path:string,Text:string,IsFolder:int,RestrictOwners:int,Owners:string,Active:int,ActiveTime:int,Valid:int} $item';
			$typ = ['typ' => ($item['IsFolder'] ? self::TYP_FOLDER : self::TYP_ITEM),
				'open' => false,
				'disabled' => false,
				'tooltip' => $item['ID'],
				'offset' => $offset,
				'contenttype' => ($item['IsFolder'] ? we_base_ContentTypes::FOLDER_COMMON : we_base_ContentTypes::VOTING),
			];

			if(!$item['IsFolder']){
				$typ['published'] = ($item['Active'] && (!((bool) $item['ActiveTime']) || (time() < strtotime((string)$item['Valid'])))) ? 1 : 0;
			}
			$fileds = array_change_key_case($item);

			$items[] = array_merge($fileds, $typ);
		}

		$total = fI('SELECT COUNT(1) FROM ' . VOTING_TABLE . ' ' . $where);
		$nextoffset = $offset + $segment;
		if($segment && ($total > $nextoffset)){
			$items[] = [
				'id' => 'next_' . $ParentID,
				'parentid' => 0,
				'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ")",
				'contenttype' => self::CT_DOWN,
				'table' => VOTING_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => false,
				'disabled' => false,
				'tooltip' => '',
				'offset' => $nextoffset
			];
		}

		return $items;
	}

}
