<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: fdb857335a7f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-11-18 01:31 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_wizard_tag{
	private const CACHE_EXPIRY = 24 * 3600;

	public static function getExistingWeTags(bool $useDeprecated = true): array{
		$retTags = [];
		$main = self::getMainTagModules($useDeprecated);
		foreach($main as $modulename => $tags){
			if($modulename === we_weTag_twbase::MODULE_BASE || we_modules_info::isActive((string) $modulename)){
				$retTags = array_merge($retTags, $tags);
			}
		}

		// add custom tags
		$retTags = array_merge($retTags, self::getCustomTags());

		// add application tags
		//$retTags = array_merge($retTags, self::getApplicationTags());
		natcasesort($retTags);
		self::initTagLists($retTags);

		return $retTags;
	}

	public static function getWeTagGroups(): array{
		try{
			return we_cache_file::load(we_cache_file::DOMAIN_TW . 'groups');
		} catch (we_exception_cache $e){
			self::getExistingWeTags();
			return we_cache_file::load(we_cache_file::DOMAIN_TW . 'groups');
		}
	}

	private static function getMainTagModules(bool $useDeprecated = true): array{
		try{
			return we_cache_file::load(we_cache_file::DOMAIN_TW . 'mainTags');
		} catch (Exception $ex){
			$main = [];
			$tags = self::getTagsFromDir(WE_CLASS_PATH . 'tw/');
			$if = [];
			foreach($tags as $tagname){
				'@phan-var string $tagname';
				$p = substr($tagname, 0, 4);
				if($p !== 'ifNot' && str_starts_with($p, 'if')){
					$if[] = str_replace('if', 'ifNot', $tagname);
				}
			}
			$tags = array_unique(array_merge($tags, $if));
			foreach($tags as $tagname){
				try{
					$tag = we_weTag_twbase::getTagData($tagname);
				} catch (Throwable $e){
					continue;
				}
				if($useDeprecated || !$tag->isDeprecated()){
					$main[$tag->getModule()][] = $tagname;
				}
			}
			we_cache_file::save(we_cache_file::DOMAIN_TW . 'mainTags', $main);
			return $main;
		}
	}

	public static function getHash(): string{
		try{
			return (string) we_cache_file::stat(we_cache_file::DOMAIN_TW . 'tagCount');
		} catch (Exception $ex){
			return '-1';
		}
	}

	/**
	 * Initializes database for all tags
	 * @param string[] $tags
	 */
	private static function initTagLists(array $tags): void{
		try{
			$count = we_cache_file::load(we_cache_file::DOMAIN_TW . 'tagCount');
			if(count($tags) == $count[0]){
				return;
			}
		} catch (Exception $ex){

		}
		$endTags = $modules = $groups = [];
		foreach($tags as $tagname){
			try{
				$tag = we_weTag_twbase::getTagData($tagname);
			} catch (Throwable $e){
				continue;
			}
			$mod = $tag->getModule();
			$modules[$mod][] = $tagname;
			$groups['alltags'][] = $tagname;
			if($mod != we_weTag_twbase::MODULE_BASE){
				$groups[$mod][] = $tagname;
			}
			foreach($tag->getGroups() as $group){
				$groups[$group][] = $tagname;
			}
			if($tag->needsEndTag()){
				$endTags[] = $tagname;
			}
		}
		we_cache_file::save(we_cache_file::DOMAIN_TW . 'tagCount', [count($tags)], self::CACHE_EXPIRY);
		we_cache_file::save(we_cache_file::DOMAIN_TW . 'needsEndTag', $endTags, self::CACHE_EXPIRY);
		we_cache_file::save(we_cache_file::DOMAIN_TW . 'groups', $groups, self::CACHE_EXPIRY);
		we_cache_file::save(we_cache_file::DOMAIN_TW . 'modules', $modules, self::CACHE_EXPIRY);
	}

	//FIXME: check if custom tags are updated correctly!
	private static function getTagsWithEndTag(): array{
		try{
			return we_cache_file::load(we_cache_file::DOMAIN_TW . 'needsEndTag');
		} catch (we_exception_cache $ex){
			self::getExistingWeTags();
			return we_cache_file::load(we_cache_file::DOMAIN_TW . 'needsEndTag');
		}
	}

	private static function getCustomTags(): array{
		try{
			return we_cache_file::load(we_cache_file::DOMAIN_TW . 'customTags');
		} catch (we_exception_cache $e){
			$customTags = self::getTagsFromDir(WE_CUSTOM_PATH . we_weTag_twbase::DIR);
			we_cache_file::save(we_cache_file::DOMAIN_TW . 'customTags', $customTags, self::CACHE_EXPIRY);
			return $customTags;
		}
	}

	private static function getTagsFromDir(string $dir): array{
		$match = $ret = [];
		if(is_dir($dir)){

			// get the custom tag-descriptions
			$handle = dir($dir);

			while(false !== ($entry = $handle->read())){
				if(preg_match('/we_(tw)_(.*).class.php/', $entry, $match)){
					$ret[] = $match[2];
				}
			}
		}
		return $ret;
	}

}
