<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2846022b1c50 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-06-27 12:58 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_xml_browser extends we_xml_parser implements Serializable{

	public function __construct(string $filename = '', string $mode = 'backup'){
		$this->mode = $mode;
		$this->xmlExt = false;
		parent::__construct($filename);
	}

	public function saveCache(string $cache = '', int $expire = 0): bool{
		return we_cache_file::save($cache, [$this], $expire ?: 1800);
	}

	/**
	 * @throws we_exception_cache
	 * @param string $cache
	 * @return static
	 */
	public static function loadCache(string $cache = '')/* :static */{
		[$ret] = we_cache_file::load($cache);
		return $ret;
	}

	public function getFile(string $file, int $timeout = 0): bool{
		$this->fileName = $file;

		switch((we_base_file::hasURL($file) ? we_http_util::getHttpOption() : 'local')){
			case 'fopen':
			case 'curl':
				$m = [];
				if(!preg_match('/^(((ht|f)tp(s?):\/\/)|(www\.))+(([a-zA-Z0-9\._-]+\.[a-zA-Z]{2,6})|([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}))(\/[a-zA-Z0-9\&amp;%_\.\/-~-]*)?/i', $file, $m)){
					return false;
				}

				try{
					$content = we_http_util::getHTTP(str_replace($m[9], '', $file), $m[9], 0, '', '', $timeout);
				} catch (we_exception_http $e){
					return false;
				}
				break;
			case 'local':
				$content = file_get_contents($file);
				break;
			default:
				// has to be implemented
				return false;
		}

		if($content){
			$encoding = $this->setEncoding('', $content);
			return $this->parseXML($content, $encoding);
		}
		return false;
	}

	public function serialize(): string{//deprecated in php 8.1
		return serialize(['encoding' => $this->mainXmlEncoding, 'nodes' => $this->nodes]);
	}

	public function unserialize($serialized): void{//deprecated in php 8.1
		['encoding' => $this->mainXmlEncoding, 'nodes' => $this->nodes] = unserialize($serialized);
	}

	public function __serialize(): array{
		return ['encoding' => $this->mainXmlEncoding, 'nodes' => $this->nodes];
	}

	public function __unserialize(array $data): void{
		['encoding' => $this->mainXmlEncoding, 'nodes' => $this->nodes] = $data;
	}

}
