/* global CodeMirror,define */

/**
 * webEdition CMS
 *
 * $Rev: ebc24d3abd4a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2021-03-25 00:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/*taken in parts from https://github.com/coderaiser/cm-show-invisibles*/
'use strict';

((mod) => {
	if (typeof exports === 'object' && typeof module === 'object') // CommonJS
		return mod(require('../../we/libraries/codemirror'));

	if (typeof define === 'function' && define.amd) // AMD
		return define(['../../we/libraries/codemirror'], mod);

	mod(CodeMirror);
})((CodeMirror) => {
	CodeMirror.defineOption('showInvisibles', false, (cm, val, prev) => {
		let Count = 0;
		const Maximum = cm.getOption('maxInvisibles') || 16;

		if (prev === CodeMirror.Init)
			prev = false;

		if (prev && !val) {
			cm.removeOverlay('invisibles');
			return;
		}

		if (!prev && val) {
			add(cm);

			cm.addOverlay({
				name: 'invisibles',
				token: function nextToken(stream) {
					let spaces = 0;
					let peek = stream.peek() === ' ';

					if (peek) {
						while (peek && spaces < Maximum) {
							++spaces;

							stream.next();
							peek = stream.peek() === ' ';
						}

						let ret = 'whitespace whitespace-' + spaces;

						/*
						 * styles should be different
						 * could not be two same styles
						 * beside because of this check in runmode
						 * function in `codemirror.js`:
						 *
						 * 6624: if (!flattenSpans || curStyle != style) {}
						 */
						if (spaces === Maximum)
							ret += ' whitespace-rand-' + Count++;

						return ret;
					}

					while (!stream.eol() && !peek) {
						stream.next();

						peek = stream.peek() === ' ';
					}

					return 'cm-eol';
				}
			});
		}
	});

	function add(cm) {
		cm.display.wrapper.classList.add("showInvisibles");
	}

});
