/* global WE, top, LinkDialog, ImageDialog */

/**
 * webEdition CMS
 *
 * $Rev: 4589e82d695f $
 * $Author: nilsole <nils.reibold@aldente-it.de> $
 * $Date: 2022-04-08 11:33 +0200 $
 *
 * This source is part of the webEdition SDK. The webEdition SDK is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU Lesser General Public License can be found at
 * http://www.gnu.org/licenses/lgpl-3.0.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionSDK/License.txt
 *
 *
 * @license    http://www.gnu.org/licenses/lgpl-3.0.html  LGPL
 */
'use strict';
var payload = WE().util.getDynamicVar(document, 'loadVarDialog_cmdFrame', 'data-payload');
var isWeDialog = payload && payload.isWeDialog ? true : false;

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
		args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments));
	//var url = WE().util.getWe_cmdArgsUrl(args);

	switch (args[0]) {
		case 'update_dialog_image':
			WE().util.dialogTop(window).update_dialog(payload);
			break;
		case 'image_writeback':
			WE().util.dialogTop(window).ImageDialog.writeBack(payload.attributes, payload.thumbdata);
			break;
		case 'link_writeback':
			WE().util.dialogTop(window).LinkDialog.writeBack(payload.attributes);
			break;
		case 'fullscreen_writeback': // probably obsolete
			WE().util.dialogTop(window).WefullscreenDialog.writeBack();
			break;
		case 'wysiwyg_writeBack':
			WE().layout.we_tinyMCE.functions.checkLen(WE().util.dialogTop(window).tinymce.activeEditor);
			if(WE().util.dialogTop(window).tinymce.activeEditor.settings.weMaxlen.currentWarning){
				WE().util.showMessage('Die erlaubte Länge für dieses Feld wurde überschritte: Der Editorinhalt kann nicht zurückgeschrieben werden.', WE().consts.message.WE_MESSAGE_ERROR, window)
				break;
			}

			var openerDoc = payload.isFrontendEdit ? WE().util.dialogTop(window).opener.document : WE().layout.weEditorFrameController.getVisibleEditorFrame().document;
			try {
				openerDoc.getElementById(payload.name).value = payload.textareaValue;
			} catch (err) {
				WE().t_e('we_cmd "wysiwyg_writeBack": could not write back to textarea', err);
			}
			try {
				openerDoc.getElementById('div_wysiwyg_' + payload.name).innerHTML = payload.divValue;
			} catch (err) {
				WE().t_e('we_cmd "wysiwyg_writeBack": could not write back to preview div', err);
			}

			// FIXME: why do we write back to visble frame instead of opener?
			try {
				WE().layout.weEditorFrameController.getVisibleEditorFrame().seeMode_dealWithLinks();
			} catch (err) {
			}

			if(payload.syncAndSaveWedoc || payload.syncAndPublishWedoc){
				var footer = WE().layout.weEditorFrameController.getActiveDocumentReference().frames.editFooter;
				if(payload.syncAndPublishWedoc){
					var btnPublish = footer.document.getElementsByClassName('weBtnPublish') ? footer.document.getElementsByClassName('weBtnPublish')[0] : null;
					if(btnPublish){
						if(footer.onclickBtnPublish){
							footer.onclickBtnPublish();
						}
					}
				} else {
					var btnSave = footer.document.getElementsByClassName('weBtnSave') ? footer.document.getElementsByClassName('weBtnSave')[0] : null;
					if(btnSave){
						if(footer.onclickBtnSave){
							footer.onclickBtnSave();
						}
					}
				}
			} else {
				WE().util.dialogTop(window).doClose();
			}
			break;
		case 'tag_welink_writeback':
			WE().util.dialogTop(window).opener.setScrollTo();
			WE().util.dialogTop(window).opener.we_cmd("change_link", args[1]);
			WE().util.dialogTop(window).doClose();
			break;
		case 'classfield_link_writeback':
			WE().util.dialogTop(window).opener.setScrollTo();
			WE().util.dialogTop(window).opener.we_cmd('we_object_class_changeLink', args[1], args[2], args[3]);
			WE().util.dialogTop(window).doClose();
			break;
		case 'objectfield_link_writeback':
			WE().util.dialogTop(window).opener.setScrollTo();
			WE().util.dialogTop(window).opener.we_cmd('we_object_file_linkChange', args[1], args[2]);
			WE().util.dialogTop(window).doClose();
			break;
		default:
			WE().util.dialogTop(window).we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}
