/**
 * webEdition CMS
 *
 * $Rev: 041d459c919f $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-12-02 18:15 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* global WE, top */
'use strict';

var editorSave = WE().util.getDynamicVar(document, 'loadVarEditor_save', 'data-editorSave'),
		isEditInclude = false,
		weWindow = top;

while(1) {
	if(!weWindow.top.opener || weWindow.treeData){
		break;
	} else {
		isEditInclude = true;
		weWindow = weWindow.opener.top;
	}
}

if(editorSave.we_editor_save){//called from we_editor_save.inc.php
	let editorFrame = WE().layout.weEditorFrameController.getEditorFrameByTransaction(editorSave.we_transaction);
	if(editorSave.isHot && editorFrame){
		editorFrame.setEditorIsHot(true);
	}

	if(editorSave.wasSaved){
		// DOC was saved, mark open tabs to reload if necessary
		// was saved - not hot anymore

		if(editorFrame){
			editorFrame.setEditorIsHot(false);
			if(editorFrame.getContentFrame().refreshContentCompare !== undefined){
				editorFrame.getContentFrame().refreshContentCompare();
			}
		}
		if(editorSave.reloadEditors){
			WE().layout.reloadUsedEditors(editorSave.reloadEditors);
		}
		if(!editorSave.isSEEMode){
			WE().layout.makeNewDoc(editorFrame, editorSave.ContentType, editorSave.docID, editorSave.saveTmpl, !editorSave.isClose);
		}
	}

	for(let i = 0; i < editorSave.we_JavaScript.length; i++) {
		we_cmd.apply(window, editorSave.we_JavaScript[i]);
	}

	window.focus();

// enable navigation box if doc has been published
	if(editorSave.wasPublished){
		let contentEditor = WE().layout.weEditorFrameController.getVisibleEditorFrame();
		try {
			if(editorFrame && editorFrame.getEditorIsInUse() && contentEditor){
				WE().layout.button.switch_button_state(contentEditor.document, "add", "enabled");
			}
		} catch(e) {
		}
	}

	if(editorSave.isSEEMode && !editorSave.showAlert){
		let editorFrameDocumentRef = editorFrame.getDocumentReference()
		showAlert = false;
		//	Confirm Box or alert in seeMode
		if(editorSave.isPublished){
			//	edit include and pulish then close window and reload
			if(isEditInclude){
				showAlert = true;
			}
			if(editorSave.docHasPreview && editorSave.EditPageNr != WE().consts.tabs.PREVIEW){
				if(!showAlert){ //	alert or confirm
					if(window.confirm(editorSave.we_responseText + "\n\n" + WE().consts.g_l.alert.confirm_change_to_preview)){
						editorFrameDocumentRef.frames.editHeader.we_cmd('switch_edit_page', WE().consts.tabs.PREVIEW, editorSave.we_transaction);
					} else {
						editorFrameDocumentRef.frames.editHeader.we_cmd('switch_edit_page', editorSave.EditPageNr, editorSave.we_transaction);
					}
				} else {
					WE().util.showMessage(editorSave.we_responseText, editorSave.we_responseTextType, window);
				}
			} else {
				//	alert when in preview mode
				WE().util.showMessage(editorSave.we_responseText, editorSave.we_responseTextType, window);
				editorFrameDocumentRef.frames.editHeader.we_cmd('switch_edit_page', editorSave.EditPageNr, editorSave.we_transaction);

				for(let i = 0; i < editorSave.we_responseJS.length; i++) {
					we_cmd.apply(window, editorSave.we_responseJS[i]);
				}
				if(editorSave.we_cmd5){
					for(let i = 0; i < editorSave.we_cmd5.length; i++) {
						we_cmd.apply(window, editorSave.we_cmd5[i]);
					}
				}
			}
			if(isEditInclude){
				WE().util.showMessage(WE().consts.g_l.alert.changed_include, WE().consts.message.WE_MESSAGE_NOTICE, window);
				weWindow.top.we_cmd("reload_editpage");
				weWindow.edit_include.close();
				top.close();
			}
		}
	} else {
		if(editorSave.we_responseText){
			WE().util.showMessage(editorSave.we_responseText, editorSave.we_responseTextType, window);
		}
		for(let i = 0; i < editorSave.we_responseJS.length; i++) {
			we_cmd.apply(window, editorSave.we_responseJS[i]);
		}

		for(let i = 0; i < editorSave.we_cmd5.length; i++) {
			we_cmd.apply(window, editorSave.we_cmd5[i]);
		}
	}
} else {//called from we_editor_publish.inc.php
	for(let i = 0; i < editorSave.we_responseJS.length; i++) {
		we_cmd.apply(window, editorSave.we_responseJS[i]);
	}
	for(let i = 0; i < editorSave.we_cmd5.length; i++) {
		we_cmd.apply(window, editorSave.we_cmd5[i]);
	}
	for(let i = 0; i < editorSave.we_JavaScript.length; i++) {
		we_cmd.apply(window, editorSave.we_JavaScript[i]);
	}
	if(editorSave.we_responseText){
		WE().util.showMessage(editorSave.we_responseText, editorSave.we_responseTextType, window);
	}
}

function we_cmd(){
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
			args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
			//var url = WE().util.getWe_cmdArgsUrl(args);
			editorFrame = WE().layout.weEditorFrameController.getEditorFrameByTransaction(editorSave.we_transaction);

	switch(args[0]) {
		case 'setEditorDocumentId':
			if(editorFrame){
				editorFrame.setEditorDocumentId(args[1]);
			}
			break;
		case "we_setPath":
			WE().layout.we_setPath(editorFrame, args[1], args[2], args[3], args[4]);
			break;
		case 'unsetHot':
			editorFrame.setEditorIsHot(false);
			break;
		case 'addHistory':
			WE().layout.navigationHistory.addDocToHistory(args[1], args[2], args[3]);
			break;
		case 'rebuildTemplates':
			(new (WE().layout.overlay)(caller, WE().layout.weEditorFrameController.getActiveEditorFrame().EditorFrameWindow, WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=rebuild&step=2&btype=rebuild_filter&templateID=' + args[1] + '&responseText=' + args[2], 'resave', WE().consts.size.dialog.small, WE().consts.size.dialog.tiny)).setTitle(WE().consts.g_l.dialogTitle.rebuild).open();
			break;
		case 'closeDocument':
			WE().layout.weEditorFrameController.closeDocument(args[1]);
			break;
		case 'updateVTab':
			if(top.treeData.table === args[1]){
				we_cmd('loadVTab', top.treeData.table, 0);
			}
			break;
		default:
			top.we_cmd.apply(caller, args);
	}
}