/* global top, WE, entries */

/**
 * webEdition CMS
 *
 * $Rev: 4d9f1761ea67 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-10-07 11:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
function drawNewFolder(){
	let dtop = WE().util.dialogTop(window);
	dtop.unselectAllFiles();
	let name = window.prompt(WE().consts.g_l.fileselector.new_folder_prompt);
	if(name){
		dtop.fscmd.location.replace(dtop.queryString(WE().consts.selectors.CREATEFOLDER, dtop.fileSelect.data.currentDir) + "&we_cmd[newFolderText]=" + name);
	}
}

function showPreview(id){
	let dtop = WE().util.dialogTop(window);
	if(dtop.document.getElementById('fspreviewDiv')){
		if(!id){
			dtop.document.getElementById('fspreviewDiv').innerHTML = "";
			return;
		}
		WE().util.rpc(dtop.queryString(WE().consts.selectors.PREVIEW, id), '', function(weResponse){
			let prev = dtop.document.getElementById('fspreviewDiv'),
					html = '<table class="default previewTable" style="width:100%">';

			if(!prev){
				return;
			}
			prev.innerHTML = "";
			if(!weResponse.Success){
				return;
			}
			setInfoSize();
			weWriteBreadCrumb(weResponse.DataArray.path);
			if(weResponse.DataArray.imgPreview){
				html += '<tr><td colspan="2" class="image">' + weResponse.DataArray.imgPreview + '</td></tr>';
			}
			for(let previewFieldName in weResponse.DataArray.previewFields) {
				let previewField = weResponse.DataArray.previewFields[previewFieldName];
				if(previewField.data.length){
					html += '<tr><td colspan="2" class="headline">' + previewField.headline + '</td></tr>';
					for(let data of previewField.data) {
						html += '<tr>' + (!data.caption ? '<td colspan="2">' : '<td>' + data.caption + ': </td><td>') + data.content + '</td></tr>';
					}
				}
			}
			prev.innerHTML = html + '</table>';
		});
	}
}

function setInfoSize(){
	let infoElem = document.getElementById("info");
	if(infoElem){
		infoElem.style.height = document.body.clientHeight - (document.getElementById("previewpic") ? 160 : 0) + "px";
	}
}

function weWriteBreadCrumb(BreadCrumb){
	if(!top.document){
		return;
	}
	let fspath = top.document.getElementById("fspath");
	if(fspath){
		fspath.innerHTML = BreadCrumb;
	}
}

function doClick(id, ct){
	let dtop = WE().util.dialogTop(window);
	if(dtop.document.getElementById('fspreviewDiv')){
		dtop.document.getElementById('fspreviewDiv').innerHTML = "";
	}
	if(ct == 1){
		if(dtop.metaKeys.doubleClick){
			selectFile(id);
			setDir(id);
			window.setTimeout(function(){
				dtop.metaKeys.doubleClick = false;
			}, 400);
		}
	} else if(dtop.fileSelect.data.currentID == id && (!dtop.metaKeys.ctrl)){
		selectFile(id);
	} else if(dtop.fileSelect.options.multiple){
		if(dtop.metaKeys.shift){
			let
					oldid = dtop.fileSelect.data.currentID,
					currendPos = dtop.getPositionByID(id),
					firstSelected = dtop.getFirstSelected();

			if(currendPos > firstSelected){
				dtop.selectFilesFrom(firstSelected, currendPos);
			} else if(currendPos < firstSelected){
				dtop.selectFilesFrom(currendPos, firstSelected);
			} else {
				dtop.selectFile(id);
			}
			dtop.fileSelect.data.currentID = oldid;

		} else if(!dtop.metaKeys.ctrl){
			dtop.selectFile(id);
		} else if(dtop.isFileSelected(id)){
			dtop.unselectFile(id);
		} else {
			dtop.selectFile(id);
		}
	} else {
		selectFile(id);
	}
	if(dtop.metaKeys.ctrl){
		dtop.metaKeys.ctrl = 0;
	}
	if(dtop.metaKeys.shift){
		dtop.metaKeys.shift = 0;
	}
}

function setDir(id){
	let
			dtop = WE().util.dialogTop(window),
			e = dtop.getEntry(id);
	showPreview(id);
	dtop.fscmd.location.replace(dtop.queryString(WE().consts.selectors.SETDIR, id));
	dtop.document.getElementById('fspath').innerHTML = e.path;
}

function selectFile(id){
	let
			dtop = WE().util.dialogTop(window),
			a = dtop.document.getElementsByName("fname")[0];
	if(id){
		if(id === dtop.fileSelect.data.currentID){
			return;
		}

		showPreview(id);
		let e = dtop.getEntry(id);
		if(a.value != e.text &&
				a.value.indexOf(e.text + ",") === -1 &&
				a.value.indexOf("," + e.text + ",") === -1 &&
				a.value.indexOf("," + e.text + ",") === -1){

			a.value = a.value ?
					(a.value + "," + e.text) :
					e.text;
		}
		if(dtop.fsbody.document.getElementById("line_" + id)){
			dtop.fsbody.document.getElementById("line_" + id).classList.add("selected");
		}
		dtop.fileSelect.data.currentPath = e.path;
		dtop.fileSelect.data.currentID = id;
		dtop.fileSelect.data.currentType = e.contentType;
	} else {
		a.value = "";
		dtop.fileSelect.data.currentPath = "";
	}
}

function elementSelected(){
	return true;
}

function addEntry(ID, text, isFolder, path, modDate, contentType){
	entries.push({
		ID: ID,
		text: text,
		isFolder: isFolder,
		path: path,
		modDate: modDate,
		contentType: contentType
	});
}

function writeBodyDir(d, withModDate){
	let body = '',
			dtop = WE().util.dialogTop(window);
	for(let i = 0; i < entries.length; i++) {
		body += '<tr id="line_' + entries[i].ID + '" class="' + ((entries[i].ID == dtop.fileSelect.data.currentID) ? 'selected' : '') + '"  onclick="return selectorOnClick(event,' + entries[i].ID + ');"' + (entries[i].isFolder ? ' onDblClick="return selectorOnDblClick(' + entries[i].ID + ');"' : '') + '>' +
				'<td class="selector selectoricon">' + WE().util.getTreeIcon(entries[i].contentType, false) + '</td>' +
				'<td class="selector cutText directory" title="' + entries[i].text + '">' + entries[i].text + '</td>' +
				(withModDate ? '<td class="selector moddate">' + entries[i].modDate + '</td>' : '') +
				'<td class="remain"></td></tr>';
	}

	d.innerHTML = '<form name="we_form" target="fscmd" method="post" action="' + dtop.fileSelect.options.formtarget + '" >' +
			'<table class="selector sort" data-skipSort="0"><thead><tr>' + dtop.fileSelect.options.selectorHeader + '</tr></thead><tbody>' + body + '</table></form>';
	window.fsbody.$("table.sort").each(WE().layout.applyTableSort);
}

function writeBody(d){
	writeBodyDir(d, true);
}

function queryString(what, id){
	let dtop = WE().util.dialogTop(window);
	return dtop.fileSelect.options.formtarget +
			'&we_cmd[what]=' + what +
			'&we_cmd[rootDirID]=' + dtop.fileSelect.options.rootDirID +
			'&we_cmd[open_doc]=' + dtop.fileSelect.options.open_doc +
			'&we_cmd[table]=' + dtop.fileSelect.options.table +
			'&we_cmd[id]=' + id;
}

function weonclick(e){
	let top = WE().util.dialogTop(window);
	if(top.document.getElementById('fspreviewDiv')){
		top.document.getElementById('fspreviewDiv').innerHTML = "";
	}

	top.metaKeys.inputClick = false;
	if(e.altKey || e.metaKey || e.ctrlKey){
		top.metaKeys.ctrl = true;
	}
	if(e.shiftKey){
		top.metaKeys.shift = true;
	}
	if(top.fileSelect.options.multiple){
		if(!top.metaKeys.shift && !top.metaKeys.ctrl){
			top.unselectAllFiles();
		}
	} else {
		top.unselectAllFiles();
	}
}
